@@ -68,7 +68,7 @@ int main()
   // Define boundary condition
   auto u0 = std::make_shared<Constant>(0.0, 0.0);
   Noslip noslip;
-  auto noslip_markers = std::make_shared<FacetFunction<std::size_t>>(mesh, 1);
+  auto noslip_markers = std::make_shared<MeshFunction<std::size_t>>(mesh, mesh->topology().dim()-1, 1);
   noslip.mark(*noslip_markers, 0);
   auto W0 = W->sub(0);
   auto bc = std::make_shared<DirichletBC>(W0, u0, noslip_markers, 0);
@@ -90,7 +90,7 @@ int main()
   auto M = std::make_shared<AdaptiveNavierStokes::GoalFunctional>(mesh);
   M->w = w;
   Outflow outflow;
-  auto outflow_markers = std::make_shared<FacetFunction<std::size_t>>(mesh, 1);
+  auto outflow_markers = std::make_shared<MeshFunction<std::size_t>>(mesh, mesh->topology().dim()-1, 1);
   outflow.mark(*outflow_markers, 0);
   M->ds = outflow_markers;
 
@@ -122,11 +122,10 @@ int main()
   // Show all timings
   list_timings(TimingClear::clear, { TimingType::wall });
 
-  // Plot solutions
+  // Output solutions
   Function solution = w->leaf_node();
-  plot(solution[0], "Velocity on finest mesh");
-  plot(solution[1], "Pressure on finest mesh");
-  interactive();
+  XDMFFile("velocity.xdmf").write(solution[0]);
+  XDMFFile("pressure.xdmf").write(solution[1]);
 
   return 0;
 }
