@@ -22,6 +22,7 @@
 # First added:  2008-12-12
 # Last changed: 2012-11-12
 
+from __future__ import print_function
 from dolfin import *
 
 # Create a simple stiffness matrix and vector
@@ -31,7 +32,8 @@ u = TrialFunction(V)
 v = TestFunction(V)
 a = dot(grad(u), grad(v))*dx
 A = assemble(a)
-L = sin(V.cell().x[0])*v*dx
+x0 = SpatialCoordinate(mesh)[0]
+L = sin(x0)*v*dx
 x = assemble(L)
 
 # Create a block matrix
@@ -48,11 +50,11 @@ xx[1] = x
 
 # Create a another block vector (that is compatible with A in parallel)
 y = Vector()
-A.resize(y, 0)
+A.init_vector(y, 0)
 yy = BlockVector(2)
 yy[0] = y
 yy[1] = y
 
 # Multiply
 AA.mult(xx, yy)
-print "||Ax|| =", y.norm("l2")
+print("||Ax|| =", y.norm("l2"))
