@@ -27,6 +27,8 @@ upwards (and not downwards) in order to minimise the potential energy."""
 
 from __future__ import print_function
 from dolfin import *
+import matplotlib.pyplot as plt
+
 
 if not has_petsc():
     print("DOLFIN must be compiled at least with PETSc 3.6 to run this demo.")
@@ -72,7 +74,7 @@ class Left(SubDomain):
 class Right(SubDomain):
     def inside(self, x, on_boundary):
         return on_boundary and near(x[0], 10)
-boundaries = FacetFunction("size_t", mesh)
+boundaries = MeshFunction("size_t", mesh, mesh.topology().dim()-1)
 boundaries.set_all(0)
 left = Left()
 left.mark(boundaries, 1)
@@ -140,4 +142,4 @@ else:
 
 # Plot the current configuration
 plot(u, mode="displacement", wireframe=True, title="Displacement field")
-interactive()
+plt.show()
