@@ -17,10 +17,7 @@
 
 from dolfin import *
 
-# Sequence of plots eats a lot of memory with matplotlib (maybe leak)
-if parameters['plotting_backend'] == 'matplotlib':
-    warning("Collision detection demo does not work with matplotlib. Bye!")
-    exit(0)
+# There is an issue with the Matplotlib backend that needs to be investigated
 
 # Some parameters
 L = 10.0
@@ -43,7 +40,7 @@ mesh_B.translate(Point(x_B, y_B))
 mesh_C.translate(Point(x_C, y_C))
 
 # Create mesh function for plotting
-f = CellFunction("uint", mesh_A)
+f = MeshFunction("size_t", mesh_A, mesh_A.topology().dim())
 
 # Build bounding box trees for background mesh
 tree_A = BoundingBoxTree()
@@ -80,17 +77,8 @@ for n in range(num_steps):
     # Mark mesh function
     f.set_all(0)
     for i in entities_AB:
-        f.set_value(i, 1)
+        f[int(i)] = 1
     for i in entities_AC:
-        f.set_value(i, 2)
+        f[int(i)] = 2
     for i in entities_BC:
-        f.set_value(i, 3)
-
-    # Plot
-    p = plot(f, wireframe=(n > num_steps / 2))
-    #p.write_png("collision-detection-%.4d" % n)
-
-interactive()
-
-# Generate movie using
-# ffmpeg -r 25 -b 1800 -i collision-detection-%04d.png collision-detection.mp4
+        f[int(i)] = 3
