@@ -27,6 +27,8 @@ in a box of the same size."""
 
 from __future__ import print_function
 from dolfin import *
+import matplotlib.pyplot as plt
+
 
 if not has_petsc():
     print("DOLFIN must be compiled with PETSc to run this demo.")
@@ -139,4 +141,4 @@ else:
 
 # Plot the current configuration
 plot(u, mode="displacement", wireframe=True, title="Displacement field")
-interactive()
+plt.show()
