@@ -21,6 +21,8 @@ smoothed aggregation algerbaric multigrid."""
 
 from __future__ import print_function
 from dolfin import *
+import matplotlib.pyplot as plt
+
 
 # Test for PETSc
 if not has_linear_algebra_backend("PETSc"):
@@ -59,8 +61,9 @@ def build_nullspace(V, x):
     return basis
 
 
-# Load mesh and define function space
-mesh = Mesh("../pulley.xml.gz")
+# Load mesh from file
+mesh = Mesh()
+XDMFFile(mpi_comm_world(), "../pulley.xdmf").read(mesh)
 
 # Function to mark inner surface of pulley
 def inner_surface(x, on_boundary):
@@ -140,7 +143,7 @@ File("elasticity.pvd", "compressed") << u
 
 # Save colored mesh partitions in VTK format if running in parallel
 if MPI.size(mesh.mpi_comm()) > 1:
-    File("partitions.pvd") << CellFunction("size_t", mesh, \
+    File("partitions.pvd") << MeshFunction("size_t", mesh, mesh.topology().dim(), \
                                            MPI.rank(mesh.mpi_comm()))
 
 # Project and write stress field to post-processing file
@@ -149,4 +152,5 @@ stress = project(sigma(u), V=W)
 File("stress.pvd") << stress
 
 # Plot solution
-plot(u, interactive=True)
+plot(u)
+plt.show()
