@@ -20,7 +20,6 @@
 //
 // Demonstrating function evaluation at arbitrary points.
 
-#include <boost/assign/list_of.hpp>
 #include <dolfin.h>
 #include "Projection.h"
 
@@ -42,14 +41,14 @@ public:
 int main()
 {
   // Create mesh and a point in the mesh
-  UnitCubeMesh mesh(8, 8, 8);
+  auto mesh = std::make_shared<UnitCubeMesh>(8, 8, 8);
   Point x(0.31, 0.32, 0.33);
 
   // A user-defined function
-  F f;
+  auto f = std::make_shared<F>();
 
   // Project to a discrete function
-  Projection::FunctionSpace V(mesh);
+  auto V = std::make_shared<Projection::FunctionSpace>(mesh);
   Projection::BilinearForm a(V, V);
   Projection::LinearForm L(V);
   L.f = f;
@@ -57,7 +56,7 @@ int main()
   solve(a == L, g);
 
   // Evaluate user-defined function f
-  info("f(x) = %g", f(x));
+  info("f(x) = %g", (*f)(x));
 
   // Evaluate discrete function g (projection of f)
   info("g(x) = %g", g(x));
