@@ -42,37 +42,37 @@ int main()
   };
 
 
-  UnitSquareMesh mesh(5, 5);
+  auto mesh = std::make_shared<UnitSquareMesh>(5, 5);
 
   parameters["refinement_algorithm"] = "plaza_with_parent_facets";
 
   // Create MeshFunction over cells
-  MeshFunction<std::size_t> right_cells(mesh, mesh.topology().dim(), 0);
+  MeshFunction<std::size_t> right_cells(mesh, mesh->topology().dim(), 0);
   Right right;
   right.mark(right_cells, 1);
 
   // Create MeshFunction over facets
-  MeshFunction<std::size_t> inflow_facets(mesh, mesh.topology().dim() - 1, 0);
+  MeshFunction<std::size_t> inflow_facets(mesh, mesh->topology().dim() - 1, 0);
   Inflow inflow;
   inflow.mark(inflow_facets, 1);
 
   // Mark cells for refinement
-  MeshFunction<bool> cell_markers(mesh, mesh.topology().dim(), false);
+  MeshFunction<bool> cell_markers(mesh, mesh->topology().dim(), false);
   Point p(1.0, 0.0);
-  for (CellIterator c(mesh); !c.end(); ++c)
+  for (CellIterator c(*mesh); !c.end(); ++c)
   {
     if (c->midpoint().distance(p) < 0.5)
       cell_markers[*c] = true;
   }
 
   // Refine mesh
-  adapt(mesh, cell_markers);
+  adapt(*mesh, cell_markers);
 
   // Adapt cell function to refined mesh
-  adapt(right_cells, mesh.child_shared_ptr());
+  adapt(right_cells, mesh->child_shared_ptr());
 
   // Adapt facet function to refined mesh
-  adapt(inflow_facets, mesh.child_shared_ptr());
+  adapt(inflow_facets, mesh->child_shared_ptr());
 
   return 0;
 }
