@@ -24,6 +24,8 @@
 # NB! This demo is work in progress and will be updated as we get further in the multidomain features implementation.
 #
 from dolfin import *
+import matplotlib.pyplot as plt
+
 
 class Left(SubDomain):
     def inside(self, x, on_boundary):
@@ -39,7 +41,7 @@ class Right(SubDomain):
 
 # Define mesh and subdomains
 mesh = UnitSquareMesh(64, 64)
-domains = CellFunction("size_t", mesh)
+domains = MeshFunction("size_t", mesh, mesh.topology().dim())
 domains.set_all(0)
 Left().mark(domains, 1)
 Mid().mark(domains, 2)
@@ -70,6 +72,8 @@ u = Function(V)
 solve(a == L, u)
 
 # Plot solution and gradient
+plt.figure()
 plot(domains, title='domains')
+plt.figure()
 plot(u, title="u")
-interactive()
+plt.show()
