@@ -22,9 +22,12 @@
 
 from __future__ import print_function
 from dolfin import *
+import matplotlib.pyplot as plt
+
 
 # Create mesh of unit square
 mesh = UnitSquareMesh(8, 8)
+plt.figure(1)
 plot(mesh)
 
 info(mesh)
@@ -32,6 +35,7 @@ print()
 
 # Uniform refinement
 mesh = refine(mesh)
+plt.figure(2)
 plot(mesh)
 
 info(mesh)
@@ -39,6 +43,7 @@ print()
 
 # Uniform refinement
 mesh = refine(mesh)
+plt.figure(3)
 plot(mesh)
 
 info(mesh)
@@ -51,7 +56,7 @@ for i in range(5):
     print("marking for refinement")
 
     # Mark cells for refinement
-    cell_markers = CellFunction("bool", mesh)
+    cell_markers = MeshFunction("bool", mesh, mesh.topology().dim())
     for c in cells(mesh):
         if c.midpoint().distance(p) < 0.1:
             cell_markers[c] = True
@@ -62,6 +67,7 @@ for i in range(5):
     mesh = refine(mesh, cell_markers)
 
     # Plot mesh
+    plt.figure()
     plot(mesh)
 
-interactive()
+plt.show()
