@@ -34,15 +34,6 @@
 
 using namespace dolfin;
 
-// ----------------------------------------------------------------------------
-AdaptiveLinearVariationalSolver::
-AdaptiveLinearVariationalSolver(LinearVariationalProblem& problem,
-                                GoalFunctional& goal)
-  : _problem(reference_to_no_delete_pointer(problem))
-{
-  init(reference_to_no_delete_pointer(problem),
-       reference_to_no_delete_pointer(goal));
-}
 // ----------------------------------------------------------------------------
 AdaptiveLinearVariationalSolver::
 AdaptiveLinearVariationalSolver(std::shared_ptr<LinearVariationalProblem> problem,
@@ -96,13 +87,13 @@ std::shared_ptr<const Function>
 AdaptiveLinearVariationalSolver::solve_primal()
 {
   LinearVariationalProblem& current = _problem->leaf_node();
-  LinearVariationalSolver solver(current);
+  LinearVariationalSolver solver(reference_to_no_delete_pointer(current));
   solver.parameters.update(parameters("linear_variational_solver"));
   solver.solve();
   return current.solution();
 }
 // ----------------------------------------------------------------------------
-std::vector<std::shared_ptr<const DirichletBC> >
+std::vector<std::shared_ptr<const DirichletBC>>
 AdaptiveLinearVariationalSolver::extract_bcs() const
 {
   const LinearVariationalProblem& current = _problem->leaf_node();
