@@ -44,21 +44,28 @@ BoxMesh::BoxMesh(const Point& p0, const Point& p1,
 BoxMesh::BoxMesh(MPI_Comm comm, const Point& p0, const Point& p1,
                  std::size_t nx, std::size_t ny, std::size_t nz) : Mesh(comm)
 {
-  build(p0, p1, nx, ny, nz);
+  build_tet(*this, {{p0, p1}}, {{nx, ny, nz}});
 }
 //-----------------------------------------------------------------------------
-void BoxMesh::build(const Point& p0, const Point& p1,
-                    std::size_t nx, std::size_t ny, std::size_t nz)
+void BoxMesh::build_tet(Mesh& mesh, const std::array<Point,2 >& p,
+                        std::array<std::size_t, 3> n)
 {
   Timer timer("Build BoxMesh");
 
   // Receive mesh according to parallel policy
-  if (MPI::is_receiver(this->mpi_comm()))
+  if (MPI::is_receiver(mesh.mpi_comm()))
   {
-    MeshPartitioning::build_distributed_mesh(*this);
+    MeshPartitioning::build_distributed_mesh(mesh);
     return;
   }
 
+  // Extract data
+  const Point& p0 = p[0];
+  const Point& p1 = p[1];
+  std::size_t nx = n[0];
+  std::size_t ny = n[1];
+  std::size_t nz = n[2];
+
   // Extract minimum and maximum coordinates
   const double x0 = std::min(p0.x(), p1.x());
   const double x1 = std::max(p0.x(), p1.x());
@@ -89,11 +96,11 @@ void BoxMesh::build(const Point& p0, const Point& p1,
                  "BoxMesh has non-positive number of vertices in some dimension: number of vertices must be at least 1 in each dimension");
   }
 
-  rename("mesh", "Mesh of the cuboid (a,b) x (c,d) x (e,f)");
+  mesh.rename("mesh", "Mesh of the cuboid (a,b) x (c,d) x (e,f)");
 
   // Open mesh for editing
   MeshEditor editor;
-  editor.open(*this, CellType::tetrahedron, 3, 3);
+  editor.open(mesh, CellType::Type::tetrahedron, 3, 3);
 
   // Storage for vertex coordinates
   std::vector<double> x(3);
@@ -155,9 +162,91 @@ void BoxMesh::build(const Point& p0, const Point& p1,
   editor.close();
 
   // Broadcast mesh according to parallel policy
-  if (MPI::is_broadcaster(this->mpi_comm()))
+  if (MPI::is_broadcaster(mesh.mpi_comm()))
   {
-    MeshPartitioning::build_distributed_mesh(*this);
+    MeshPartitioning::build_distributed_mesh(mesh);
+    return;
+  }
+}
+//-----------------------------------------------------------------------------
+void BoxMesh::build_hex(Mesh& mesh, std::array<std::size_t, 3> n)
+{
+  // Receive mesh according to parallel policy
+  if (MPI::is_receiver(mesh.mpi_comm()))
+  {
+    MeshPartitioning::build_distributed_mesh(mesh);
+    return;
+  }
+
+  const std::size_t nx = n[0];
+  const std::size_t ny = n[1];
+  const std::size_t nz = n[2];
+
+  MeshEditor editor;
+  editor.open(mesh, CellType::Type::hexahedron, 3, 3);
+
+  // Create vertices and cells:
+  editor.init_vertices_global((nx + 1)*(ny + 1)*(nz + 1),
+                              (nx + 1)*(ny + 1)*(nz + 1));
+  editor.init_cells_global(nx*ny*nz, nx*ny*nz);
+
+  // Storage for vertices
+  std::vector<double> x(3);
+
+  const double a = 0.0;
+  const double b = 1.0;
+  const double c = 0.0;
+  const double d = 1.0;
+  const double e = 0.0;
+  const double f = 1.0;
+
+  // Create main vertices:
+  std::size_t vertex = 0;
+  for (std::size_t iz = 0; iz <= nz; iz++)
+  {
+    x[2] = e + ((static_cast<double>(iz))*(f - e)/static_cast<double>(nz));
+    for (std::size_t iy = 0; iy <= ny; iy++)
+    {
+      x[1] = c + ((static_cast<double>(iy))*(d - c)/static_cast<double>(ny));
+      for (std::size_t ix = 0; ix <= nx; ix++)
+      {
+        x[0] = a + ((static_cast<double>(ix))*(b - a)/static_cast<double>(nx));
+        editor.add_vertex(vertex, x);
+        vertex++;
+      }
+    }
+  }
+
+  // Create cuboids
+  std::size_t cell = 0;
+  std::vector<std::size_t> v(8);
+  for (std::size_t iz = 0; iz < nz; iz++)
+  {
+    for (std::size_t iy = 0; iy < ny; iy++)
+    {
+      for (std::size_t ix = 0; ix < nx; ix++)
+      {
+        v[0] = (iz*(ny + 1) + iy)*(nx + 1) + ix;
+        v[1] = v[0] + 1;
+        v[2] = v[0] + (nx + 1);
+        v[3] = v[1] + (nx + 1);
+        v[4] = v[0] + (nx + 1)*(ny + 1);
+        v[5] = v[1] + (nx + 1)*(ny + 1);
+        v[6] = v[2] + (nx + 1)*(ny + 1);
+        v[7] = v[3] + (nx + 1)*(ny + 1);
+        editor.add_cell(cell, v);
+        ++cell;
+      }
+    }
+  }
+
+  // Close mesh editor
+  editor.close();
+
+  // Broadcast mesh according to parallel policy
+  if (MPI::is_broadcaster(mesh.mpi_comm()))
+  {
+    MeshPartitioning::build_distributed_mesh(mesh);
     return;
   }
 }
