@@ -1,4 +1,4 @@
-// Copyright (C) 2005-2015 Anders Logg
+// Copyright (C) 2005-2017 Anders Logg and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -14,17 +14,13 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// Modified by Nuno Lopes, 2008.
-// Modified by Mikael Mortensen, 2014.
-//
-// First added:  2005-12-02
-// Last changed: 2015-06-15
 
 #ifndef __BOX_H
 #define __BOX_H
 
+#include <array>
 #include <cstddef>
+#include <dolfin/log/log.h>
 #include <dolfin/common/MPI.h>
 #include <dolfin/mesh/Mesh.h>
 
@@ -32,10 +28,9 @@ namespace dolfin
 {
 
   /// Tetrahedral mesh of the 3D rectangular prism spanned by two
-  /// points p0 and p1. Given the number of cells (nx, ny, nz) in
-  /// each direction, the total number of tetrahedra will be
-  /// 6*nx*ny*nz and the total number of vertices will be
-  /// (nx + 1)*(ny + 1)*(nz + 1).
+  /// points p0 and p1. Given the number of cells (nx, ny, nz) in each
+  /// direction, the total number of tetrahedra will be 6*nx*ny*nz and
+  /// the total number of vertices will be (nx + 1)*(ny + 1)*(nz + 1).
 
   class BoxMesh : public Mesh
   {
@@ -46,67 +41,125 @@ namespace dolfin
     /// two points is not important in terms of minimum and maximum
     /// coordinates.
     ///
-    /// *Arguments*
-    ///     p0 (_Point_)
-    ///         First point.
-    ///     p1 (_Point_)
-    ///         Second point.
-    ///     nx (double)
-    ///         Number of cells in :math:`x`-direction.
-    ///     ny (double)
-    ///         Number of cells in :math:`y`-direction.
-    ///     nz (double)
-    ///         Number of cells in :math:`z`-direction.
-    ///
-    /// *Example*
-    ///     .. code-block:: c++
+    /// @param p (std::array<_Point_, 2>)
+    ///         Points of box.
+    /// @param n (std::array<double, 3> )
+    ///         Number of cells in each direction.
+    /// @param cell_type
+    ///         Tetrahedron or Hexahedron
     ///
+    /// @code{.cpp}
     ///         // Mesh with 8 cells in each direction on the
     ///         // set [-1,2] x [-1,2] x [-1,2].
     ///         Point p0(-1, -1, -1);
     ///         Point p1(2, 2, 2);
-    ///         BoxMesh mesh(p0, p1, 8, 8, 8);
-    ///
-    BoxMesh(const Point& p0, const Point& p1,
-            std::size_t nx, std::size_t ny, std::size_t nz);
+    ///         auto mesh = BoxMesh::create({p0, p1}, {8, 8, 8});
+    /// @endcode
+    static Mesh create(const std::array<Point,2 >& p, std::array<std::size_t, 3> n, CellType::Type cell_type)
+    { return create(MPI_COMM_WORLD, p, n, cell_type); }
 
     /// Create a uniform finite element _Mesh_ over the rectangular
     /// prism spanned by the two _Point_s p0 and p1. The order of the
     /// two points is not important in terms of minimum and maximum
     /// coordinates.
     ///
-    /// *Arguments*
-    ///     comm (MPI_Comm)
+    /// @param comm (MPI_Comm)
     ///         MPI communicator
-    ///     p0 (_Point_)
+    /// @param p (std::array<_Point_, 2>)
+    ///         Points of box.
+    /// @param n (std::array<double, 3> )
+    ///         Number of cells in each direction.
+    /// @param cell_type
+    ///         Tetrahedron or hexahedron
+    ///
+    /// @code{.cpp}
+    ///         // Mesh with 8 cells in each direction on the
+    ///         // set [-1,2] x [-1,2] x [-1,2].
+    ///         Point p0(-1, -1, -1);
+    ///         Point p1(2, 2, 2);
+    ///         auto mesh = BoxMesh::create({p0, p1}, {8, 8, 8});
+    /// @endcode
+    static Mesh create(MPI_Comm comm, const std::array<Point, 2>& p,
+                       std::array<std::size_t, 3> n, CellType::Type cell_type)
+    {
+      Mesh mesh(comm);
+      if (cell_type == CellType::Type::tetrahedron)
+        build_tet(mesh, p, n);
+      else if (cell_type == CellType::Type::hexahedron)
+        build_hex(mesh, n);
+      else
+      {
+        dolfin_error("BoxMesh.h",
+                     "generate box mesh",
+                     "Wrong cell type '%d'", cell_type);
+      }
+
+      return mesh;
+    }
+
+    /// Deprecated
+    /// Create a uniform finite element _Mesh_ over the rectangular
+    /// prism spanned by the two _Point_s p0 and p1. The order of the
+    /// two points is not important in terms of minimum and maximum
+    /// coordinates.
+    ///
+    /// @param p0 (_Point_)
     ///         First point.
-    ///     p1 (_Point_)
+    /// @param p1 (_Point_)
     ///         Second point.
-    ///     nx (double)
-    ///         Number of cells in :math:`x`-direction.
-    ///     ny (double)
-    ///         Number of cells in :math:`y`-direction.
-    ///     nz (double)
-    ///         Number of cells in :math:`z`-direction.
+    /// @param nx (double)
+    ///         Number of cells in x-direction.
+    /// @param ny (double)
+    ///         Number of cells in y-direction.
+    /// @param nz (double)
+    ///         Number of cells in z-direction.
     ///
-    /// *Example*
-    ///     .. code-block:: c++
+    /// @code{.cpp}
+    ///         // Mesh with 8 cells in each direction on the
+    ///         // set [-1,2] x [-1,2] x [-1,2].
+    ///         Point p0(-1, -1, -1);
+    ///         Point p1(2, 2, 2);
+    ///         BoxMesh mesh(p0, p1, 8, 8, 8);
+    /// @endcode
+    BoxMesh(const Point& p0, const Point& p1,
+            std::size_t nx, std::size_t ny, std::size_t nz);
+
+    /// Deprecated
+    /// Create a uniform finite element _Mesh_ over the rectangular
+    /// prism spanned by the two _Point_s p0 and p1. The order of the
+    /// two points is not important in terms of minimum and maximum
+    /// coordinates.
     ///
+    /// @param comm (MPI_Comm)
+    ///         MPI communicator
+    /// @param p0 (_Point_)
+    ///         First point.
+    /// @param p1 (_Point_)
+    ///         Second point.
+    /// @param nx (double)
+    ///         Number of cells in x-direction.
+    /// @param ny (double)
+    ///         Number of cells in y-direction.
+    /// @param nz (double)
+    ///         Number of cells in z-direction.
+    ///
+    /// @code{.cpp}
     ///         // Mesh with 8 cells in each direction on the
     ///         // set [-1,2] x [-1,2] x [-1,2].
     ///         Point p0(-1, -1, -1);
     ///         Point p1(2, 2, 2);
     ///         BoxMesh mesh(MPI_COMM_WORLD, p0, p1, 8, 8, 8);
-    ///
-    BoxMesh(MPI_Comm comm,
-            const Point& p0, const Point& p1,
+    /// @endcode
+    BoxMesh(MPI_Comm comm, const Point& p0, const Point& p1,
             std::size_t nx, std::size_t ny, std::size_t nz);
 
   private:
 
     // Build mesh
-    void build(const Point& p0, const Point& p1,
-               std::size_t nx, std::size_t ny, std::size_t nz);
+    static void build_tet(Mesh& mesh, const std::array<Point, 2>& p,
+                          std::array<std::size_t, 3> n);
+
+    static void build_hex(Mesh& mesh, std::array<std::size_t, 3> n);
 
   };
 
