@@ -14,16 +14,11 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// Modified by N. Lopes, 2008.
-// Modified by Mikael Mortensen, 2014.
-//
-// First added:  2007-11-23
-// Last changed: 2014-02-17
 
 #ifndef __INTERVAL_MESH_H
 #define __INTERVAL_MESH_H
 
+#include <array>
 #include <cstddef>
 #include <dolfin/common/MPI.h>
 #include <dolfin/mesh/Mesh.h>
@@ -32,55 +27,87 @@ namespace dolfin
 {
 
   /// Interval mesh of the 1D line [a,b].  Given the number of cells
-  /// (nx) in the axial direction, the total number of intervals will
-  /// be nx and the total number of vertices will be (nx + 1).
+  /// (n) in the axial direction, the total number of intervals will
+  /// be n and the total number of vertices will be (n + 1).
 
   class IntervalMesh : public Mesh
   {
   public:
 
-    /// Constructor
+    /// Factory
     ///
-    /// *Arguments*
-    ///     nx (std::size_t)
+    /// @param    n (std::size_t)
     ///         The number of cells.
-    ///     a (double)
-    ///         The minimum point (inclusive).
-    ///     b (double)
-    ///         The maximum point (inclusive).
+    /// @param    x (std::array<double, 2>)
+    ///         The end points
     ///
-    /// *Example*
-    ///     .. code-block:: c++
+    /// @code{.cpp}
     ///
     ///         // Create a mesh of 25 cells in the interval [-1,1]
-    ///         IntervalMesh mesh(25, -1.0, 1.0);
-    ///
-    IntervalMesh(std::size_t nx, double a, double b);
+    ///         auto mesh = IntervalMesh::create(25, {-1.0, 1.0});
+    /// @endcode
+    static Mesh create(std::size_t n, std::array<double, 2> x)
+    { return create(MPI_COMM_WORLD, n, x); }
 
-    /// Constructor
+    /// Factory
     ///
-    /// *Arguments*
-    ///     comm (MPI_Comm)
+    /// @param    comm (MPI_Comm)
     ///         MPI communicator
-    ///     nx (std::size_t)
+    /// @param    n (std::size_t)
     ///         The number of cells.
-    ///     a (double)
-    ///         The minimum point (inclusive).
-    ///     b (double)
-    ///         The maximum point (inclusive).
+    /// @param    x (std::array<double, 2>)
+    ///         The end points
     ///
-    /// *Example*
-    ///     .. code-block:: c++
+    /// @code{.cpp}
     ///
     ///         // Create a mesh of 25 cells in the interval [-1,1]
     ///         IntervalMesh mesh(MPI_COMM_WORLD, 25, -1.0, 1.0);
+    /// @endcode
+    static Mesh create(MPI_Comm comm, std::size_t n, std::array<double, 2> x)
+    {
+      Mesh mesh;
+      build(mesh, n, x);
+      return mesh;
+    }
+
+    /// Constructor
     ///
-    IntervalMesh(MPI_Comm comm, std::size_t nx, double a, double b);
+    /// @param    n (std::size_t)
+    ///         The number of cells.
+    /// @param    a (double)
+    ///         The minimum point (inclusive).
+    /// @param    b (double)
+    ///         The maximum point (inclusive).
+    ///
+    /// @code{.cpp}
+    ///
+    ///         // Create a mesh of 25 cells in the interval [-1,1]
+    ///         IntervalMesh mesh(25, -1.0, 1.0);
+    /// @endcode
+    IntervalMesh(std::size_t n, double a, double b);
+
+    /// Constructor
+    ///
+    /// @param    comm (MPI_Comm)
+    ///         MPI communicator
+    /// @param    n (std::size_t)
+    ///         The number of cells.
+    /// @param    a (double)
+    ///         The minimum point (inclusive).
+    /// @param    b (double)
+    ///         The maximum point (inclusive).
+    ///
+    /// @code{.cpp}
+    ///
+    ///         // Create a mesh of 25 cells in the interval [-1,1]
+    ///         IntervalMesh mesh(MPI_COMM_WORLD, 25, -1.0, 1.0);
+    /// @endcode
+    IntervalMesh(MPI_Comm comm, std::size_t n, double a, double b);
 
   private:
 
     // Build mesh
-    void build(std::size_t nx, double a, double b);
+    static void build(Mesh& mesh, std::size_t n, std::array<double, 2> x);
 
   };
 
