@@ -14,10 +14,6 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// Modified by Garth N. Wells 2007
-// Modified by Nuno Lopes 2008
-// Modified by Kristian B. Oelgaard 2009
 
 #include <cmath>
 #include <boost/multi_array.hpp>
@@ -44,17 +40,17 @@ RectangleMesh::RectangleMesh(MPI_Comm comm,
                              std::size_t nx, std::size_t ny,
                              std::string diagonal) : Mesh(comm)
 {
-  build(p0, p1, nx, ny, diagonal);
+  build_tri(*this, {{p0, p1}}, {{nx, ny}}, diagonal);
 }
 //-----------------------------------------------------------------------------
-void RectangleMesh::build(const Point& p0, const Point& p1,
-                          std::size_t nx, std::size_t ny,
-                          std::string diagonal)
+void RectangleMesh::build_tri(Mesh& mesh, const std::array<Point, 2>& p,
+                              std::array<std::size_t, 2> n,
+                              std::string diagonal)
 {
   // Receive mesh according to parallel policy
-  if (MPI::is_receiver(this->mpi_comm()))
+  if (MPI::is_receiver(mesh.mpi_comm()))
   {
-    MeshPartitioning::build_distributed_mesh(*this);
+    MeshPartitioning::build_distributed_mesh(mesh);
     return;
   }
 
@@ -67,6 +63,12 @@ void RectangleMesh::build(const Point& p0, const Point& p1,
                  "Unknown mesh diagonal definition: allowed options are \"left\", \"right\", \"left/right\", \"right/left\" and \"crossed\"");
   }
 
+  const Point& p0 = p[0];
+  const Point& p1 = p[1];
+
+  const std::size_t nx = n[0];
+  const std::size_t ny = n[1];
+
   // Extract minimum and maximum coordinates
   const double x0 = std::min(p0.x(), p1.x());
   const double x1 = std::max(p0.x(), p1.x());
@@ -92,10 +94,11 @@ void RectangleMesh::build(const Point& p0, const Point& p1,
                  "Rectangle has non-positive number of vertices in some dimension: number of vertices must be at least 1 in each dimension");
   }
 
-  rename("mesh", "Mesh of the unit square (a,b) x (c,d)");
+  mesh.rename("mesh", "Mesh of the unit square (a,b) x (c,d)");
+
   // Open mesh for editing
   MeshEditor editor;
-  editor.open(*this, CellType::triangle, 2, 2);
+  editor.open(mesh, CellType::Type::triangle, 2, 2);
 
   // Create vertices and cells:
   if (diagonal == "crossed")
@@ -222,10 +225,77 @@ void RectangleMesh::build(const Point& p0, const Point& p1,
   editor.close();
 
   // Broadcast mesh according to parallel policy
-  if (MPI::is_broadcaster(this->mpi_comm()))
+  if (MPI::is_broadcaster(mesh.mpi_comm()))
   {
-    MeshPartitioning::build_distributed_mesh(*this);
+    MeshPartitioning::build_distributed_mesh(mesh);
     return;
   }
 }
 //-----------------------------------------------------------------------------
+void RectangleMesh::build_quad(Mesh& mesh, const std::array<Point, 2>& p,
+                               std::array<std::size_t, 2> n)
+{
+  // Receive mesh according to parallel policy
+  if (MPI::is_receiver(mesh.mpi_comm()))
+  {
+    MeshPartitioning::build_distributed_mesh(mesh);
+    return;
+  }
+
+  const std::size_t nx = n[0];
+  const std::size_t ny = n[1];
+
+  MeshEditor editor;
+  editor.open(mesh, CellType::Type::quadrilateral, 2, 2);
+
+  // Create vertices and cells:
+  editor.init_vertices_global((nx + 1)*(ny + 1), (nx + 1)*(ny + 1));
+  editor.init_cells_global(nx*ny, nx*ny);
+
+  // Storage for vertices
+  std::vector<double> x(2);
+
+  const double a = 0.0;
+  const double b = 1.0;
+  const double c = 0.0;
+  const double d = 1.0;
+
+  // Create main vertices:
+  std::size_t vertex = 0;
+  for (std::size_t iy = 0; iy <= ny; iy++)
+  {
+    x[1] = c + ((static_cast<double>(iy))*(d - c)/static_cast<double>(ny));
+    for (std::size_t ix = 0; ix <= nx; ix++)
+    {
+      x[0] = a + ((static_cast<double>(ix))*(b - a)/static_cast<double>(nx));
+      editor.add_vertex(vertex, x);
+      vertex++;
+    }
+  }
+
+  // Create rectangles
+  std::size_t cell = 0;
+  std::vector<std::size_t> v(4);
+  for (std::size_t iy = 0; iy < ny; iy++)
+    for (std::size_t ix = 0; ix < nx; ix++)
+    {
+      v[0] = iy*(nx + 1) + ix;
+      v[1] = v[0] + 1;
+      v[2] = v[0] + (nx + 1);
+      v[3] = v[1] + (nx + 1);
+      editor.add_cell(cell, v);
+      ++cell;
+    }
+
+  // Close mesh editor
+  editor.close();
+
+  // Broadcast mesh according to parallel policy
+  if (MPI::is_broadcaster(mesh.mpi_comm()))
+  {
+    MeshPartitioning::build_distributed_mesh(mesh);
+    return;
+  }
+
+}
+//-----------------------------------------------------------------------------
