@@ -14,15 +14,6 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// Modified by Anders Logg 2005-2011
-// Modified by Kristian Oelgaard 2006
-// Modified by Martin Alnes 2008
-// Modified by Niclas Jansson 2009
-// Modified by Johannes Ring 2012
-//
-// First added:  2005-07-05
-// Last changed: 2013-03-11
 
 #include <ostream>
 #include <sstream>
@@ -91,71 +82,64 @@ VTKFile::~VTKFile()
   // Do nothing
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const Mesh& mesh)
+void VTKFile::write(const Mesh& mesh)
 {
   write_mesh(mesh, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const MeshFunction<bool>& meshfunction)
+void VTKFile::write(const MeshFunction<bool>& meshfunction)
 {
   mesh_function_write(meshfunction, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const MeshFunction<std::size_t>& meshfunction)
+void VTKFile::write(const MeshFunction<std::size_t>& meshfunction)
 {
   mesh_function_write(meshfunction, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const MeshFunction<int>& meshfunction)
+void VTKFile::write(const MeshFunction<int>& meshfunction)
 {
   mesh_function_write(meshfunction, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const MeshFunction<double>& meshfunction)
+void VTKFile::write(const MeshFunction<double>& meshfunction)
 {
   mesh_function_write(meshfunction, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const Function& u)
+void VTKFile::write(const Function& u)
 {
   write_function(u, counter);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const Mesh*, double> mesh)
+void VTKFile::write(const Mesh& mesh, double time)
 {
-  dolfin_assert(mesh.first);
-  write_mesh(*(mesh.first), mesh.second);
+  write_mesh(mesh, time);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const MeshFunction<int>*, double> f)
+void VTKFile::write(const MeshFunction<int>& mf, double time)
 {
-  dolfin_assert(f.first);
-  mesh_function_write(*(f.first), f.second);
+  mesh_function_write(mf, time);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const MeshFunction<std::size_t>*,
-                                         double> f)
+void VTKFile::write(const MeshFunction<std::size_t>& mf, double time)
 {
-  dolfin_assert(f.first);
-  mesh_function_write(*(f.first), f.second);
+  mesh_function_write(mf, time);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const MeshFunction<double>*, double> f)
+void VTKFile::write(const MeshFunction<double>& mf, double time)
 {
-  dolfin_assert(f.first);
-  mesh_function_write(*(f.first), f.second);
+  mesh_function_write(mf, time);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const MeshFunction<bool>*, double> f)
+void VTKFile::write(const MeshFunction<bool>& mf, double time)
 {
-  dolfin_assert(f.first);
-  mesh_function_write(*(f.first), f.second);
+  mesh_function_write(mf, time);
 }
 //----------------------------------------------------------------------------
-void VTKFile::operator<<(const std::pair<const Function*, double> u)
+void VTKFile::write(const Function& u, double time)
 {
-  dolfin_assert(u.first);
-  write_function(*(u.first), u.second);
+  write_function(u, time);
 }
 //----------------------------------------------------------------------------
 void VTKFile::write_function(const Function& u, double time)
@@ -237,11 +221,12 @@ std::string VTKFile::init(const Mesh& mesh, std::size_t cell_dim) const
                                       counter, ".vtu");
   clear_file(vtu_filename);
 
-  // Number of cells
-  const std::size_t num_cells = mesh.topology().size(cell_dim);
+  // Number of cells and vertices
+  const std::size_t num_cells = mesh.topology().ghost_offset(cell_dim);
+  const std::size_t num_vertices = mesh.topology().ghost_offset(0);
 
   // Write headers
-  vtk_header_open(mesh.num_vertices(), num_cells, vtu_filename);
+  vtk_header_open(num_vertices, num_cells, vtu_filename);
 
   return vtu_filename;
 }
@@ -298,7 +283,7 @@ void VTKFile::results_write(const Function& u, std::string vtu_filename) const
 
   dolfin_assert(u.function_space()->dofmap());
   const GenericDofMap& dofmap= *u.function_space()->dofmap();
-  if (dofmap.max_cell_dimension() == cell_based_dim)
+  if (dofmap.max_element_dofs() == cell_based_dim)
     VTKWriter::write_cell_data(u, vtu_filename, binary, compress);
   else
     write_point_data(u, mesh, vtu_filename);
@@ -321,30 +306,27 @@ void VTKFile::write_point_data(const GenericFunction& u, const Mesh& mesh,
   const std::size_t size = num_vertices*dim;
   std::vector<double> values(size);
 
-  // Get function values at vertices and zero any small values
+  // Get function values at vertices
   u.compute_vertex_values(values, mesh);
   dolfin_assert(values.size() == size);
-  std::vector<double>::iterator it;
-  for (it = values.begin(); it != values.end(); ++it)
-  {
-    if (std::abs(*it) < DOLFIN_EPS)
-      *it = 0.0;
-  }
 
   if (rank == 0)
   {
     fp << "<PointData  Scalars=\"" << u.name() << "\"> " << std::endl;
-    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name() << "\"  format=\""<< encode_string <<"\">";
+    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name()
+       << "\"  format=\""<< encode_string <<"\">";
   }
   else if (rank == 1)
   {
     fp << "<PointData  Vectors=\"" << u.name() << "\"> " << std::endl;
-    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name() << "\"  NumberOfComponents=\"3\" format=\""<< encode_string <<"\">";
+    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name()
+       << "\"  NumberOfComponents=\"3\" format=\""<< encode_string <<"\">";
   }
   else if (rank == 2)
   {
     fp << "<PointData  Tensors=\"" << u.name() << "\"> " << std::endl;
-    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name() << "\"  NumberOfComponents=\"9\" format=\""<< encode_string <<"\">";
+    fp << "<DataArray  type=\"Float64\"  Name=\"" << u.name()
+       << "\"  NumberOfComponents=\"9\" format=\""<< encode_string <<"\">";
   }
 
   if (_encoding == "ascii")
@@ -608,7 +590,7 @@ void VTKFile::pvtu_write(const Function& u, const std::string fname) const
   dolfin_assert(u.function_space()->dofmap());
   for (std::size_t i = 0; i < rank; i++)
     cell_based_dim *= mesh.topology().dim();
-  if (u.function_space()->dofmap()->max_cell_dimension() == cell_based_dim)
+  if (u.function_space()->dofmap()->max_element_dofs() == cell_based_dim)
     data_type = "cell";
 
   const std::size_t num_processes = MPI::size(mesh.mpi_comm());
