@@ -34,6 +34,7 @@ typedef int hid_t;
 
 #include <dolfin/common/MPI.h>
 #include <dolfin/common/Variable.h>
+#include <dolfin/mesh/CellType.h>
 
 namespace boost
 {
@@ -62,7 +63,8 @@ namespace dolfin
   template<typename T> class MeshFunction;
   template<typename T> class MeshValueCollection;
   class Point;
-  class XDMFxml;
+
+  /// Read and write Mesh, Function, MeshFunction and other objects in XDMF
 
   /// This class supports the output of meshes and functions in XDMF
   /// (http://www.xdmf.org) format. It creates an XML file that
@@ -79,6 +81,13 @@ namespace dolfin
     /// File encoding type
     enum class Encoding {HDF5, ASCII};
 
+    /// Default encoding type
+#ifdef HAS_HDF5
+    static const Encoding default_encoding = Encoding::HDF5;
+#else
+    static const Encoding default_encoding = Encoding::ASCII;
+#endif
+
     /// Constructor
     XDMFFile(const std::string filename)
       : XDMFFile(MPI_COMM_WORLD, filename) {}
@@ -89,173 +98,290 @@ namespace dolfin
     /// Destructor
     ~XDMFFile();
 
+    /// Close the file
+    ///
+    /// This closes any open HDF5 files. In ASCII mode the XML file is
+    /// closed each time it is written to or read from, so close() has
+    /// no effect.
+    ///
+    /// From Python you can also use XDMFFile as a context manager:
+    ///
+    ///     with XDMFFile(mpi_comm_world(), 'name.xdmf') as xdmf:
+    ///         xdmf.write(mesh)
+    ///
+    /// The file is automatically closed at the end of the with block
+    void close();
+
     /// Save a mesh to XDMF format, either using an associated HDF5
     /// file, or storing the data inline as XML Create function on
     /// given function space
     ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
+    /// @param    mesh (_Mesh_)
     ///         A mesh to save.
-    ///     encoding (_Encoding_)
+    /// @param    encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
-    void write(const Mesh& mesh, Encoding encoding=Encoding::HDF5);
+    void write(const Mesh& mesh, Encoding encoding=default_encoding);
+
+    /// Save a Function to XDMF file for checkpointing, using an
+    /// associated HDF5 file, or storing the data inline as XML.
+    ///
+    /// If the file where we would like to write exists, then
+    /// the function is appended to the file.
+    ///
+    /// @param    u (_Function_)
+    ///         A function to save.
+    /// @param    function_name (string)
+    ///         Name of the function used in XDMF XML and HDF file
+    ///         (if encoding = HDF). The string is used to fill value for XML
+    ///         attribute Name of Grid node. It is also used in HDF file in
+    ///         path to datasets. Must be the same on all processes in parallel.
+    /// @param    time_step (_double_)
+    ///         Time step. It is saved only in XDMF file. The function could
+    ///         be saved with the same time step several times. There is an
+    ///         internal "counter" value stored in XDMF which differentiates
+    ///         the same time steps.
+    /// @param    encoding (_Encoding_)
+    ///         Encoding to use: HDF5 or ASCII
+    ///
+    void write_checkpoint(const Function& u,
+                          std::string function_name,
+                          double time_step = 0.0,
+                          Encoding encoding=default_encoding);
 
     /// Save a Function to XDMF file for visualisation, using an
     /// associated HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     u (_Function_)
+    /// @param    u (_Function_)
     ///         A function to save.
-    ///     encoding (_Encoding_)
+    /// @param    encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
-    void write(const Function& u, Encoding encoding=Encoding::HDF5);
+    void write(const Function& u, Encoding encoding=default_encoding);
 
     /// Save a Function with timestamp to XDMF file for visualisation,
     /// using an associated HDF5 file, or storing the data inline as
     /// XML.
     ///
-    /// *Arguments*
-    ///     u (_Function_)
+    /// You can control the output with the following boolean
+    /// parameters on the XDMFFile class:
+    ///
+    /// * rewrite_function_mesh (default true):
+    ///   Controls whether the mesh will be rewritten every timestep.
+    ///   If the mesh does not change this can be turned off to create
+    ///   smaller files.
+    ///
+    /// * functions_share_mesh (default false):
+    ///   Controls whether all functions on a single time step share
+    ///   the same mesh. If true the files created will be smaller and
+    ///   also behave better in Paraview, at least in version 5.3.0
+    ///
+    /// @param    u (_Function_)
     ///         A function to save.
-    ///     t (_double_)
+    /// @param    t (_double_)
     ///         Timestep
-    ///     encoding (_Encoding_)
+    /// @param   encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
-    void write(const Function& u, double t, Encoding encoding=Encoding::HDF5);
+    void write(const Function& u, double t,
+               Encoding encoding=default_encoding);
 
     /// Save MeshFunction to file using an associated HDF5 file, or
     /// storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     meshfunction (_MeshFunction_)
+    /// @param    meshfunction (_MeshFunction_)
     ///         A meshfunction to save.
-    ///     encoding (_Encoding_)
+    /// @param    encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const MeshFunction<bool>& meshfunction,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
+
+    /// Save MeshFunction to file using an associated HDF5 file, or
+    /// storing the data inline as XML.
+    ///
+    /// @param    meshfunction (_MeshFunction_)
+    ///         A meshfunction to save.
+    /// @param    encoding (_Encoding_)
+    ///         Encoding to use: HDF5 or ASCII
+    ///
     void write(const MeshFunction<int>& meshfunction,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
+
+    /// Save MeshFunction to file using an associated HDF5 file, or
+    /// storing the data inline as XML.
+    ///
+    /// @param    meshfunction (_MeshFunction_)
+    ///         A meshfunction to save.
+    /// @param    encoding (_Encoding_)
+    ///         Encoding to use: HDF5 or ASCII
+    ///
     void write(const MeshFunction<std::size_t>& meshfunction,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
+
+    /// Save MeshFunction to file using an associated HDF5 file, or
+    /// storing the data inline as XML.
+    ///
+    /// @param    meshfunction (_MeshFunction_)
+    ///         A meshfunction to save.
+    /// @param    encoding (_Encoding_)
+    ///         Encoding to use: HDF5 or ASCII
+    ///
     void write(const MeshFunction<double>& meshfunction,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Write out mesh value collection (subset) using an associated
     /// HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     mvc (_MeshValueCollection<bool>_)
-    ///         A list of points to save.
-    ///     encoding (_Encoding_)
+    /// @param mvc (_MeshValueCollection<bool>_)
+    ///         MeshValueCollection to save
+    /// @param encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const MeshValueCollection<bool>& mvc,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Write out mesh value collection (subset) using an associated
     /// HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     mvc (_MeshValueCollection<int>_)
-    ///         A list of points to save.
-    ///     encoding (_Encoding_)
+    /// @param mvc (_MeshValueCollection<int>_)
+    ///         MeshValueCollection to save
+    /// @param encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const MeshValueCollection<int>& mvc,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Write out mesh value collection (subset) using an associated
     /// HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     mvc (_MeshValueCollection<int>_)
-    ///         A list of points to save.
-    ///     encoding (_Encoding_)
+    /// @param  mvc (_MeshValueCollection<int>_)
+    ///         MeshValueCollection to save
+    /// @param  encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const MeshValueCollection<std::size_t>& mvc,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Write out mesh value collection (subset) using an associated
     /// HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     mvc (_MeshValueCollection<double>_)
-    ///         A list of points to save.
-    ///     encoding (_Encoding_)
+    /// @param mvc (_MeshValueCollection<double>_)
+    ///         MeshValueCollection to save
+    /// @param encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const MeshValueCollection<double>& mvc,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Save a cloud of points to file using an associated HDF5 file,
     /// or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     points (_std::vector<Point>_)
+    /// @param    points (_std::vector<Point>_)
     ///         A list of points to save.
-    ///     encoding (_Encoding_)
+    /// @param    encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const std::vector<Point>& points,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
     /// Save a cloud of points, with scalar values using an associated
     /// HDF5 file, or storing the data inline as XML.
     ///
-    /// *Arguments*
-    ///     points (_std::vector<Point>_)
+    /// @param   points (_std::vector<Point>_)
     ///         A list of points to save.
-    ///     values (_std::vector<double>_)
+    /// @param    values (_std::vector<double>_)
     ///         A list of values at each point.
-    ///     encoding (_Encoding_)
+    /// @param    encoding (_Encoding_)
     ///         Encoding to use: HDF5 or ASCII
     ///
     void write(const std::vector<Point>& points,
                const std::vector<double>& values,
-               Encoding encoding=Encoding::HDF5);
+               Encoding encoding=default_encoding);
 
-    /// Read in a mesh
-    ///
-    /// *Arguments*
-    ///     mesh (_Mesh_)
+    /// Read in the first Mesh in XDMF file
     ///
+    /// @param mesh (_Mesh_)
+    ///        Mesh to fill from XDMF file
     void read(Mesh& mesh) const;
 
+    /// Read a function from the XDMF file. Supplied function must
+    /// come with already initialized and compatible function space.
+    ///
+    /// Functions saved as time-series must be addressed through
+    /// (integer) internal counter and function name. Function name
+    /// is a name of function that was saved. Counter stands for
+    /// the number of function from time-series, e.g. counter=0
+    /// refers to first saved function regardless of its time-step value.
+    ///
+    /// @param    u (_Function_)
+    ///         A function to read.
+    /// @param    func_name (_string_)
+    ///         A name of a function to read. Must be the same on all processes
+    ///         in parallel.
+    /// @param    counter (_int64_t_)
+    ///         Internal integer counter - used in time-series. Default value
+    ///         is -1 which points to last saved function. Counter works same as
+    ///         python array position key, i.e. counter = -2 points to the
+    ///         function before the last one.
+    ///
+    void read_checkpoint(Function& u, std::string func_name,
+                         std::int64_t counter=-1);
+
     /// Read first MeshFunction from file
-    /// @param meshfunction
-    /// @param name
+    /// @param meshfunction (_MeshFunction<bool>_)
+    ///        MeshFunction to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshFunction<bool>& meshfunction, std::string name="");
 
     /// Read first MeshFunction from file
-    /// @param meshfunction
-    /// @param name
+    /// @param meshfunction (_MeshFunction<int>_)
+    ///        MeshFunction to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshFunction<int>& meshfunction, std::string name="");
 
-    /// Read first MeshFunction from file
-    /// @param meshfunction
-    /// @param name
+    /// Read MeshFunction from file, optionally specifying dataset name
+    /// @param meshfunction (_MeshFunction<std::size_t>_)
+    ///        MeshFunction to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshFunction<std::size_t>& meshfunction, std::string name="");
 
-    /// Read first MeshFunction from file
-    /// @param meshfunction
-    /// @param name
+    /// Read MeshFunction from file, optionally specifying dataset name
+    /// @param meshfunction (_MeshFunction<double>_)
+    ///        MeshFunction to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshFunction<double>& meshfunction, std::string name="");
 
-    /// Read MeshValueCollection from file
+    /// Read MeshValueCollection from file, optionally specifying dataset name
+    /// @param mvc (_MeshValueCollection<bool>_)
+    ///        MeshValueCollection to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshValueCollection<bool>& mvc, std::string name="");
 
-    /// Read MeshValueCollection from file
+    /// Read MeshValueCollection from file, optionally specifying dataset name
+    /// @param mvc (_MeshValueCollection<int>_)
+    ///        MeshValueCollection to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshValueCollection<int>& mvc, std::string name="");
 
-    /// Read MeshValueCollection from file
+    /// Read MeshValueCollection from file, optionally specifying dataset name
+    /// @param mvc (_MeshValueCollection<std::size_t>_)
+    ///        MeshValueCollection to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshValueCollection<std::size_t>& mvc, std::string name="");
 
-    /// Read MeshValueCollection from file
+    /// Read MeshValueCollection from file, optionally specifying dataset name
+    /// @param mvc (_MeshValueCollection<double>_)
+    ///        MeshValueCollection to restore
+    /// @param name (std::string)
+    ///        Name of data attribute in XDMF file
     void read(MeshValueCollection<double>& mvc, std::string name="");
 
   private:
@@ -303,11 +429,18 @@ namespace dolfin
 
 
 
-    // Add mesh to XDMF xml_node (usually a Domain or Time Grid) and write data
+    // Add mesh to XDMF xml_node (usually a Domain or Time Grid) and
+    // write data
     static void add_mesh(MPI_Comm comm, pugi::xml_node& xml_node,
                          hid_t h5_id, const Mesh& mesh,
                          const std::string path_prefix);
 
+    // Add function to a XML node
+    static void add_function(MPI_Comm comm, pugi::xml_node& xml_node,
+                             hid_t h5_id, std::string h5_path,
+                             const Function& u, std::string function_name,
+                             const Mesh& mesh);
+
     // Add set of points to XDMF xml_node and write data
     static void add_points(MPI_Comm comm, pugi::xml_node& xml_node,
                            hid_t h5_id, const std::vector<Point>& points);
@@ -324,17 +457,19 @@ namespace dolfin
                                   hid_t h5_id, const std::string path_prefix,
                                   const Mesh& mesh);
 
-    // Add DataItem node to an XML node. If HDF5 is open (h5_id > 0) the data is
-    // written to the HDFF5 file with the path 'h5_path'. Otherwise, data is
-    // witten to the XML node and 'h5_path' is ignored
+    // Add DataItem node to an XML node. If HDF5 is open (h5_id > 0)
+    // the data is written to the HDFF5 file with the path
+    // 'h5_path'. Otherwise, data is witten to the XML node and
+    // 'h5_path' is ignored
     template<typename T>
     static void add_data_item(MPI_Comm comm, pugi::xml_node& xml_node,
                               hid_t h5_id, const std::string h5_path, const T& x,
                               const std::vector<std::int64_t> dimensions,
                               const std::string number_type="");
 
-    // Calculate set of entities of dimension cell_dim which are duplicated
-    // on other processes and should not be output on this process
+    // Calculate set of entities of dimension cell_dim which are
+    // duplicated on other processes and should not be output on this
+    // process
     static std::set<unsigned int> compute_nonlocal_entities(const Mesh& mesh,
                                                             int cell_dim);
 
@@ -365,9 +500,11 @@ namespace dolfin
     template <typename T>
     static std::vector<T> get_dataset(MPI_Comm comm,
                                       const pugi::xml_node& dataset_node,
-                                      const boost::filesystem::path& parent_path);
+                                      const boost::filesystem::path& parent_path,
+                                      std::pair<std::int64_t, std::int64_t> range={0, 0});
 
-    // Return (0) HDF5 filename and (1) path in HDF5 file from a DataItem node
+    // Return (0) HDF5 filename and (1) path in HDF5 file from a
+    // DataItem node
     static std::array<std::string, 2> get_hdf5_paths(const pugi::xml_node& dataitem_node);
 
     static std::string get_hdf5_filename(std::string xdmf_filename);
@@ -381,8 +518,9 @@ namespace dolfin
     void write_mesh_function(const MeshFunction<T>& meshfunction,
                              Encoding encoding);
 
-    // Get data width - normally the same as u.value_size(), but expand for 2D
-    // vector/tensor because XDMF presents everything as 3D
+    // Get data width - normally the same as u.value_size(), but
+    // expand for 2D vector/tensor because XDMF presents everything as
+    // 3D
     static std::int64_t get_padded_width(const Function& u);
 
     // Returns true for DG0 Functions
@@ -392,8 +530,8 @@ namespace dolfin
     // flattened 2D array
     static std::vector<double> get_point_data_values(const Function& u);
 
-    // Get point data values collocated at P2 geometry points (vertices
-    // and edges) flattened as a 2D array
+    // Get point data values collocated at P2 geometry points
+    // (vertices and edges) flattened as a 2D array
     static std::vector<double> get_p2_data_values(const Function& u);
 
     // Get cell data values as a flattened 2D array
@@ -402,6 +540,9 @@ namespace dolfin
     // Check whether the requested encoding is supported
     void check_encoding(Encoding encoding) const;
 
+    // Check function names equality across processes
+    void check_function_name(std::string function_name);
+
     // Generate the XDMF format string based on the Encoding
     // enumeration
     static std::string xdmf_format_str(Encoding encoding)
@@ -413,18 +554,20 @@ namespace dolfin
     template <typename X, typename Y>
     static std::string to_string(X x, Y y);
 
-    // Return a vector of numerical values from a vector of stringstream
+    // Return a vector of numerical values from a vector of
+    // stringstream
     template <typename T>
     static std::vector<T> string_to_vector(const std::vector<std::string>& x_str);
 
-    // Convert a value_rank to the XDMF string description (Scalar, Vector, Tensor)
+    // Convert a value_rank to the XDMF string description (Scalar,
+    // Vector, Tensor)
     static std::string rank_to_string(std::size_t value_rank);
 
     // MPI communicator
-    MPI_Comm _mpi_comm;
+    dolfin::MPI::Comm _mpi_comm;
 
-    // HDF5 data file
 #ifdef HAS_HDF5
+    // HDF5 data file
     std::unique_ptr<HDF5File> _hdf5_file;
 #endif
 
@@ -440,6 +583,7 @@ namespace dolfin
 
   };
 
+#ifndef DOXYGEN_IGNORE
   // Specialisation for std::vector<bool>, as HDF5 does not support it natively
   template<> inline
   void XDMFFile::add_data_item(MPI_Comm comm, pugi::xml_node& xml_node,
@@ -454,6 +598,7 @@ namespace dolfin
       x_int[i] = (int)x[i];
     add_data_item(comm, xml_node, h5_id, h5_path, x_int, shape, number_type);
   }
+#endif
 
 }
 
