@@ -24,23 +24,22 @@
 #include "DefaultFactory.h"
 #include "KrylovSolver.h"
 #include "LUSolver.h"
-#include "CholmodCholeskySolver.h"
 #include "LinearSolver.h"
 
 using namespace dolfin;
 
 //-----------------------------------------------------------------------------
-LinearSolver::LinearSolver(std::string method,
+LinearSolver::LinearSolver(MPI_Comm comm,
+                           std::string method,
                            std::string preconditioner)
 {
   // Get default linear algebra factory
   DefaultFactory factory;
 
   // Get list of available methods
-  std::vector<std::pair<std::string, std::string> >
-    lu_methods = factory.lu_solver_methods();
-  std::vector<std::pair<std::string, std::string> >
-    krylov_methods = factory.krylov_solver_methods();
+  std::map<std::string, std::string> lu_methods = factory.lu_solver_methods();
+  std::map<std::string, std::string> krylov_methods
+    = factory.krylov_solver_methods();
 
   // Handle some default and generic solver options
   if (method == "default")
@@ -70,27 +69,7 @@ LinearSolver::LinearSolver(std::string method,
       method = "default";
 
     // Initialize solver
-    solver.reset(new LUSolver(method));
-
-    // Set parameter type
-    _parameter_type = "lu_solver";
-  }
-  else if (method == "cholesky")
-  {
-    // Adjust preconditioner default --> none
-    if (preconditioner == "default")
-      preconditioner = "none";
-
-    // Check that preconditioner has not been set
-    if (preconditioner != "none")
-    {
-      dolfin_error("LinearSolver.cpp",
-                   "solve linear system",
-                   "Preconditioner may not be specified for Cholesky solver");
-    }
-
-    // Initialize solver
-    solver.reset(new CholmodCholeskySolver());
+    solver.reset(new LUSolver(comm, method));
 
     // Set parameter type
     _parameter_type = "lu_solver";
@@ -100,7 +79,7 @@ LinearSolver::LinearSolver(std::string method,
     // Method and preconditioner will be checked by KrylovSolver
 
     // Initialize solver
-    solver.reset(new KrylovSolver(method, preconditioner));
+    solver.reset(new KrylovSolver(comm, method, preconditioner));
 
     // Set parameter type
     _parameter_type = "krylov_solver";
@@ -159,14 +138,9 @@ std::size_t LinearSolver::solve(GenericVector& x, const GenericVector& b)
   return solver->solve(x, b);
 }
 //-----------------------------------------------------------------------------
-bool
-LinearSolver::in_list(const std::string& method,
-                      const std::vector<std::pair<std::string, std::string> > methods)
+bool LinearSolver::in_list(const std::string& method,
+                           const std::map<std::string, std::string>& methods)
 {
-  for (std::size_t i = 0; i < methods.size(); i++)
-    if (method == methods[i].first)
-      return true;
-
-  return false;
+  return methods.find(method) != methods.end();
 }
 //-----------------------------------------------------------------------------
