@@ -1,4 +1,4 @@
-// Copyright (C) 2005-2011 Anders Logg
+// Copyright (C) 2005-2017 Anders Logg and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -14,11 +14,6 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// Modified by Garth N. Wells, 2009-2010.
-//
-// First added:  2005
-// Last changed: 2011-10-19
 
 #ifndef __DOLFIN_PETSC_LU_SOLVER_H
 #define __DOLFIN_PETSC_LU_SOLVER_H
@@ -28,11 +23,10 @@
 #include <map>
 #include <memory>
 #include <string>
-#include <petscksp.h>
+#include <petscmat.h>
 #include <petscpc.h>
 #include <dolfin/common/MPI.h>
-#include "GenericLUSolver.h"
-#include "PETScObject.h"
+#include "PETScKrylovSolver.h"
 
 namespace dolfin
 {
@@ -42,13 +36,12 @@ namespace dolfin
   class PETScLinearOperator;
   class PETScMatrix;
   class PETScVector;
-  class PETScSNESSolver;
 
   /// This class implements the direct solution (LU factorization) for
   /// linear systems of the form Ax = b. It is a wrapper for the LU
   /// solver of PETSc.
 
-  class PETScLUSolver : public GenericLUSolver, public PETScObject
+  class PETScLUSolver : public GenericLinearSolver
   {
   public:
 
@@ -74,15 +67,12 @@ namespace dolfin
     void set_operator(std::shared_ptr<const GenericLinearOperator> A);
 
     /// Set operator (matrix)
-    void set_operator(std::shared_ptr<const PETScMatrix> A);
-
-    /// Get operator (matrix)
-    const GenericLinearOperator& get_operator() const;
+    void set_operator(const PETScMatrix& A);
 
     /// Solve linear system Ax = b
     std::size_t solve(GenericVector& x, const GenericVector& b);
 
-    /// Solve linear system Ax = b
+    /// Solve linear system Ax = b (A^t x = b if transpose is true)
     std::size_t solve(GenericVector& x, const GenericVector& b,
                       bool transpose);
 
@@ -94,17 +84,6 @@ namespace dolfin
     std::size_t solve(const PETScMatrix& A, PETScVector& x,
                       const PETScVector& b);
 
-    /// Solve linear system A^Tx = b
-    std::size_t solve_transpose(GenericVector& x, const GenericVector& b);
-
-    /// Solve linear system A^Tx = b
-    std::size_t solve_transpose(const GenericLinearOperator& A,
-                                GenericVector& x, const GenericVector& b);
-
-    /// Solve linear system A^Tx = b
-    std::size_t solve_transpose(const PETScMatrix& A, PETScVector& x,
-                                const PETScVector& b);
-
     /// Sets the prefix used by PETSc when searching the options
     /// database
     void set_options_prefix(std::string options_prefix);
@@ -131,39 +110,25 @@ namespace dolfin
     /// Default parameter values
     static Parameters default_parameters();
 
+    /// Return parameter type: "krylov_solver" or "lu_solver"
+    std::string parameter_type() const
+    { return "lu_solver"; }
+
+    // FIXME: These should not be friend classes
     friend class PETScSNESSolver;
     friend class PETScTAOSolver;
 
   private:
 
-    const MatSolverPackage _solver_package;
-
+    // FIXME: Remove
     // Available LU solvers
-    static std::map<std::string, const MatSolverPackage> _methods;
-
-    // Whether those solvers support Cholesky
-    static std::map<const MatSolverPackage, bool> _methods_cholesky;
-
-    // Available LU solvers descriptions
-    static const std::map<std::string, std::string> _methods_descr;
+    static std::map<std::string, const MatSolverPackage> lumethods;
 
     // Select LU solver type
-    static const MatSolverPackage select_solver(MPI_Comm comm, std::string& method);
+    static const MatSolverPackage select_solver(MPI_Comm comm,
+                                                std::string method);
 
-    // Does an LU solver support Cholesky?
-    static bool solver_has_cholesky(const MatSolverPackage package);
-
-    // Configure PETSc options
-    void configure_ksp(const MatSolverPackage solver_package);
-
-    // Print pre-solve report
-    void pre_report(const PETScMatrix& A) const;
-
-    // PETSc solver pointer
-    KSP _ksp;
-
-    // Operator (the matrix)
-    std::shared_ptr<const PETScMatrix> _matA;
+    PETScKrylovSolver _solver;
 
   };
 
