@@ -1,4 +1,5 @@
-// Copyright (C) 2004-2012 Johan Hoffman, Johan Jansson and Anders Logg
+// Copyright (C) 2004-2016 Johan Hoffman, Johan Jansson, Anders Logg
+// and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -19,93 +20,73 @@
 // Modified by Martin Sandve Alnes 2008
 // Modified by Johannes Ring 2011.
 // Modified by Fredrik Valdmanis 2011-2012
-//
-// First added:  2004
-// Last changed: 2012-08-22
 
 #ifdef HAS_PETSC
 
 #include <cmath>
+#include <cstddef>
+#include <cstring>
 #include <numeric>
 #include <dolfin/common/Timer.h>
 #include <dolfin/common/Array.h>
-#include <dolfin/common/NoDeleter.h>
-#include <dolfin/common/Set.h>
-#include <dolfin/log/dolfin_log.h>
-#include "PETScVector.h"
-#include "uBLASVector.h"
-#include "PETScFactory.h"
-#include "PETScCuspFactory.h"
 #include <dolfin/common/MPI.h>
+#include <dolfin/log/log.h>
+#include "SparsityPattern.h"
+#include "PETScVector.h"
+#include "PETScFactory.h"
 
 using namespace dolfin;
 
 const std::map<std::string, NormType> PETScVector::norm_types
 = { {"l1",   NORM_1}, {"l2",   NORM_2},  {"linf", NORM_INFINITY} };
 
-//-----------------------------------------------------------------------------
-PETScVector::PETScVector() : _x(NULL), _use_gpu(false)
-{
-#ifndef HAS_PETSC_CUSP
-  if (_use_gpu)
-  {
-    dolfin_error("PETScVector.cpp",
-                 "create empty GPU vector",
-                 "PETSc not compiled with Cusp support");
-  }
-#endif
-}
-//-----------------------------------------------------------------------------
-PETScVector::PETScVector(MPI_Comm comm, std::size_t N, bool use_gpu)
-  : _x(NULL), _use_gpu(use_gpu)
-{
-  #ifndef HAS_PETSC_CUSP
-  if (_use_gpu)
-  {
-    dolfin_error("PETScVector.cpp",
-                 "create GPU vector",
-                 "PETSc not compiled with Cusp support");
-  }
-  #endif
 
-  // Empty ghost indices vector
-  const std::vector<la_index> ghost_indices;
-  const std::vector<std::size_t> local_to_global_map;
+#define CHECK_ERROR(NAME) do { if (ierr != 0) petsc_error(ierr, __FILE__, NAME); } while(0)
 
-  // Compute a local range
-  const std::pair<std::size_t, std::size_t> range = MPI::local_range(comm, N);
-  _init(comm, range, local_to_global_map, ghost_indices);
+
+//-----------------------------------------------------------------------------
+PETScVector::PETScVector() : PETScVector(MPI_COMM_WORLD)
+{
+  // Do nothing
 }
 //-----------------------------------------------------------------------------
-PETScVector::PETScVector(const GenericSparsityPattern& sparsity_pattern)
-  : _x(NULL), _use_gpu(false)
+PETScVector::PETScVector(MPI_Comm comm) : _x(nullptr)
 {
-  std::vector<la_index> ghost_indices;
-  std::vector<std::size_t> local_to_global_map;
-  _init(sparsity_pattern.mpi_comm(), sparsity_pattern.local_range(0),
-        local_to_global_map, ghost_indices);
+  PetscErrorCode ierr = VecCreate(comm, &_x);
+  CHECK_ERROR("VecCreate");
 }
 //-----------------------------------------------------------------------------
-PETScVector::PETScVector(Vec x): _x(x), _use_gpu(false)
+PETScVector::PETScVector(MPI_Comm comm, std::size_t N) : PETScVector(comm)
 {
-  // Increase reference count
+  // Compute a local range and initialise vector
+  const auto range = dolfin::MPI::local_range(comm, N);
+  _init(range, {}, {});
+}
+//-----------------------------------------------------------------------------
+PETScVector::PETScVector(const SparsityPattern& sparsity_pattern)
+  : PETScVector(sparsity_pattern.mpi_comm())
+{
+  _init(sparsity_pattern.local_range(0), {}, {});
+}
+//-----------------------------------------------------------------------------
+PETScVector::PETScVector(Vec x) : _x(x)
+{
+  // Increase reference count to PETSc object
   PetscObjectReference((PetscObject)_x);
 }
 //-----------------------------------------------------------------------------
-PETScVector::PETScVector(const PETScVector& v) : _x(NULL), _use_gpu(false)
+PETScVector::PETScVector(const PETScVector& v) : _x(nullptr)
 {
-  PetscErrorCode ierr;
+  dolfin_assert(v._x);
 
   // Create new vector
+  PetscErrorCode ierr;
   ierr = VecDuplicate(v._x, &_x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecDuplicate");
+  CHECK_ERROR("VecDuplicate");
 
   // Copy data
   ierr = VecCopy(v._x, _x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecCopy");
-
-  // Copy ghost data
-  ghost_global_to_local = v.ghost_global_to_local;
+  CHECK_ERROR("VecCopy");
 
   // Update ghost values
   update_ghost_values();
@@ -117,76 +98,35 @@ PETScVector::~PETScVector()
     VecDestroy(&_x);
 }
 //-----------------------------------------------------------------------------
-bool PETScVector::distributed() const
-{
-  dolfin_assert(_x);
-
-  // Get type
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 3
-  const VecType petsc_type;
-  #else
-  VecType petsc_type;
-  #endif
-  PetscErrorCode ierr = VecGetType(_x, &petsc_type);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetType");
-
-  // Return type
-  bool _distributed = false;
-  if (strcmp(petsc_type, VECMPI) == 0)
-    _distributed = true;
-  else if (strcmp(petsc_type, VECSEQ) == 0)
-    _distributed =  false;
-  #ifdef HAS_PETSC_CUSP
-  // TODO: Uncomment these two lines after implementing MPI Cusp vectors
-  //else if (strcmp(petsc_type, VECMPICUSP) == 0)
-  //  _distributed = true;
-  else if (strcmp(petsc_type, VECSEQCUSP) == 0)
-    _distributed = false;
-  #endif
-  else
-  {
-    dolfin_error("PETScVector.cpp",
-                 "check whether PETSc vector is distributed",
-                 "Unknown vector type (\"%s\")", petsc_type);
-  }
-
-  return _distributed;
-}
-//-----------------------------------------------------------------------------
 std::shared_ptr<GenericVector> PETScVector::copy() const
 {
-  return std::shared_ptr<GenericVector>(new PETScVector(*this));
+  return std::make_shared<PETScVector>(*this);
 }
 //-----------------------------------------------------------------------------
-void PETScVector::init(MPI_Comm comm, std::size_t N)
+void PETScVector::init(std::size_t N)
 {
-  const std::pair<std::size_t, std::size_t> range
-    = MPI::local_range(comm, N);
-  init(comm, range);
+  const auto range = dolfin::MPI::local_range(this->mpi_comm(), N);
+  _init(range, {}, {});
 }
 //-----------------------------------------------------------------------------
-void PETScVector::init(MPI_Comm comm,
-                       std::pair<std::size_t, std::size_t> range)
+void PETScVector::init(std::pair<std::size_t, std::size_t> range)
 {
-  // Create empty ghost indices vector
-  std::vector<la_index> ghost_indices;
-  std::vector<std::size_t> local_to_global_map;
-  init(comm, range, local_to_global_map, ghost_indices);
+  _init(range, {}, {});
 }
 //-----------------------------------------------------------------------------
-void PETScVector::init(MPI_Comm comm,
-                       std::pair<std::size_t, std::size_t> range,
+void PETScVector::init(std::pair<std::size_t, std::size_t> range,
                        const std::vector<std::size_t>& local_to_global_map,
                        const std::vector<la_index>& ghost_indices)
 {
-  // Re-initialise vector
-  _init(comm, range, local_to_global_map, ghost_indices);
+  // Initialise vector
+  _init(range, local_to_global_map, ghost_indices);
 }
 //-----------------------------------------------------------------------------
 void PETScVector::get_local(std::vector<double>& values) const
 {
   dolfin_assert(_x);
-  const std::size_t local_size = local_range().second - local_range().first;
+  const auto _local_range = local_range();
+  const std::size_t local_size = _local_range.second - _local_range.first;
   values.resize(local_size);
 
   if (local_size == 0)
@@ -195,20 +135,21 @@ void PETScVector::get_local(std::vector<double>& values) const
   // Get pointer to PETSc vector data
   const PetscScalar* data;
   PetscErrorCode ierr = VecGetArrayRead(_x, &data);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetArrayRead");
+  CHECK_ERROR("VecGetArrayRead");
 
   // Copy data into vector
   std::copy(data, data + local_size, values.begin());
 
   // Restore array
   ierr = VecRestoreArrayRead(_x, &data);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecRestoreArrayRead");
+  CHECK_ERROR("VecRestoreArrayRead");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::set_local(const std::vector<double>& values)
 {
   dolfin_assert(_x);
-  const std::size_t local_size = local_range().second - local_range().first;
+  const auto _local_range = local_range();
+  const std::size_t local_size = _local_range.second - _local_range.first;
   if (values.size() != local_size)
   {
     dolfin_error("PETScVector.cpp",
@@ -221,18 +162,18 @@ void PETScVector::set_local(const std::vector<double>& values)
 
   // Build array of local indices
   std::vector<PetscInt> rows(local_size, 0);
-  for (std::size_t i = 0; i < local_size; ++i)
-    rows[i] += i;
+  std::iota(rows.begin(), rows.end(), 0);
 
   PetscErrorCode ierr = VecSetValuesLocal(_x, local_size, rows.data(),
                                           values.data(), INSERT_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValues");
+  CHECK_ERROR("VecSetValuesLocal");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::add_local(const Array<double>& values)
 {
   dolfin_assert(_x);
-  const std::size_t local_size = local_range().second - local_range().first;
+  const auto _local_range = local_range();
+  const std::size_t local_size = _local_range.second - _local_range.first;
   if (values.size() != local_size)
   {
     dolfin_error("PETScVector.cpp",
@@ -244,13 +185,12 @@ void PETScVector::add_local(const Array<double>& values)
     return;
 
   // Build array of local indices
-  std::vector<PetscInt> rows(local_size, 0);
-  for (std::size_t i = 0; i < local_size; ++i)
-    rows[i] += i;
+  std::vector<PetscInt> rows(local_size);
+  std::iota(rows.begin(), rows.end(), 0);
 
   PetscErrorCode ierr = VecSetValuesLocal(_x, local_size, rows.data(),
-                                     values.data(), ADD_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValuesLocal");
+                                          values.data(), ADD_VALUES);
+  CHECK_ERROR("VecSetValuesLocal");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::get_local(double* block, std::size_t m,
@@ -262,9 +202,10 @@ void PETScVector::get_local(double* block, std::size_t m,
   dolfin_assert(_x);
   PetscErrorCode ierr;
 
-  Vec xg;
+  // Get ghost vector
+  Vec xg = nullptr;
   ierr = VecGhostGetLocalForm(_x, &xg);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostGetLocalForm");
+  CHECK_ERROR("VecGhostGetLocalForm");
 
   // Use array access if no ghost points, otherwise use VecGetValues
   // on local ghosted form of vector
@@ -272,23 +213,24 @@ void PETScVector::get_local(double* block, std::size_t m,
   {
     // Get pointer to PETSc vector data
     const PetscScalar* data;
-    PetscErrorCode ierr = VecGetArrayRead(_x, &data);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetArrayRead");
+    ierr = VecGetArrayRead(_x, &data);
+    CHECK_ERROR("VecGetArrayRead");
 
     for (std::size_t i = 0; i < m; ++i)
       block[i] = data[rows[i]];
 
     // Restore array
     ierr = VecRestoreArrayRead(_x, &data);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecRestoreArrayRead");
+    CHECK_ERROR("VecRestoreArrayRead");
   }
   else
   {
     dolfin_assert(xg);
     ierr = VecGetValues(xg, m, rows, block);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetValues");
+    CHECK_ERROR("VecGetValues");
+
     ierr = VecGhostRestoreLocalForm(_x, &xg);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostRestoreLocalForm");
+    CHECK_ERROR("VecGhostRestoreLocalForm");
   }
 }
 //-----------------------------------------------------------------------------
@@ -299,49 +241,40 @@ void PETScVector::get(double* block, std::size_t m,
     return;
 
   dolfin_assert(_x);
-  PetscErrorCode ierr;
-  ierr = VecGetValues(_x, m, rows, block);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetValues");
+  PetscErrorCode ierr = VecGetValues(_x, m, rows, block);
+  CHECK_ERROR("VecGetValues");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::set(const double* block, std::size_t m,
                       const dolfin::la_index* rows)
 {
   dolfin_assert(_x);
-  if (m == 0)
-    return;
   PetscErrorCode ierr = VecSetValues(_x, m, rows, block, INSERT_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValues");
+  CHECK_ERROR("VecSetValues");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::set_local(const double* block, std::size_t m,
                             const dolfin::la_index* rows)
 {
   dolfin_assert(_x);
-  if (m == 0)
-    return;
   PetscErrorCode ierr = VecSetValuesLocal(_x, m, rows, block, INSERT_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValuesLocal");
+  CHECK_ERROR("VecSetValuesLocal");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::add(const double* block, std::size_t m,
                       const dolfin::la_index* rows)
 {
   dolfin_assert(_x);
-  if (m == 0)
-    return;
   PetscErrorCode ierr = VecSetValues(_x, m, rows, block, ADD_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValues");
+  CHECK_ERROR("VecSetValues");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::add_local(const double* block, std::size_t m,
                             const dolfin::la_index* rows)
 {
   dolfin_assert(_x);
-  if (m == 0)
-    return;
   PetscErrorCode ierr = VecSetValuesLocal(_x, m, rows, block, ADD_VALUES);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetValuesLocal");
+  CHECK_ERROR("VecSetValuesLocal");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::apply(std::string mode)
@@ -350,9 +283,9 @@ void PETScVector::apply(std::string mode)
   dolfin_assert(_x);
   PetscErrorCode ierr;
   ierr = VecAssemblyBegin(_x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecAssemblyBegin");
+  CHECK_ERROR("VecAssemblyBegin");
   ierr = VecAssemblyEnd(_x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecAssemblyEnd");
+  CHECK_ERROR("VecAssemblyEnd");
 
   // Update any ghost values
   update_ghost_values();
@@ -362,7 +295,8 @@ MPI_Comm PETScVector::mpi_comm() const
 {
   dolfin_assert(_x);
   MPI_Comm mpi_comm = MPI_COMM_NULL;
-  PetscObjectGetComm((PetscObject)(_x), &mpi_comm);
+  PetscErrorCode ierr = PetscObjectGetComm((PetscObject)(_x), &mpi_comm);
+  CHECK_ERROR("PetscObjectGetComm");
   return mpi_comm;
 }
 //-----------------------------------------------------------------------------
@@ -371,52 +305,71 @@ void PETScVector::zero()
   dolfin_assert(_x);
   double a = 0.0;
   PetscErrorCode ierr = VecSet(_x, a);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
+  CHECK_ERROR("VecSet");
   this->apply("insert");
 }
 //-----------------------------------------------------------------------------
 bool PETScVector::empty() const
 {
-  return size() == 0;
+  return this->size() == 0;
 }
 //-----------------------------------------------------------------------------
 std::size_t PETScVector::size() const
 {
+  dolfin_assert(_x);
+  PetscErrorCode ierr;
+
+  // Return zero if vector type has not been set (Vec has not been
+  // initialized)
+  VecType vec_type = nullptr;
+  ierr = VecGetType(_x, &vec_type);
+  if (vec_type == nullptr)
+    return 0;
+  CHECK_ERROR("VecGetType");
+
   PetscInt n = 0;
-  if (_x)
-  {
-    PetscErrorCode ierr = VecGetSize(_x, &n);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetSize");
-  }
-  return n;
+  dolfin_assert(_x);
+  ierr = VecGetSize(_x, &n);
+  CHECK_ERROR("VecGetSize");
+
+  return n > 0 ? n : 0;
 }
 //-----------------------------------------------------------------------------
 std::size_t PETScVector::local_size() const
 {
+  dolfin_assert(_x);
+  PetscErrorCode ierr;
+
+  // Return zero if vector type has not been set
+  VecType vec_type = nullptr;
+  ierr = VecGetType(_x, &vec_type);
+  if (vec_type == nullptr)
+    return 0;
+  CHECK_ERROR("VecGetType");
+
   PetscInt n = 0;
-  if (_x)
-  {
-    PetscErrorCode ierr = VecGetLocalSize(_x, &n);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetLocalSize");
-  }
+  ierr = VecGetLocalSize(_x, &n);
+  CHECK_ERROR("VecGetLocalSize");
+
   return n;
 }
 //-----------------------------------------------------------------------------
-std::pair<std::size_t, std::size_t> PETScVector::local_range() const
+std::pair<std::int64_t, std::int64_t> PETScVector::local_range() const
 {
+  dolfin_assert(_x);
+
   PetscInt n0, n1;
   PetscErrorCode ierr = VecGetOwnershipRange(_x, &n0, &n1);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecGetOwnershipRange");
+  CHECK_ERROR("VecGetOwnershipRange");
   dolfin_assert(n0 <= n1);
-  return std::make_pair(n0, n1);
+  return {n0, n1};
 }
 //-----------------------------------------------------------------------------
 bool PETScVector::owns_index(std::size_t i) const
 {
-  if (i >= local_range().first && i < local_range().second)
-    return true;
-  else
-    return false;
+  const auto _local_range = local_range();
+  const std::int64_t _i = i;
+  return _i >= _local_range.first && _i < _local_range.second;
 }
 //-----------------------------------------------------------------------------
 const GenericVector& PETScVector::operator= (const GenericVector& v)
@@ -452,7 +405,7 @@ const PETScVector& PETScVector::operator= (const PETScVector& v)
     dolfin_assert(v._x);
     dolfin_assert(_x);
     PetscErrorCode ierr = VecCopy(v._x, _x);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecCopy");
+    CHECK_ERROR("VecCopy");
 
     // Update ghost values
     update_ghost_values();
@@ -464,7 +417,7 @@ const PETScVector& PETScVector::operator= (double a)
 {
   dolfin_assert(_x);
   PetscErrorCode ierr = VecSet(_x, a);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
+  CHECK_ERROR("VecSet");
   apply("insert");
   return *this;
 }
@@ -474,27 +427,22 @@ void PETScVector::update_ghost_values()
   dolfin_assert(_x);
   PetscErrorCode ierr;
 
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 3
-  if (dolfin::MPI::size(mpi_comm()) > 1)
-  #endif
+  // Check of vector is ghosted
+  Vec xg;
+  ierr = VecGhostGetLocalForm(_x, &xg);
+  CHECK_ERROR("VecGhostGetLocalForm");
+
+  // If ghosted, update
+  if (xg)
   {
-    // Check of vector is ghosted
-    Vec xg;
-    ierr = VecGhostGetLocalForm(_x, &xg);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostGetLocalForm");
-
-    // If ghosted, update
-    if (xg)
-    {
-      ierr = VecGhostUpdateBegin(_x, INSERT_VALUES, SCATTER_FORWARD);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostUpdateBegin");
-      ierr = VecGhostUpdateEnd(_x, INSERT_VALUES, SCATTER_FORWARD);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostUpdateEnd");
-    }
-
-    ierr = VecGhostRestoreLocalForm(_x, &xg);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGhostRestoreLocalForm");
+    ierr = VecGhostUpdateBegin(_x, INSERT_VALUES, SCATTER_FORWARD);
+    CHECK_ERROR("VecGhostUpdateBegin");
+    ierr = VecGhostUpdateEnd(_x, INSERT_VALUES, SCATTER_FORWARD);
+    CHECK_ERROR("VecGhostUpdateEnd");
   }
+
+  ierr = VecGhostRestoreLocalForm(_x, &xg);
+  CHECK_ERROR("VecGhostRestoreLocalForm");
 }
 //-----------------------------------------------------------------------------
 const PETScVector& PETScVector::operator+= (const GenericVector& x)
@@ -507,7 +455,7 @@ const PETScVector& PETScVector::operator+= (double a)
 {
   dolfin_assert(_x);
   PetscErrorCode ierr = VecShift(_x, a);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecShift");
+  CHECK_ERROR("VecShift");
 
   // Update any ghost values
   update_ghost_values();
@@ -532,7 +480,7 @@ const PETScVector& PETScVector::operator*= (const double a)
 {
   dolfin_assert(_x);
   PetscErrorCode ierr = VecScale(_x, a);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScale");
+  CHECK_ERROR("VecScale");
 
   // Update ghost values
   update_ghost_values();
@@ -553,7 +501,7 @@ const PETScVector& PETScVector::operator*= (const GenericVector& y)
   }
 
   PetscErrorCode ierr = VecPointwiseMult(_x, _x, v._x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecPointwiseMult");
+  CHECK_ERROR("VecPointwiseMult");
 
   // Update ghost values
   update_ghost_values();
@@ -577,7 +525,7 @@ double PETScVector::inner(const GenericVector& y) const
   dolfin_assert(_y._x);
   double a;
   PetscErrorCode ierr = VecDot(_y._x, _x, &a);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecDot");
+  CHECK_ERROR("VecDot");
   return a;
 }
 //-----------------------------------------------------------------------------
@@ -595,7 +543,7 @@ void PETScVector::axpy(double a, const GenericVector& y)
   }
 
   PetscErrorCode ierr = VecAXPY(_x, a, _y._x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecAXPY");
+  CHECK_ERROR("VecAXPY");
 
   // Update ghost values
   update_ghost_values();
@@ -604,7 +552,8 @@ void PETScVector::axpy(double a, const GenericVector& y)
 void PETScVector::abs()
 {
   dolfin_assert(_x);
-  VecAbs(_x);
+  PetscErrorCode ierr = VecAbs(_x);
+  CHECK_ERROR("VecAbs");
 
   // Update ghost values
   update_ghost_values();
@@ -623,7 +572,7 @@ double PETScVector::norm(std::string norm_type) const
   double value = 0.0;
   PetscErrorCode ierr = VecNorm(_x, norm_types.find(norm_type)->second,
                                 &value);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecNorm");
+  CHECK_ERROR("VecNorm");
   return value;
 }
 //-----------------------------------------------------------------------------
@@ -633,7 +582,7 @@ double PETScVector::min() const
   double value = 0.0;
   PetscInt position = 0;
   PetscErrorCode ierr = VecMin(_x, &position, &value);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecMin");
+  CHECK_ERROR("VecMin");
   return value;
 }
 //-----------------------------------------------------------------------------
@@ -643,7 +592,7 @@ double PETScVector::max() const
   double value = 0.0;
   PetscInt position = 0;
   PetscErrorCode ierr = VecMax(_x, &position, &value);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecMax");
+  CHECK_ERROR("VecMax");
   return value;
 }
 //-----------------------------------------------------------------------------
@@ -652,15 +601,16 @@ double PETScVector::sum() const
   dolfin_assert(_x);
   double value = 0.0;
   PetscErrorCode ierr = VecSum(_x, &value);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSum");
+  CHECK_ERROR("VecSum");
   return value;
 }
 //-----------------------------------------------------------------------------
 double PETScVector::sum(const Array<std::size_t>& rows) const
 {
   dolfin_assert(_x);
-  const std::size_t n0 = local_range().first;
-  const std::size_t n1 = local_range().second;
+  const auto _local_range = local_range();
+  const std::size_t n0 = _local_range.first;
+  const std::size_t n1 = _local_range.second;
 
   // Build sets of local and nonlocal entries
   Set<PetscInt> local_rows;
@@ -674,8 +624,8 @@ double PETScVector::sum(const Array<std::size_t>& rows) const
   }
 
   // Send nonlocal rows indices to other processes
-  const std::size_t num_processes  = MPI::size(mpi_comm());
-  const std::size_t process_number = MPI::rank(mpi_comm());
+  const std::size_t num_processes  = dolfin::MPI::size(mpi_comm());
+  const std::size_t process_number = dolfin::MPI::rank(mpi_comm());
   for (std::size_t i = 1; i < num_processes; ++i)
   {
     // Receive data from process p - i (i steps to the left), send
@@ -686,8 +636,8 @@ double PETScVector::sum(const Array<std::size_t>& rows) const
 
     // Send and receive data
     std::vector<std::size_t> received_nonlocal_rows;
-    MPI::send_recv(mpi_comm(), send_nonlocal_rows.set(), dest,
-                   received_nonlocal_rows, source);
+    dolfin::MPI::send_recv(mpi_comm(), send_nonlocal_rows.set(), dest,
+                           received_nonlocal_rows, source);
 
     // Add rows which reside on this process
     for (std::size_t j = 0; j < received_nonlocal_rows.size(); ++j)
@@ -705,48 +655,40 @@ double PETScVector::sum(const Array<std::size_t>& rows) const
   const double local_sum = std::accumulate(local_values.begin(),
                                            local_values.end(), 0.0);
 
-  return MPI::sum(mpi_comm(), local_sum);
+  return dolfin::MPI::sum(mpi_comm(), local_sum);
 }
 //-----------------------------------------------------------------------------
 std::string PETScVector::str(bool verbose) const
 {
-  if (!_x)
-    return "<Uninitialized PETScVector>";
-
+  dolfin_assert(_x);
   PetscErrorCode ierr;
+
+  // Check if vector type has not been set
+  VecType vec_type = nullptr;
+  ierr = VecGetType(_x, &vec_type);
+  if (vec_type == nullptr)
+    return "<Uninitialized PETScVector>";
+  CHECK_ERROR("VecGetType");
+
   std::stringstream s;
   if (verbose)
   {
     // Get vector type
-    #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 3
-    const VecType petsc_type;
-    #else
-    VecType petsc_type;
-    #endif
+    VecType petsc_type = nullptr;
     dolfin_assert(_x);
     ierr = VecGetType(_x, &petsc_type);
-    if (ierr != 0) petsc_error(ierr, __FILE__, "VecGet");
+    CHECK_ERROR("VecGetType");
 
     if (strcmp(petsc_type, VECSEQ) == 0)
     {
       ierr = VecView(_x, PETSC_VIEWER_STDOUT_SELF);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "VecView");
+      CHECK_ERROR("VecView");
     }
     else if (strcmp(petsc_type, VECMPI) == 0)
     {
       ierr = VecView(_x, PETSC_VIEWER_STDOUT_WORLD);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "VecView");
+      CHECK_ERROR("VecView");
     }
-    #ifdef HAS_PETSC_CUSP
-    else if (strcmp(petsc_type, VECSEQCUSP) == 0)
-    {
-      ierr = VecView(*_x, PETSC_VIEWER_STDOUT_SELF);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "VecView");
-    // TODO: Uncomment these two lines after implementing MPI Cusp vectors
-    //else if (strcmp(petsc_type, VECMPICUSP) == 0)
-    //  VecView(*x, PETSC_VIEWER_STDOUT_WORLD);
-    }
-    #endif
   }
   else
     s << "<PETScVector of size " << size() << ">";
@@ -763,27 +705,36 @@ void PETScVector::gather(GenericVector& y,
   // Down cast to a PETScVector
   PETScVector& _y = as_type<PETScVector>(y);
 
-  // Prepare data for index sets (global indices)
-  std::vector<PetscInt> global_indices(indices.begin(), indices.end());
-
-  // Prepare data for index sets (local indices)
+  // Get number of required entries
   const std::size_t n = indices.size();
 
-  if (_y.empty())
-  {
-    // Initialise vector and make local
-    y.init(MPI_COMM_SELF, n);
-  }
-  else if (y.size() != n || MPI::size(y.mpi_comm()))
+  // Check that passed vector is local
+  if (MPI::size(_y.mpi_comm()) != 1)
   {
     dolfin_error("PETScVector.cpp",
                  "gather vector entries",
-                 "Cannot re-initialize gather vector. Must be empty, or have correct size and be a local vector");
+                 "Gather vector must be a local vector (MPI_COMM_SELF)");
   }
 
+  // Initialize vector if empty
+  if (_y.empty())
+    _y.init(n);
 
-  // PETSc will bail out if it receives a NULL pointer even though m == 0.
-  // Can't return from function since function calls are collective.
+  // Check that passed vector has correct size
+  if (_y.size() != n)
+  {
+    dolfin_error("PETScVector.cpp",
+                 "gather vector entries",
+                 "Gather vector must be empty or of correct size "
+                 "(same as provided indices)");
+  }
+
+  // Prepare data for index sets (global indices)
+  std::vector<PetscInt> global_indices(indices.begin(), indices.end());
+
+  // PETSc will bail out if it receives a NULL pointer even though m
+  // == 0.  Can't return from function since function calls are
+  // collective.
   if (n == 0)
     global_indices.resize(1);
 
@@ -791,36 +742,36 @@ void PETScVector::gather(GenericVector& y,
   IS from, to;
   ierr = ISCreateGeneral(PETSC_COMM_SELF, n, global_indices.data(),
                          PETSC_COPY_VALUES, &from);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "ISCreateGeneral");
+  CHECK_ERROR("ISCreateGeneral");
   ierr = ISCreateStride(PETSC_COMM_SELF, n, 0 , 1, &to);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "ISCreateStride");
+  CHECK_ERROR("ISCreateStride");
 
 
   // Perform scatter
   VecScatter scatter;
   ierr = VecScatterCreate(_x, from, _y.vec(), to, &scatter);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterCreate");
+  CHECK_ERROR("VecScatterCreate");
   ierr = VecScatterBegin(scatter, _x, _y.vec(), INSERT_VALUES,
                          SCATTER_FORWARD);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterBegin");
+  CHECK_ERROR("VecScatterBegin");
   ierr = VecScatterEnd(scatter, _x, _y.vec(), INSERT_VALUES,
                        SCATTER_FORWARD);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterEnd");
+  CHECK_ERROR("VecScatterEnd");
 
   // Clean up
   ierr = VecScatterDestroy(&scatter);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterDestroy");
+  CHECK_ERROR("VecScatterDestroy");
   ierr = ISDestroy(&from);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "ISDestroy");
+  CHECK_ERROR("ISDestroy");
   ierr = ISDestroy(&to);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "ISDestroy");
+  CHECK_ERROR("ISDestroy");
 }
 //-----------------------------------------------------------------------------
 void PETScVector::gather(std::vector<double>& x,
                          const std::vector<dolfin::la_index>& indices) const
 {
   x.resize(indices.size());
-  PETScVector y;
+  PETScVector y(PETSC_COMM_SELF);
   gather(y, indices);
   dolfin_assert(y.local_size() == x.size());
   y.get_local(x);
@@ -830,90 +781,76 @@ void PETScVector::gather_on_zero(std::vector<double>& x) const
 {
   PetscErrorCode ierr;
 
-  if (MPI::rank(mpi_comm()) == 0)
+  if (dolfin::MPI::rank(mpi_comm()) == 0)
     x.resize(size());
   else
     x.resize(0);
 
+  dolfin_assert(_x);
   Vec vout;
   VecScatter scatter;
   ierr = VecScatterCreateToZero(_x, &scatter, &vout);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterCreateToZero");
+  CHECK_ERROR("VecScatterCreateToZero");
   ierr = VecScatterBegin(scatter, _x, vout, INSERT_VALUES, SCATTER_FORWARD);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterBegin");
+  CHECK_ERROR("VecScatterBegin");
   ierr = VecScatterEnd(scatter, _x, vout, INSERT_VALUES, SCATTER_FORWARD);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterEnd");
+  CHECK_ERROR("VecScatterEnd");
   ierr = VecScatterDestroy(&scatter);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecScatterDestroy");
+  CHECK_ERROR("VecScatterDestroy");
 
   // Wrap PETSc vector
-  if (MPI::rank(mpi_comm()) == 0)
+  if (dolfin::MPI::rank(mpi_comm()) == 0)
   {
     PETScVector _vout(vout);
     _vout.get_local(x);
   }
 }
 //-----------------------------------------------------------------------------
-void PETScVector::_init(MPI_Comm comm,
-                        std::pair<std::size_t, std::size_t> range,
-                        const std::vector<std::size_t>& local_to_global_map,
-                        const std::vector<la_index>& ghost_indices)
+GenericLinearAlgebraFactory& PETScVector::factory() const
 {
-  PetscErrorCode ierr;
-  if (_x)
-    error("PETScVector cannot be initialized more than once.");
-
-  // GPU support does not work in parallel
-  if (_use_gpu && MPI::size(comm) > 1)
+  return PETScFactory::instance();
+}
+//-----------------------------------------------------------------------------
+void PETScVector::set_options_prefix(std::string options_prefix)
+{
+  if (!_x)
   {
-    not_working_in_parallel("Due to limitations in PETSc, "
-                            "distributed PETSc Cusp vectors");
+    dolfin_error("PETScVector.cpp",
+                 "setting PETSc options prefix",
+                 "Cannot set options prefix since PETSc Vec has not been initialized");
   }
 
-  #ifdef HAS_PETSC_CUSP
-  ierr = VecSetType(_x, VECSEQCUSP);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecSetType");
-  #endif
-
-  const std::size_t local_size = range.second - range.first;
-  dolfin_assert(range.second >= range.first);
-
-  // Copy ghost indices
-  ierr = VecCreateGhost(comm, local_size, PETSC_DECIDE,
-                        ghost_indices.size(), ghost_indices.data(), &_x);
-  if (ierr != 0) petsc_error(ierr, __FILE__, "VecCreateGhost");
-
-  // Build global-to-local map for ghost indices
-  ghost_global_to_local.clear();
-  for (std::size_t i = 0; i < ghost_indices.size(); ++i)
+  // Set PETSc options prefix
+  PetscErrorCode ierr = VecSetOptionsPrefix(_x, options_prefix.c_str());
+  CHECK_ERROR("VecSetOptionsPrefix");
+}
+//-----------------------------------------------------------------------------
+std::string PETScVector::get_options_prefix() const
+{
+  if (!_x)
   {
-    ghost_global_to_local.insert(std::pair<std::size_t,
-                                 std::size_t>(ghost_indices[i], i));
+    dolfin_error("PETScVector.cpp",
+                 "get PETSc options prefix",
+                 "Cannot get options prefix since PETSc Vec has not been initialized");
   }
 
-  ISLocalToGlobalMapping petsc_local_to_global;
-  std::vector<PetscInt> _map;
-  if (!local_to_global_map.empty())
+  const char* prefix = nullptr;
+  PetscErrorCode ierr = VecGetOptionsPrefix(_x, &prefix);
+  CHECK_ERROR("VecGetOptionsPrefix");
+  return std::string(prefix);
+}
+//-----------------------------------------------------------------------------
+void PETScVector::set_from_options()
+{
+  if (!_x)
   {
-    _map = std::vector<PetscInt>(local_to_global_map.begin(),
-                                 local_to_global_map.end());
+    dolfin_error("PETScVector.cpp",
+                 "call VecSetFromOptions on PETSc Vec object",
+                 "Vec object has not been initialized");
   }
-  else
-  {
-    const std::size_t size = range.second - range.first;
-    _map.resize(size);
-    for (std::size_t i = 0; i < size; ++i)
-      _map[i] = i + range.first;
-  }
-  #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 4
-  ISLocalToGlobalMappingCreate(PETSC_COMM_SELF, _map.size(), _map.data(),
-                               PETSC_COPY_VALUES, &petsc_local_to_global);
-  #else
-  ISLocalToGlobalMappingCreate(PETSC_COMM_SELF, 1, _map.size(), _map.data(),
-                               PETSC_COPY_VALUES, &petsc_local_to_global);
-  #endif
-  VecSetLocalToGlobalMapping(_x, petsc_local_to_global);
-  ISLocalToGlobalMappingDestroy(&petsc_local_to_global);
+
+  PetscErrorCode ierr = VecSetFromOptions(_x);
+  CHECK_ERROR("VecSetFromOptions");
 }
 //-----------------------------------------------------------------------------
 Vec PETScVector::vec() const
@@ -921,17 +858,95 @@ Vec PETScVector::vec() const
   return _x;
 }
 //-----------------------------------------------------------------------------
-GenericLinearAlgebraFactory& PETScVector::factory() const
+void PETScVector::reset(Vec vec)
 {
-  if (!_use_gpu)
-    return PETScFactory::instance();
-  #ifdef HAS_PETSC_CUSP
-  else
-    return PETScCuspFactory::instance();
-  #endif
+  dolfin_assert(_x);
+  PetscErrorCode ierr;
 
-  // Return something to keep the compiler happy. Code will never be reached.
-  return PETScFactory::instance();
+  // Decrease reference count to old Vec object
+  ierr = VecDestroy(&_x);
+  CHECK_ERROR("VecDestroy");
+
+  // Store new Vec object and increment reference count
+  _x = vec;
+  ierr = PetscObjectReference((PetscObject)_x);
+  CHECK_ERROR("PetscObjectReference");
 }
 //-----------------------------------------------------------------------------
+void PETScVector::_init(std::pair<std::size_t, std::size_t> range,
+                        const std::vector<std::size_t>& local_to_global_map,
+                        const std::vector<la_index>& ghost_indices)
+{
+  if (!_x)
+  {
+    dolfin_error("PETScVector.h",
+                 "initialize vector",
+                 "Underlying PETSc Vec has not been initialized");
+  }
+
+  PetscErrorCode ierr;
+
+  // Set from PETSc options. This will set the vector type.
+  ierr = VecSetFromOptions(_x);
+  CHECK_ERROR("VecSetFromOptions");
+
+  // Get local size
+  const std::size_t local_size = range.second - range.first;
+  dolfin_assert(range.second >= range.first);
+
+  // Set vector size
+  ierr = VecSetSizes(_x, local_size, PETSC_DECIDE);
+  CHECK_ERROR("VecSetSizes");
+
+  // Get PETSc Vec type
+  VecType vec_type = nullptr;
+  ierr = VecGetType(_x, &vec_type);
+  CHECK_ERROR("VecGetType");
+
+  // Add ghost points if Vec type is MPI (throw an error if Vec is not
+  // VECMPI and ghost entry vector is not empty)
+  if (strcmp(vec_type, VECMPI) == 0)
+  {
+    ierr = VecMPISetGhost(_x, ghost_indices.size(), ghost_indices.data());
+    CHECK_ERROR("VecMPISetGhost");
+  }
+  else if (!ghost_indices.empty())
+  {
+    dolfin_error("PETScVector.cpp",
+                 "initialize vector",
+                 "Sequential PETSc Vec objects cannot have ghost entries");
+  }
+
+  // Build local-to-global map
+  std::vector<PetscInt> _map;
+  if (!local_to_global_map.empty())
+  {
+    // Copy data to get correct PETSc integer type
+    _map = std::vector<PetscInt>(local_to_global_map.begin(),
+                                 local_to_global_map.end());
+  }
+  else
+  {
+    // Fill vector with [i0 + 0, i0 + 1, i0 +2, . . .]
+    const std::size_t size = range.second - range.first;
+    _map.assign(size, range.first);
+    std::iota(_map.begin(), _map.end(), range.first);
+  }
+
+  // Create PETSc local-to-global map
+  ISLocalToGlobalMapping petsc_local_to_global;
+  ierr = ISLocalToGlobalMappingCreate(PETSC_COMM_SELF, 1, _map.size(), _map.data(),
+                               PETSC_COPY_VALUES, &petsc_local_to_global);
+  CHECK_ERROR("ISLocalToGlobalMappingCreate");
+
+  // Apply local-to-global map to vector
+  ierr = VecSetLocalToGlobalMapping(_x, petsc_local_to_global);
+  CHECK_ERROR("VecSetLocalToGlobalMapping");
+
+  // Clean-up PETSc local-to-global map
+  ierr = ISLocalToGlobalMappingDestroy(&petsc_local_to_global);
+  CHECK_ERROR("ISLocalToGlobalMappingDestroy");
+}
+//-----------------------------------------------------------------------------
+
 #endif
