@@ -48,7 +48,8 @@ namespace dolfin
   class SparsityPattern;
   template<typename T> class Array;
 
-  /// This class provides a simple vector class based on PETSc.
+  /// A simple vector class based on PETSc.
+  ///
   /// It is a simple wrapper for a PETSc vector pointer (Vec)
   /// implementing the GenericVector interface.
   ///
@@ -103,16 +104,14 @@ namespace dolfin
     virtual std::shared_ptr<GenericVector> copy() const;
 
     /// Initialize vector to global size N
-    virtual void init(MPI_Comm comm, std::size_t N);
+    virtual void init(std::size_t N);
 
     /// Initialize vector with given ownership range
-    virtual void init(MPI_Comm comm,
-                      std::pair<std::size_t, std::size_t> range);
+    virtual void init(std::pair<std::size_t, std::size_t> range);
 
     /// Initialize vector with given ownership range and with ghost
     /// values
-    virtual void init(MPI_Comm comm,
-                      std::pair<std::size_t, std::size_t> range,
+    virtual void init(std::pair<std::size_t, std::size_t> range,
                       const std::vector<std::size_t>& local_to_global_map,
                       const std::vector<la_index>& ghost_indices);
 
@@ -168,15 +167,20 @@ namespace dolfin
     /// Add values to each entry on local process
     virtual void add_local(const Array<double>& values);
 
-    /// Gather vector entries into a local vector
+    /// Gather entries (given by global indices) into local
+    /// (MPI_COMM_SELF) vector x. Provided x must be empty
+    /// or of correct dimension (same as provided indices).
+    /// This operation is collective
     virtual void gather(GenericVector& y,
                         const std::vector<dolfin::la_index>& indices) const;
 
-    /// Gather entries into x
+    /// Gather entries (given by global indices) into x.
+    /// This operation is collective
     virtual void gather(std::vector<double>& x,
                         const std::vector<dolfin::la_index>& indices) const;
 
-    /// Gather all entries into x on process 0
+    /// Gather all entries into x on process 0.
+    /// This operation is collective
     virtual void gather_on_zero(std::vector<double>& x) const;
 
     /// Add multiple of given vector (AXPY operation)
@@ -230,6 +234,7 @@ namespace dolfin
     /// Assignment operator
     virtual const PETScVector& operator= (double a);
 
+    /// Update values shared from remote processes
     virtual void update_ghost_values();
 
     //--- Special functions ---
