@@ -21,7 +21,6 @@
 // Last changed: 2011-02-08
 
 #include <cmath>
-#include <dolfin/common/types.h>
 #include "Vertex.h"
 #include "Edge.h"
 
@@ -30,7 +29,7 @@ using namespace dolfin;
 //-----------------------------------------------------------------------------
 double Edge::length() const
 {
-  const uint* vertices = entities(0);
+  const unsigned int* vertices = entities(0);
   dolfin_assert(vertices);
 
   const Vertex v0(*_mesh, vertices[0]);
@@ -48,8 +47,8 @@ double Edge::length() const
 //-----------------------------------------------------------------------------
 double Edge::dot(const Edge& edge) const
 {
-  const uint* v0 = entities(0);
-  const uint* v1 = edge.entities(0);
+  const unsigned int* v0 = entities(0);
+  const unsigned int* v1 = edge.entities(0);
   dolfin_assert(v0);
   dolfin_assert(v1);
 
@@ -60,8 +59,8 @@ double Edge::dot(const Edge& edge) const
   const double* x11 = g.x(v1[1]);
 
   double sum = 0.0;
-  const uint gdim = g.dim();
-  for (uint i = 0; i < gdim; i++)
+  const std::size_t gdim = g.dim();
+  for (std::size_t i = 0; i < gdim; i++)
     sum += (x01[i] - x00[i]) * (x11[i] - x10[i]);
 
   return sum;
