@@ -66,11 +66,21 @@ namespace dolfin
   {
   public:
 
+    /// Constructor on Mesh
     FaceFunction(std::shared_ptr<const Mesh> mesh)
-      : MeshFunction<T>(mesh, 2) {}
+      : MeshFunction<T>(mesh, 2) {
+        deprecation("FaceFunction<T>(mesh)",
+                    "2017.2.0",
+                    "Use MeshFunction<T>(mesh, 2)");
+      }
 
+    /// Constructor on Mesh and value
     FaceFunction(std::shared_ptr<const Mesh> mesh, const T& value)
-      : MeshFunction<T>(mesh, 2, value) {}
+      : MeshFunction<T>(mesh, 2, value) {
+        deprecation("FaceFunction<T>(mesh, value)",
+                    "2017.2.0",
+                    "Use MeshFunction<T>(mesh, 2, value)");
+      }
 
   };
 
