@@ -110,12 +110,12 @@ void HexahedronCell::create_entities(boost::multi_array<unsigned int, 2>& e,
     e.resize(boost::extents[6][4]);
 
     // Create the 6 faces
-    e[0][0] = v[0]; e[0][1] = v[2]; e[0][2] = v[4]; e[0][3] = v[6];
-    e[1][0] = v[1]; e[1][1] = v[3]; e[1][2] = v[5]; e[1][3] = v[7];
+    e[0][0] = v[0]; e[0][1] = v[1]; e[0][2] = v[2]; e[0][3] = v[3];
+    e[1][0] = v[4]; e[1][1] = v[5]; e[1][2] = v[6]; e[1][3] = v[7];
     e[2][0] = v[0]; e[2][1] = v[1]; e[2][2] = v[4]; e[2][3] = v[5];
     e[3][0] = v[2]; e[3][1] = v[3]; e[3][2] = v[6]; e[3][3] = v[7];
-    e[4][0] = v[0]; e[4][1] = v[1]; e[4][2] = v[2]; e[4][3] = v[3];
-    e[5][0] = v[4]; e[5][1] = v[5]; e[5][2] = v[6]; e[5][3] = v[7];
+    e[4][0] = v[0]; e[4][1] = v[2]; e[4][2] = v[4]; e[4][3] = v[6];
+    e[5][0] = v[1]; e[5][1] = v[3]; e[5][2] = v[5]; e[5][3] = v[7];
     break;
   default:
     dolfin_error("HexahedronCell.cpp",
@@ -241,13 +241,6 @@ bool HexahedronCell::collides(const Cell& cell, const MeshEntity& entity) const
   return false;
 }
 //-----------------------------------------------------------------------------
-std::vector<double>
-HexahedronCell::triangulate_intersection(const Cell& c0, const Cell& c1) const
-{
-  dolfin_not_implemented();
-  return std::vector<double>();
-}
-//-----------------------------------------------------------------------------
 std::string HexahedronCell::description(bool plural) const
 {
   if (plural)
