@@ -1,4 +1,4 @@
-// Copyright (C) 2006-2013 Anders Logg
+// Copyright (C) 2006-2017 Anders Logg
 //
 // This file is part of DOLFIN.
 //
@@ -20,7 +20,7 @@
 // Modified by Kristoffer Selim, 2008.
 //
 // First added:  2006-06-05
-// Last changed: 2014-01-31
+// Last changed: 2017-09-26
 
 #ifndef __TETRAHEDRON_CELL_H
 #define __TETRAHEDRON_CELL_H
@@ -35,14 +35,18 @@ namespace dolfin
 
   class Cell;
 
-  /// This class implements functionality for tetrahedral meshes.
+  /// This class implements functionality for tetrahedral cell meshes.
 
   class TetrahedronCell : public CellType
   {
   public:
 
     /// Specify cell type and facet type
-    TetrahedronCell() : CellType(tetrahedron, triangle) {}
+    TetrahedronCell() : CellType(Type::tetrahedron, Type::triangle) {}
+
+    /// Check if cell is a simplex
+    bool is_simplex() const
+    { return true; }
 
     /// Return topological dimension of cell
     std::size_t dim() const;
@@ -92,10 +96,6 @@ namespace dolfin
     /// Check whether given entity collides with cell
     bool collides(const Cell& cell, const MeshEntity& entity) const;
 
-    /// Compute triangulation of intersection of two cells
-    virtual std::vector<double>
-    triangulate_intersection(const Cell& c0, const Cell& c1) const;
-
     /// Return description of cell type
     std::string description(bool plural) const;
 
