@@ -69,16 +69,21 @@ namespace dolfin
   {
   public:
 
-    VertexFunction(const Mesh& mesh) : MeshFunction<T>(mesh, 0) {}
-
+    /// Constructor on Mesh
     VertexFunction(std::shared_ptr<const Mesh> mesh)
-      : MeshFunction<T>(mesh, 0) {}
-
-    VertexFunction(const Mesh& mesh, const T& value)
-      : MeshFunction<T>(mesh, 0, value) {}
+      : MeshFunction<T>(mesh, 0) {
+        deprecation("VertexFunction<T>(mesh)",
+                    "2017.2.0",
+                    "Use MeshFunction<T>(mesh, 0)");
+      }
 
+    /// Constructor on Mesh and value
     VertexFunction(std::shared_ptr<const Mesh> mesh, const T& value)
-      : MeshFunction<T>(mesh, 0, value) {}
+      : MeshFunction<T>(mesh, 0, value) {
+        deprecation("VertexFunction<T>(mesh, value)",
+                    "2017.2.0",
+                    "Use MeshFunction<T>(mesh, 0, value)");
+      }
 
   };
 
