@@ -84,7 +84,6 @@ Parameters PETScSNESSolver::default_parameters()
   p.add("maximum_residual_evaluations", 2000);
   p.remove("convergence_criterion");
   p.remove("relaxation_parameter");
-  p.remove("method");
   p.add("method", "default");
   p.add("line_search", "basic",  {"basic", "bt", "l2", "cp", "nleqerr"});
   p.add("sign", "default", {"default", "nonnegative", "nonpositive"});
@@ -92,41 +91,41 @@ Parameters PETScSNESSolver::default_parameters()
   return p;
 }
 //-----------------------------------------------------------------------------
-PETScSNESSolver::PETScSNESSolver(MPI_Comm comm) : _snes(nullptr),
-                                                  _has_explicit_bounds(false)
+PETScSNESSolver::PETScSNESSolver(MPI_Comm comm, std::string nls_type)
+  : _snes(nullptr), _matJ(comm), _matP(comm), _has_explicit_bounds(false)
 {
+  PetscErrorCode ierr;
+
   // Create SNES object
-  SNESCreate(comm, &_snes);
+  ierr = SNESCreate(comm, &_snes);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESCreate");
+
+  // Set type if not default
+  if (nls_type != "default")
+  {
+    const auto method = _methods.find(nls_type);
+    if (method == _methods.end())
+    {
+      dolfin_error("PETScSNESSolver.cpp",
+                   "create PETSc SNES solver",
+                   "Unknown SNES method \"%s\"", nls_type.c_str());
+    }
+    ierr = SNESSetType(_snes, method->second.second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetType");
+  }
+
+  // Set Jacobian and preconditioner only once
+  ierr = SNESSetJacobian(_snes, _matJ.mat(), _matP.mat(), nullptr, nullptr);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetJacobian");
 
   // Set parameter values
   parameters = default_parameters();
 }
 //-----------------------------------------------------------------------------
-PETScSNESSolver::PETScSNESSolver(std::string nls_type) : _snes(nullptr),
-                                                         _has_explicit_bounds(false)
+PETScSNESSolver::PETScSNESSolver(std::string nls_type)
+  : PETScSNESSolver(MPI_COMM_WORLD, nls_type)
 {
-  // Create SNES object
-  SNESCreate(MPI_COMM_WORLD, &_snes);
-
-  const std::map<std::string, std::pair<std::string, const SNESType>>::const_iterator
-    method = _methods.find(nls_type);
-
-  // Check that the requested method is known
-  if (method != _methods.end())
-  {
-    // Set solver type
-    if (method->first != "default")
-      SNESSetType(_snes, method->second.second);
-  }
-  else
-  {
-    dolfin_error("PETScSNESSolver.cpp",
-                 "create PETSc SNES solver",
-                 "Unknown SNES method \"%s\"", nls_type.c_str());
-  }
-
-  // Set parameter values
-  parameters = default_parameters();
+  // Do nothing
 }
 //-----------------------------------------------------------------------------
 PETScSNESSolver::~PETScSNESSolver()
@@ -141,9 +140,6 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
                        const GenericVector& lb,
                        const GenericVector& ub)
 {
-  // Set linear solver parameters
-  set_linear_solver_parameters();
-
   // Check size of the bound vectors
   if (lb.size() != ub.size())
   {
@@ -160,9 +156,9 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
 
   // Set the bounds
   std::shared_ptr<const PETScVector>
-    _ub(&ub.down_cast<PETScVector>(), NoDeleter());
+    _ub(&as_type<const PETScVector>(ub), NoDeleter());
   std::shared_ptr<const PETScVector>
-    _lb(&lb.down_cast<PETScVector>(), NoDeleter());
+    _lb(&as_type<const PETScVector>(lb), NoDeleter());
   this->lb = _lb;
   this->ub = _ub;
   _has_explicit_bounds = true;
@@ -174,39 +170,35 @@ void PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
                            GenericVector& x)
 {
   Timer timer("SNES solver init");
-  PETScMatrix A(this->mpi_comm());
+  PetscErrorCode ierr;
 
   // Set linear solver parameters
   set_linear_solver_parameters();
 
+  // Prepare context for evaluation routines
   _snes_ctx.nonlinear_problem = &nonlinear_problem;
-  _snes_ctx.x = &x.down_cast<PETScVector>();
-  VecDuplicate(_snes_ctx.x->vec(), &_snes_ctx.f_tmp);
+  _snes_ctx.x = &as_type<PETScVector>(x);
+  // FIXME: We are duplicating ghosted vector, while we don't need ghosted
+  // NOTE: Seems that we can get rid of f_tmp and use working vec obtained by
+  //       SNESLineSearchGetVecs
+  ierr = VecDuplicate(_snes_ctx.x->vec(), &_snes_ctx.f_tmp);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "VecDuplicate");
 
-  // Compute F(u)
-  PETScVector f(_snes_ctx.f_tmp);
-  nonlinear_problem.form(A, f, x);
-  nonlinear_problem.F(f, x);
-  nonlinear_problem.J(A, x);
-
-  SNESSetFunction(_snes, _snes_ctx.f_tmp, PETScSNESSolver::FormFunction,
-                  &_snes_ctx);
-  SNESSetJacobian(_snes, A.mat(), A.mat(), PETScSNESSolver::FormJacobian,
-                  &_snes_ctx);
+  // Set function, Jacobian and objective
+  ierr = SNESSetFunction(_snes, _snes_ctx.f_tmp,
+                         PETScSNESSolver::FormFunction, &_snes_ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetFunction");
+  ierr = SNESSetJacobian(_snes, nullptr, nullptr,
+                         PETScSNESSolver::FormJacobian, &_snes_ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetJacobian");
   SNESSetObjective(_snes, PETScSNESSolver::FormObjective, &_snes_ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetObjective");
 
   // Set some options from the parameters
   if (parameters["report"].is_set())
   {
     if (parameters["report"])
     {
-      PetscErrorCode ierr;
-      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
-      ierr = SNESMonitorSet(_snes, SNESMonitorDefault,
-                            PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_snes)),
-                            NULL);
-      if (ierr != 0) petsc_error(ierr, __FILE__, "SNESMonitorSet");
-      #else
       PetscViewer viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)_snes));
       PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
       PetscViewerAndFormat *vf;
@@ -216,7 +208,6 @@ void PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
                             vf,
                             (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
       if (ierr != 0) petsc_error(ierr, __FILE__, "SNESMonitorSet");
-      #endif
     }
   }
 
@@ -229,7 +220,8 @@ void PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
   {
     auto it = _methods.find(method);
     dolfin_assert(it != _methods.end());
-    SNESSetType(_snes, it->second.second);
+    ierr = SNESSetType(_snes, it->second.second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetType");
 
     // Check if bounds/sign are set when VI method requested
     if ((method == "vinewtonrsls" || method == "vinewtonssls") && !is_vi())
@@ -249,37 +241,45 @@ void PETScSNESSolver::init(NonlinearProblem& nonlinear_problem,
     // (most methods do not support bounds)
     auto it = _methods.find("vinewtonssls");
     dolfin_assert(it != _methods.end());
-    SNESSetType(_snes, it->second.second);
+    ierr = SNESSetType(_snes, it->second.second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetType");
   }
 
   SNESLineSearch linesearch;
-  SNESGetLineSearch(_snes, &linesearch);
+  ierr = SNESGetLineSearch(_snes, &linesearch);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESGetLineSearch");
 
-  if (parameters["report"].is_set())
+  if (parameters["report"].is_set() && parameters["report"])
   {
-    if (parameters["report"])
-    {
-      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
-      SNESLineSearchSetMonitor(linesearch, PETSC_TRUE);
-      #else
-      SNESLineSearchMonitor(linesearch);
-      #endif
-    }
+    ierr = SNESLineSearchMonitor(linesearch);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESLineSearchMonitor");
   }
 
   const std::string line_search_type = std::string(parameters["line_search"]);
-  SNESLineSearchSetType(linesearch, line_search_type.c_str());
+  ierr = SNESLineSearchSetType(linesearch, line_search_type.c_str());
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESLineSearchSetType");
 
   // Tolerances
   const int max_iters = parameters["maximum_iterations"];
   const int max_residual_evals = parameters["maximum_residual_evaluations"];
-  SNESSetTolerances(_snes, parameters["absolute_tolerance"],
-                    parameters["relative_tolerance"],
-                    parameters["solution_tolerance"],
-                    max_iters, max_residual_evals);
+  ierr = SNESSetTolerances(_snes, parameters["absolute_tolerance"],
+                           parameters["relative_tolerance"],
+                           parameters["solution_tolerance"],
+                           max_iters, max_residual_evals);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetTolerances");
+
+  // Following set from options call requires Mat type to be set (at least
+  // when PETSC_USE_DEBUG) and we don't have any better way
+  if (_matJ.empty())
+  {
+    ierr = FormJacobian(_snes, _snes_ctx.x->vec(),
+                        _matJ.mat(), _matP.mat(), &_snes_ctx);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "PETScSNESSolver::FormJacobian");
+  }
 
   // Set some options
-  SNESSetFromOptions(_snes);
+  ierr = SNESSetFromOptions(_snes);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetFromOptions");
 }
 //-----------------------------------------------------------------------------
 void PETScSNESSolver::set_options_prefix(std::string options_prefix)
@@ -319,27 +319,33 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
                        GenericVector& x)
 {
   Timer timer("SNES solver execution");
-  PETScVector f;
-  PETScMatrix A;
+  PetscErrorCode ierr;
   PetscInt its;
   SNESConvergedReason reason;
 
+  PETScVector& _x = as_type<PETScVector>(x);
+
   this->init(nonlinear_problem, x);
+
   // for line searches, making this copy is necessary:
   // when linesearching, the working space can't be the
   // same as the vector that holds the current solution
   // guess in the dolfin form.
-  PETScVector x_copy(x.down_cast<PETScVector>());
-  SNESSolve(_snes, NULL, x_copy.vec());
-  x.zero();
-  x.axpy(1.0, x_copy);
+  {
+    PETScVector x_copy(_x);
+    ierr = SNESSolve(_snes, NULL, x_copy.vec());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSolve");
+    ierr = VecSwap(_x.vec(), x_copy.vec());
+    if (ierr != 0) petsc_error(ierr, __FILE__, "VecSwap");
+  }
 
   // Update any ghost values
-  PETScVector& _x = x.down_cast<PETScVector>();
   _x.update_ghost_values();
 
-  SNESGetIterationNumber(_snes, &its);
-  SNESGetConvergedReason(_snes, &reason);
+  ierr = SNESGetIterationNumber(_snes, &its);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESGetIterationNumber");
+  ierr = SNESGetConvergedReason(_snes, &reason);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "SNESGetConvergedReason");
 
   const bool report = parameters["report"];
 
@@ -359,8 +365,7 @@ PETScSNESSolver::solve(NonlinearProblem& nonlinear_problem,
   {
     dolfin_error("PETScSNESSolver.cpp",
                  "solve nonlinear system with PETScSNESSolver",
-                 "Solver did not converge",
-                 "Bummer");
+                 "Solver did not converge");
   }
 
   return std::make_pair(its, reason > 0);
@@ -387,7 +392,8 @@ PetscErrorCode PETScSNESSolver::FormFunction(SNES snes, Vec x, Vec f, void* ctx)
 
   // Compute F(u)
   PETScMatrix A(_x->mpi_comm());
-  nonlinear_problem->form(A, f_wrap, *_x);
+  PETScMatrix P(_x->mpi_comm());
+  nonlinear_problem->form(A, P, f_wrap, *_x);
   nonlinear_problem->F(f_wrap, *_x);
 
   return 0;
@@ -396,10 +402,15 @@ PetscErrorCode PETScSNESSolver::FormFunction(SNES snes, Vec x, Vec f, void* ctx)
 PetscErrorCode PETScSNESSolver::FormObjective(SNES snes, Vec x,
                                               PetscReal* out, void* ctx)
 {
+  PetscErrorCode ierr;
   auto snes_ctx = static_cast<struct snes_ctx_t*>(ctx);
-  PETScSNESSolver::FormFunction(snes, x, snes_ctx->f_tmp, ctx);
+
+  ierr = PETScSNESSolver::FormFunction(snes, x, snes_ctx->f_tmp, ctx);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "PETScSNESSolver::FormFunction");
+
   PetscReal f_norm;
-  VecNorm(snes_ctx->f_tmp, NORM_2, &f_norm);
+  ierr = VecNorm(snes_ctx->f_tmp, NORM_2, &f_norm);
+  if (ierr != 0) petsc_error(ierr, __FILE__, "VecNorm");
 
   if (std::isnan(f_norm) || std::isinf(f_norm))
     *out = 1.0e100;
@@ -412,27 +423,29 @@ PetscErrorCode PETScSNESSolver::FormObjective(SNES snes, Vec x,
 PetscErrorCode PETScSNESSolver::FormJacobian(SNES snes, Vec x, Mat A, Mat P,
                                              void* ctx)
 {
-  // Interface does not presently support a preconditioner that
-  // differs from operator A
-  if (A != P)
-  {
-    dolfin_error("PETScSNESSolver.cpp",
-                 "for Jacobian",
-                 "Matrix object incompatibility. The Jacobian matrix must not be reset when using PETSc SNES.");
-  }
-
   // Get nonlinear problem object
   auto snes_ctx = static_cast<struct snes_ctx_t*>(ctx);
   NonlinearProblem* nonlinear_problem = snes_ctx->nonlinear_problem;
 
   // Wrap the PETSc objects
-  PETScMatrix A_wrap(P);
+  PETScMatrix A_wrap(A);
+  PETScMatrix P_wrap(P);
   PETScVector x_wrap(x);
 
   // Form Jacobian
   PETScVector f(x_wrap.mpi_comm());
-  nonlinear_problem->form(A_wrap, f, x_wrap);
+  nonlinear_problem->form(A_wrap, P_wrap, f, x_wrap);
   nonlinear_problem->J(A_wrap, x_wrap);
+  if (A != P)
+    nonlinear_problem->J_pc(P_wrap, x_wrap);
+
+  // Use Jacobian as preconditioner if not provided
+  if (P_wrap.empty())
+  {
+    log(TRACE, "SNES FormJacobian: using Jacobian as preconditioner matrix");
+    PetscErrorCode ierr = SNESSetJacobian(snes, nullptr, A, nullptr, nullptr);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "SNESSetJacobian");
+  }
 
   return 0;
 }
@@ -441,8 +454,8 @@ void PETScSNESSolver::set_linear_solver_parameters()
 {
   KSP ksp;
   PC pc;
-
   PetscErrorCode ierr;
+
   ierr = SNESGetKSP(_snes, &ksp);
   if (ierr != 0) petsc_error(ierr, __FILE__, "SNESGetKSP");
 
@@ -452,24 +465,6 @@ void PETScSNESSolver::set_linear_solver_parameters()
   // Get MPI communicator
   MPI_Comm comm = mpi_comm();
 
-  if (parameters["report"].is_set())
-  {
-    if (parameters["report"])
-    {
-      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
-      KSPMonitorSet(ksp, KSPMonitorDefault, PETSC_VIEWER_STDOUT_(comm), NULL);
-      #else
-      PetscViewer viewer = PETSC_VIEWER_STDOUT_(comm);
-      PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
-      PetscViewerAndFormat *vf;
-      ierr = PetscViewerAndFormatCreate(viewer,format,&vf);
-      ierr = KSPMonitorSet(ksp,
-                           (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorDefault,
-                           vf,
-                           (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
-      #endif
-    }
-  }
   const std::string linear_solver  = parameters["linear_solver"];
   const std::string preconditioner = parameters["preconditioner"];
 
@@ -481,13 +476,15 @@ void PETScSNESSolver::set_linear_solver_parameters()
   {
     auto  solver_pair = PETScKrylovSolver::_methods.find(linear_solver);
     dolfin_assert(solver_pair != PETScKrylovSolver::_methods.end());
-    KSPSetType(ksp, solver_pair->second);
+    ierr = KSPSetType(ksp, solver_pair->second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
     if (preconditioner != "default"
         && PETScPreconditioner::_methods.count(preconditioner) != 0)
     {
       auto it = PETScPreconditioner::_methods.find(preconditioner);
       dolfin_assert(it != PETScPreconditioner::_methods.end());
-      PCSetType(pc, it->second);
+      ierr = PCSetType(pc, it->second);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "PCSetType");
     }
 
     Parameters krylov_parameters = parameters("krylov_solver");
@@ -497,9 +494,10 @@ void PETScSNESSolver::set_linear_solver_parameters()
     {
       const bool nonzero_guess = krylov_parameters["nonzero_initial_guess"];
       if (nonzero_guess)
-        KSPSetInitialGuessNonzero(ksp, PETSC_TRUE);
+        ierr = KSPSetInitialGuessNonzero(ksp, PETSC_TRUE);
       else
-        KSPSetInitialGuessNonzero(ksp, PETSC_FALSE);
+        ierr = KSPSetInitialGuessNonzero(ksp, PETSC_FALSE);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetInitialGuessNonzero");
     }
     // ... otherwise use the default, what has been set before,
     // or directly through PETSc API or parameter system
@@ -507,21 +505,16 @@ void PETScSNESSolver::set_linear_solver_parameters()
     if (krylov_parameters["monitor_convergence"].is_set()
         && krylov_parameters["monitor_convergence"])
     {
-      #if PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR <= 6 && PETSC_VERSION_RELEASE == 1
-      KSPMonitorSet(ksp, KSPMonitorTrueResidualNorm,
-                    PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp)),
-                    NULL);
-
-      #else
       PetscViewer viewer = PETSC_VIEWER_STDOUT_(PetscObjectComm((PetscObject)ksp));
       PetscViewerFormat format = PETSC_VIEWER_DEFAULT;
       PetscViewerAndFormat *vf;
       ierr = PetscViewerAndFormatCreate(viewer,format,&vf);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "PetscViewerAndFormatCreate");
       ierr = KSPMonitorSet(ksp,
-                         (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorTrueResidualNorm,
-                         vf,
-                         (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
-      #endif
+                           (PetscErrorCode (*)(KSP,PetscInt,PetscReal,void*)) KSPMonitorTrueResidualNorm,
+                           vf,
+                           (PetscErrorCode (*)(void**))PetscViewerAndFormatDestroy);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "KSPMonitorSet");
     }
 
     // Check if a tolerance has been set
@@ -540,11 +533,11 @@ void PETScSNESSolver::set_linear_solver_parameters()
     }
   }
   else if (linear_solver == "lu"
-           || PETScLUSolver::_methods.count(linear_solver) != 0)
+           || PETScLUSolver::lumethods.count(linear_solver) != 0)
   {
     std::string lu_method;
-    if (PETScLUSolver::_methods.find(linear_solver)
-        != PETScLUSolver::_methods.end())
+    if (PETScLUSolver::lumethods.find(linear_solver)
+        != PETScLUSolver::lumethods.end())
     {
       lu_method = linear_solver;
     }
@@ -581,11 +574,14 @@ void PETScSNESSolver::set_linear_solver_parameters()
       }
     }
 
-    KSPSetType(ksp, KSPPREONLY);
-    PCSetType(pc, PCLU);
-    auto it = PETScLUSolver::_methods.find(lu_method);
-    dolfin_assert(it != PETScLUSolver::_methods.end());
-    PCFactorSetMatSolverPackage(pc, it->second);
+    ierr = KSPSetType(ksp, KSPPREONLY);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "KSPSetType");
+    ierr = PCSetType(pc, PCLU);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "PCSetType");
+    auto it = PETScLUSolver::lumethods.find(lu_method);
+    dolfin_assert(it != PETScLUSolver::lumethods.end());
+    ierr = PCFactorSetMatSolverPackage(pc, it->second);
+    if (ierr != 0) petsc_error(ierr, __FILE__, "PCFactorSetMatSolverPackage");
   }
   else
   {
@@ -599,6 +595,8 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
 {
   if (is_vi())
   {
+    PetscErrorCode ierr;
+
     dolfin_assert(_snes);
     const std::string sign   = parameters["sign"];
     const std::string method = parameters["method"];
@@ -615,18 +613,24 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
       // tell PETSc the bounds.
       Vec ub, lb;
 
-      PETScVector _x = x.down_cast<PETScVector>();
-      VecDuplicate(_x.vec(), &ub);
-      VecDuplicate(_x.vec(), &lb);
+      PETScVector _x = as_type<PETScVector>(x);
+      ierr = VecDuplicate(_x.vec(), &ub);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "VecDuplicate");
+      ierr = VecDuplicate(_x.vec(), &lb);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "VecDuplicate");
       if (sign == "nonnegative")
       {
-        VecSet(ub, PETSC_INFINITY);
-        VecSet(lb, 0.0);
+        ierr = VecSet(ub, PETSC_INFINITY);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
+        ierr = VecSet(lb, 0.0);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
       }
       else if (sign == "nonpositive")
       {
-        VecSet(ub, 0.0);
-        VecSet(lb, PETSC_INFINITY);
+        ierr = VecSet(ub, 0.0);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
+        ierr = VecSet(lb, PETSC_INFINITY);
+        if (ierr != 0) petsc_error(ierr, __FILE__, "VecSet");
       }
       else
       {
@@ -635,15 +639,19 @@ void PETScSNESSolver::set_bounds(GenericVector& x)
                      "Unknown bound type \"%s\"", sign.c_str());
       }
 
-      SNESVISetVariableBounds(_snes, lb, ub);
-      VecDestroy(&ub);
-      VecDestroy(&lb);
+      ierr = SNESVISetVariableBounds(_snes, lb, ub);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "SNESVISetVariableBounds");
+      ierr = VecDestroy(&ub);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "VecDestroy");
+      ierr = VecDestroy(&lb);
+      if (ierr != 0) petsc_error(ierr, __FILE__, "VecDestroy");
     }
     else if (_has_explicit_bounds)
     {
       const PETScVector* lb = this->lb.get();
       const PETScVector* ub = this->ub.get();
-      SNESVISetVariableBounds(_snes, lb->vec(), ub->vec());
+      ierr = SNESVISetVariableBounds(_snes, lb->vec(), ub->vec());
+      if (ierr != 0) petsc_error(ierr, __FILE__, "SNESVISetVariableBounds");
     }
   }
 }
