@@ -1,9 +1,10 @@
+# -*- coding: utf-8 -*-
 """Convenience functions that are useful in subdomains and expressions. Matches
 some of the functions in dolfin/math/basic.h."""
 
 __all__ = ["sqr"]
 
-from constants import DOLFIN_EPS
+from . constants import DOLFIN_EPS
 
 # The cpp version of these functions are about 3 times faster
 #def between(x0, x, x1):
