@@ -24,7 +24,6 @@
 
 from __future__ import print_function
 import os
-from distutils.version import StrictVersion
 
 import dolfin
 import dolfin.cpp as cpp
@@ -34,12 +33,14 @@ import numpy as np
 __all__ = ['plot']
 
 _meshfunction_types = (cpp.MeshFunction, cpp.MeshFunctionBool, cpp.MeshFunctionInt,
-                      cpp.MeshFunctionDouble, cpp.MeshFunctionSizet)
-_plottable_types = (cpp.Function,
-                    cpp.Expression,
-                    cpp.Mesh,
-                    cpp.MultiMesh,
-                    cpp.DirichletBC) + _meshfunction_types
+                       cpp.MeshFunctionDouble, cpp.MeshFunctionSizet)
+_matplotlib_plottable_types = (cpp.Function,
+                               cpp.Expression,
+                               cpp.Mesh,
+                               cpp.DirichletBC) + _meshfunction_types
+_x3dom_plottable_types = (cpp.Function,
+                          cpp.Mesh)
+_all_plottable_types = tuple(set.union(set(_matplotlib_plottable_types), set(_x3dom_plottable_types)))
 
 def _has_matplotlib():
     try:
@@ -47,34 +48,15 @@ def _has_matplotlib():
     except ImportError:
         return False
 
-    # Switch to Agg backend if DISPLAY not set
-    if not os.environ.get("DISPLAY"):
-        cpp.log(cpp.PROGRESS, "Environment variable DISPLAY not set. Switching "
-                              "to 'Agg' matplotlib backend.")
-        try:
-            matplotlib.use("Agg")
-        except ValueError as e:
-            cpp.warning("Switching to 'Agg' backend failed with the message:")
-            print('"%s"' % str(e))
-            cpp.warning("Trying to continue...")
-
     return True
 
-# Add plotting_backend to parameter system (don't need this in C++)
-_available_backends = []
-if cpp.has_vtk():
-    _available_backends.append("vtk")
-if _has_matplotlib():
-    _available_backends.append("matplotlib")
-_available_backends.append("none")
-cpp.parameters.add("plotting_backend", _available_backends[0],
-        _available_backends)
 
 def mesh2triang(mesh):
     import matplotlib.tri as tri
     xy = mesh.coordinates()
     return tri.Triangulation(xy[:, 0], xy[:, 1], mesh.cells())
 
+
 def mplot_mesh(ax, mesh, **kwargs):
     tdim = mesh.topology().dim()
     gdim = mesh.geometry().dim()
@@ -83,11 +65,11 @@ def mplot_mesh(ax, mesh, **kwargs):
         return ax.triplot(mesh2triang(mesh), color=color, **kwargs)
     elif gdim == 3 and tdim == 3:
         bmesh = dolfin.BoundaryMesh(mesh, "exterior", order=False)
-        mplot_mesh(ax, bmesh, **kwargs)
+        return mplot_mesh(ax, bmesh, **kwargs)
     elif gdim == 3 and tdim == 2:
         xy = mesh.coordinates()
         return ax.plot_trisurf(*[xy[:,i] for i in range(gdim)],
-                triangles=mesh.cells(), **kwargs)
+                               triangles=mesh.cells(), **kwargs)
     elif tdim == 1:
         x = [mesh.coordinates()[:,i] for i in range(gdim)]
         if gdim == 1:
@@ -98,6 +80,7 @@ def mplot_mesh(ax, mesh, **kwargs):
     else:
         assert False, "this code should not be reached"
 
+
 # TODO: This is duplicated somewhere else
 def create_cg1_function_space(mesh, sh):
     r = len(sh)
@@ -109,6 +92,7 @@ def create_cg1_function_space(mesh, sh):
         V = dolfin.TensorFunctionSpace(mesh, "CG", 1, shape=sh)
     return V
 
+
 def mplot_expression(ax, f, mesh, **kwargs):
     # TODO: Can probably avoid creating the function space here by
     # restructuring mplot_function a bit so it can handle Expression
@@ -117,6 +101,7 @@ def mplot_expression(ax, f, mesh, **kwargs):
     g = dolfin.interpolate(f, V)
     return mplot_function(ax, g, **kwargs)
 
+
 def mplot_function(ax, f, **kwargs):
     mesh = f.function_space().mesh()
     gdim = mesh.geometry().dim()
@@ -139,10 +124,10 @@ def mplot_function(ax, f, **kwargs):
 
     if fvec.size() == mesh.num_cells():
         # DG0 cellwise function
-        C = fvec.array() # NB! Assuming here dof ordering matching cell numbering
+        C = fvec.get_local()  # NB! Assuming here dof ordering matching cell numbering
         if gdim == 2 and tdim == 2:
             return ax.tripcolor(mesh2triang(mesh), C, **kwargs)
-        elif gdim == 3 and tdim == 2: # surface in 3d
+        elif gdim == 3 and tdim == 2:  # surface in 3d
             # FIXME: Not tested, probably broken
             xy = mesh.coordinates()
             shade = kwargs.pop("shade", True)
@@ -203,11 +188,16 @@ def mplot_function(ax, f, **kwargs):
             x = mesh.coordinates()[:,0]
             ax.set_aspect('auto')
 
+            p = ax.plot(x, C, **kwargs)
+            
             # Setting limits for Line2D objects
+            # Must be done after generating plot to avoid ignoring function
+            # range if no vmin/vmax are supplied
             vmin = kwargs.pop("vmin", None)
             vmax = kwargs.pop("vmax", None)
             ax.set_ylim([vmin, vmax])
-            return ax.plot(x, C, **kwargs)
+            
+            return p
         #elif tdim == 1: # FIXME: Plot embedded line
         else:
             raise AttributeError('Matplotlib plotting backend only supports 2D mesh for scalar functions.')
@@ -232,14 +222,6 @@ def mplot_function(ax, f, **kwargs):
         if mode == "glyphs":
             args = X + U + [C]
             if gdim == 3:
-                # 3d quiver plot works only since matplotlib 1.4
-                import matplotlib
-                if StrictVersion(matplotlib.__version__) < '1.4':
-                    cpp.warning('Matplotlib version %s does not support 3d '
-                                'quiver plot. Continuing without plotting...'
-                                % matplotlib.__version__)
-                    return
-
                 length = kwargs.pop("length", 0.1)
                 return ax.quiver(*args, length=length, **kwargs)
             else:
@@ -259,6 +241,7 @@ def mplot_function(ax, f, **kwargs):
                             'plotting...' % (tdim, gdim))
                 return
 
+
 def mplot_meshfunction(ax, obj, **kwargs):
     mesh = obj.mesh()
     tdim = mesh.topology().dim()
@@ -274,20 +257,27 @@ def mplot_meshfunction(ax, obj, **kwargs):
                     'function of dim %d. Continuing without plotting...' % d)
         return
 
+
 def mplot_dirichletbc(ax, obj, **kwargs):
     raise AttributeError("Matplotlib plotting backend doesn't handle DirichletBC.")
 
+
 def _plot_matplotlib(obj, mesh, kwargs):
-    if isinstance(obj, (cpp.MultiMesh, cpp.MultiMeshFunction,
-                        cpp.MultiMeshDirichletBC)):
-        cpp.warning("Don't know how to plot type %s" % type(obj))
+    if not isinstance(obj, _matplotlib_plottable_types):
+        cpp.warning("Don't know how to plot type %s." % type(obj))
         return
 
+    # Plotting is not working with all ufl cells
+    if mesh.ufl_cell().cellname() not in ['interval', 'triangle', 'tetrahedron']:
+        raise AttributeError(("Matplotlib plotting backend doesn't handle %s mesh.\n"
+                              "Possible options are saving the output to XDMF file "
+                              "or using 'x3dom' backend.") % mesh.ufl_cell().cellname())
+
     # Avoid importing pyplot until used
     try:
         import matplotlib.pyplot as plt
     except:
-        print("*** Warning: matplotlib.pyplot not available, cannot plot")
+        cpp.warning("matplotlib.pyplot not available, cannot plot.")
         return
 
     gdim = mesh.geometry().dim()
@@ -313,7 +303,7 @@ def _plot_matplotlib(obj, mesh, kwargs):
         kwargs["vmax"] = vmax
 
     # Drop unsupported kwargs and inform user
-    _unsupported_kwargs = ["interactive", "rescale", "wireframe"]
+    _unsupported_kwargs = ["rescale", "wireframe"]
     for kw in _unsupported_kwargs:
         if kwargs.pop(kw, None):
             cpp.warning("Matplotlib backend does not support '%s' kwarg yet. "
@@ -333,39 +323,15 @@ def _plot_matplotlib(obj, mesh, kwargs):
         raise AttributeError('Failed to plot %s' % type(obj))
 
 
-# Compatibility with book
-def _VTKPlotter_write_ps(self, *args, **kwargs) :
-    print("*** Warning: VTKPlotter::write_ps() is not implemented -- use write_pdf instead")
+def _plot_x3dom(obj, kwargs):
+    if not isinstance(obj, _x3dom_plottable_types):
+        cpp.warning("Don't know how to plot type %s." % type(obj))
+        return 
+    
+    x3dom = dolfin.X3DOM()
+    out = x3dom.html(obj)
 
-_objects_referenced_from_plot_windows = {}
-def _plot_cpp(object, mesh, kwargs):
-    # Convert kwargs to cpp format
-    p = cpp.Parameters()
-    for key in kwargs:
-        try:
-            p.add(key, kwargs[key])
-        except TypeError:
-            cpp.warning("Incompatible type for keyword argument \"%s\". Ignoring." % key)
-
-    if isinstance(object, cpp.Expression):
-        plot_object = cpp.plot(object, mesh, p)
-    elif isinstance(object, cpp.MultiMesh):
-        return cpp.io._plot_multimesh(object)
-    else:
-        plot_object = cpp.plot(object, p)
-
-    # Compatibility with book
-    plot_object.write_ps = _VTKPlotter_write_ps
-
-    # Avoid premature deletion of plotted objects if they go out of
-    # scope before the plot window is closed. The plotter itself is
-    # safe, since it's created in the plot() C++ function, not
-    # directly from Python. But the Python plotter proxy may
-    # disappear, so we can't store the references there.
-    global _objects_referenced_from_plot_windows
-    _objects_referenced_from_plot_windows[plot_object.key()] = (object, mesh, p)
-
-    return plot_object
+    return out
 
 def plot(object, *args, **kwargs):
     """
@@ -386,7 +352,7 @@ def plot(object, *args, **kwargs):
 
         .. code-block:: python
 
-            mesh = UnitSquare(4,4)
+            mesh = UnitSquare(4, 4)
             plot(mesh)
 
         Use the ``title`` argument to specify title of the plot
@@ -419,19 +385,26 @@ def plot(object, *args, **kwargs):
                  hardcopy_prefix = "myplot",    # default plotfile name
                  scale = 2.0,                   # scale the warping/glyphs
                  title = "Fancy plot",          # set your own title
-                 backend = "vtk"                # choose plotting backend
                  )
 
     """
 
+    # Return if plotting is disables
+    if os.environ.get("DOLFIN_NOPLOT", "0") != "0":
+        return
+
+    # Return if Matplotlib is not available
+    if not _has_matplotlib():
+        cpp.info("Matplotlib is required to plot from Python.")
+        return
+
     # Plot element
     if isinstance(object, ufl.FiniteElementBase):
-        if os.environ.get("DOLFIN_NOPLOT", "0") != "0":
-            return
         import ffc
         return ffc.plot(object, *args, **kwargs)
 
-    # Get mesh from explicit mesh kwarg, only positional arg, or via object
+    # Get mesh from explicit mesh kwarg, only positional arg, or via
+    # object
     mesh = kwargs.pop('mesh', None)
     if isinstance(object, cpp.Mesh):
         if mesh is not None and mesh.id() != object.id():
@@ -451,8 +424,14 @@ def plot(object, *args, **kwargs):
                          "plot expression",
                          "Expecting a mesh as keyword argument")
 
+    backend = kwargs.pop("backend", "matplotlib")
+    if backend not in ("matplotlib", "x3dom"):
+        cpp.dolfin_error("plotting.py",
+                         "plot",
+                         "Plotting backend %s not recognised" % backend)
+    
     # Try to project if object is not a standard plottable type
-    if not isinstance(object, _plottable_types):
+    if not isinstance(object, _all_plottable_types):
         from dolfin.fem.projection import project
         try:
             cpp.info("Object cannot be plotted directly, projecting to"\
@@ -464,15 +443,10 @@ def plot(object, *args, **kwargs):
                   "and projection failed:\n  %s" % (str(object), str(e))
             cpp.dolfin_error("plotting.py", "plot object", msg)
 
-    # Select backend
-    backend = kwargs.pop("backend", None) or cpp.parameters["plotting_backend"]
-    if backend == "vtk":
-        return _plot_cpp(object, mesh, kwargs)
-    elif backend == "matplotlib":
+    # Plot
+    if backend == "matplotlib":
         return _plot_matplotlib(object, mesh, kwargs)
-    elif backend == "none":
-        cpp.warning("Plotting backend set to 'none'. Plotting disabled.")
-        return
+    elif backend == "x3dom":
+        return _plot_x3dom(object, kwargs)
     else:
-        cpp.dolfin_error("plotting.py", "plot object",
-                         "Unknown plotting backend '%s'" % backend)
+        assert False, "This code should not be reached."
