@@ -29,6 +29,7 @@ import hashlib
 
 import instant
 from ufl.utils.sorting import canonicalize_metadata
+import ffc
 from dolfin_utils.cppparser import *
 
 # Import PyDOLFIN
@@ -43,10 +44,10 @@ __all__ = ["compile_extension_module",
            "expression_to_code_fragments",
            "math_header"]
 
-# Bump the interface version if anything changes that invalidates cached
-# modules (not required for change in generated code, swig version or dolfin
-# version)
-_interface_version = 3
+# Bump the interface version if anything changes that invalidates
+# cached modules (not required for change in generated code, swig
+# version or dolfin version)
+_interface_version = 8
 
 # Add max_signature_length to parameter system (don't need this in C++)
 cpp.parameters.add("max_signature_length", 0)
@@ -84,17 +85,127 @@ const double pi = DOLFIN_PI;
 """ % "\n".join("using std::%s;" % mf for mf in _cpp_math_builtins)
 
 
-_cpp_keywords = ["auto","const","double","float","int","short","struct","unsigned",
-                 "break","continue","else","for","long","signed","switch","void",
-                 "case","default","enum","goto","register","sizeof","typedef",
-                 "char","do","extern","if","return","static","union","while",
-                 "asm","dynamic_cast","namespace","reinterpret_cast","try",
-                 "bool","explicit","new","static_cast","typeid","volatile",
-                 "catch","operator","template","typename",
-                 "class","friend","private","this","using",
-                 "const_cast","inline","public","throw","virtual",
-                 "delete","mutable","protected","wchar_t",
-                 "or","and","xor","not"]
+# Copied from http://en.cppreference.com/w/cpp/keyword 2017/02/02
+_allowed_cpp_keywords = """and
+and_eq
+bitand
+bitor
+bool
+char
+char16_t
+char32_t
+double
+false
+float
+int
+long
+not
+not_eq
+or
+or_eq
+short
+signed
+sizeof
+true
+unsigned
+wchar_t
+xor
+xor_eq
+""".splitlines()
+
+_disallowed_cpp_keywords = """alignas
+alignof
+asm
+atomic_cancel
+atomic_commit
+atomic_noexcept
+auto
+break
+case
+catch
+class
+compl
+concept
+const
+constexpr
+const_cast
+continue
+decltype
+default
+delete
+do
+dynamic_cast
+else
+enum
+explicit
+export
+extern
+for
+friend
+goto
+if
+import
+inline
+module
+mutable
+namespace
+new
+noexcept
+nullptr
+operator
+private
+protected
+public
+register
+reinterpret_cast
+requires
+return
+static
+static_assert
+static_cast
+struct
+switch
+synchronized
+template
+this
+thread_local
+throw
+try
+typedef
+typeid
+typename
+union
+using
+virtual
+void
+volatile
+while""".splitlines()
+
+
+# User defined attributes with same names as attributes
+# of any compiled expression base class cause confusion
+# and segfaults with swig
+_fenics_keywords = [
+    # Attributes of ufl.Expr and ufl.Coefficient
+    "T",
+    "dx",
+    "evaluate",
+    "geometric_dimension",
+    "count",
+    # Attributes of dolfin Variable, GenericFunction, and Function
+    "id",
+    "eval",
+    "value_rank",
+    "value_dimension",
+    "value_size",
+    "restrict",
+    "update",
+    "vector",
+    "in",
+    "interpolate",
+    "extrapolate",
+    ]
+
 
 _additional_declarations = r"""
 %%init%%{
@@ -167,10 +278,28 @@ re_symbol = re.compile(r"(?:[a-zA-Z_]*::)*[\w\.]+")
 re_number = re.compile("[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?")
 
 
-def expression_to_code_fragments(expr, arguments, generic_function_members=None):
-    "A help function which extract a dict with code snippets from an expression"
+def _check_valid_symbol(sym):
+    if sym in _disallowed_cpp_keywords:
+        cpp.dolfin_error("compilemodule.py",
+                         "parse expression string",
+                         "Detected C++ keyword (%s) in C++ expression" % sym)
+    if sym in _fenics_keywords:
+        cpp.dolfin_error("compilemodule.py",
+                         "parse expression string",
+                         "Detected name reserved by FEniCS (%s) in C++ expression" % sym)
+    if sym.startswith("_") or sym.endswith("_"):
+        cpp.dolfin_error("compilemodule.py",
+                         "parse expression string",
+                         "Detected invalid name with trailing underscores (%s) in C++ expression" % sym)
+    if sym.startswith("ufl_"):
+        cpp.dolfin_error("compilemodule.py",
+                         "parse expression string",
+                         "Detected invalid name with initial ufl_ (%s) in C++ expression" % sym)
 
-    generic_function_members = generic_function_members or []
+
+def expression_to_code_fragments(expr, arguments,
+        generic_function_members=(), mesh_function_members=()):
+    "A help function which extract a dict with code snippets from an expression"
     expr = list(expr)
 
     # Autodetect variables from function strings
@@ -182,17 +311,14 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
         assert isinstance(symbols, list)
         variables.update(symbols)
         for sym in symbols:
-            if sym in _cpp_keywords:
-                cpp.dolfin_error("compilemodule.py",
-                                 "parse expression string",
-                                 "Detected C++ keyword (%s) in C++ expression" % sym)
+            _check_valid_symbol(sym)
         # FIXME: Remove outcommented code?
         # NOTE: A hack to get around an ambiguous overloading of
         #       dolfin::pow(double,int)
 
         #if "pow" in symbols:
-        #    c = c.replace("pow","std::pow")
-        expr[i] = c
+        #    c = c.replace("pow", "std::pow")
+        #expr[i] = c
 
     # Remove any variables defined in the arguments list
     variables.difference_update(arguments)
@@ -200,27 +326,31 @@ def expression_to_code_fragments(expr, arguments, generic_function_members=None)
     # Remove the builtin math functions from the variables
     variables.difference_update(_all_math)
 
+    # Remove allowed C++ keywords
+    variables.difference_update(_allowed_cpp_keywords)
+
     # Remove the numerals from the variables
     numerals = [v for v in variables if re_number.match(v)]
     variables.difference_update(numerals)
 
-    # Remove bool literals
-    variables.difference_update(["true", "false"])
-
     # Remove namespace-specified variables
     namespacevars = [v for v in variables if "::" in v]
     variables.difference_update(namespacevars)
 
+    # Variables are either GenericFunction, MeshFunction, or scalar
+    mesh_function_member_names = [item[0] for item in mesh_function_members]
+    scalar_members = sorted(variables - set(generic_function_members) - set(mesh_function_member_names))
+
     # Generate code for member variables
-    members_code = ["  double %s;" % name for name in variables \
-                    if name not in generic_function_members]
-    members_code.extend("  std::shared_ptr<dolfin::GenericFunction> shared_%s;" % name \
+    members_code = ["  double %s;" % name for name in scalar_members]
+    members_code.extend("  std::shared_ptr<dolfin::MeshFunction<%s>> shared_%s;" % (typename, name)
+                        for name, typename in mesh_function_members)
+    members_code.extend("  std::shared_ptr<dolfin::GenericFunction> shared_%s;" % name
                         for name in generic_function_members)
     members_code = "\n".join(members_code)
 
-    # Generate constructor code for initialization of member variables
-    constructor_code = "\n".join("    %s = 0;" % name for name in variables\
-                                 if name not in generic_function_members)
+    # Generate constructor code for initialization of scalar member variables
+    constructor_code = "\n".join("    %s = 0;" % name for name in scalar_members)
 
     # Connect the code fragments using the function template code
     fragments = {}
@@ -363,6 +493,7 @@ def compile_extension_module(code, module_name="",
         module_signature = hashlib.sha1((repr(code) +
                                    dolfin.__version__ +
                                    str(_interface_version) +
+                                   ffc.ufc_signature() +
                                    sys.version +
                                    additional_declarations +
                                    str(additional_system_headers) +
