@@ -25,6 +25,8 @@ from six import string_types
 import re
 import hashlib
 
+from ufl.utils.sequences import product
+
 from dolfin.compilemodules.compilemodule import (compile_extension_module,
                                                  expression_to_code_fragments,
                                                  math_header)
@@ -69,43 +71,83 @@ def flatten_and_check_expression(expr):
             return expr, shape
     raise TypeError("Wrong type of expressions. Provide a 'str', a 'tuple' of 'str' or a 'tuple' of 'tuple' of 'str': %s" % str(expr))
 
-def expression_to_dolfin_expression(expr, generic_function_members):
+
+def expression_to_dolfin_expression(expr, generic_function_members, mesh_function_members):
     "Generates code for a dolfin::Expression subclass for a single expression."
 
+    # TODO: Make this configurable through global dolfin 'debug mode' parameter?
+    add_runtime_checks = True
+
     # Check and flattern provided expression
-    expr, shape = flatten_and_check_expression(expr)
+    expr, expr_shape = flatten_and_check_expression(expr)
 
     # Extract code fragments from the expr
+    generic_function_member_names = [item[0] for item in generic_function_members]
     fragments, members = expression_to_code_fragments(
-        expr, ["values","x"], generic_function_members)
+        expr, ["values", "x"],
+        generic_function_member_names,
+        mesh_function_members)
 
     # Generate code for value_rank
     value_shape_code = ["    _value_shape.push_back(%d);" % value_dim
-                        for value_dim in shape]
+                        for value_dim in expr_shape]
 
     evalcode = []
 
-    # Generate code for constant members
-    for name in generic_function_members:
-        evalcode.append("    if (shared_%s->value_size()!=1)" % name)
-        evalcode.append("      dolfin_error(\"generated code\",")
-        evalcode.append("                   \"calling eval\", ")
-        evalcode.append("                   \"Parameter \\'%s\\' is not scalar valued\");" % name)
-        evalcode.append("    if (shared_%s.get()==this)" % name)
-        evalcode.append("      dolfin_error(\"generated code\",")
-        evalcode.append("                   \"calling eval\",")
-        evalcode.append("                   \"Circular eval call detected. Cannot use itself as parameter \\'%s\\' within eval\");" % name)
-        evalcode.append("    Array<double> %s__array_(1);" % name)
+    # Runtime checks (TODO: Better to check these when updating the value instead...)
+    if add_runtime_checks:
+        for name, shape in generic_function_members:
+            dim = product(shape)
+            evalcode.append("    if (shared_%s->value_size() != %d)" % (name, dim))
+            evalcode.append("      dolfin_error(\"generated code\",")
+            evalcode.append("                   \"calling eval\", ")
+            evalcode.append("                   \"Expecting value size %d for parameter \\'%s\\'\");" % (dim, name))
+            evalcode.append("    if (shared_%s.get() == this)" % name)
+            evalcode.append("      dolfin_error(\"generated code\",")
+            evalcode.append("                   \"calling eval\",")
+            evalcode.append("                   \"Circular eval call detected. Cannot use itself as parameter \\'%s\\' within eval\");" % name)
+            evalcode.append("")
+
+    # Generate code for evaluating genericfunction members
+    for name, shape in generic_function_members:
+        dim = product(shape)
+
+        # Setup output array and call eval
+        evalcode.append("    double %s__data_[%d];" % (name, dim))
+        evalcode.append("    Array<double> %s__array_(%d, %s__data_);" % (name, dim, name))
         evalcode.append("    shared_%s->eval(%s__array_, x);" % (name, name))
-        evalcode.append("    const double %s = %s__array_[0];" % (name, name))
+
+        # Ensure const access through userdefined name
+        if shape:
+            # Vector valued result
+            evalcode.append("    const Array<double> & %s = %s__array_;" % (name, name))
+        else:
+            # Scalar valued result
+            evalcode.append("    const double %s = %s__array_[0];" % (name, name))
+        evalcode.append("")
+
+    # Lookup in MeshFunction<typename>(mesh, tdim)
+    for name, typename in mesh_function_members:
+        evalcode.append("    const %s %s = (*shared_%s)[cell.index];" % (typename, name, name))
 
     # Generate code for the actual expression evaluation
-    evalcode.extend("    values[%d] = %s;" % (i, c) for (i,c) in enumerate(expr))
+    evalcode.extend("    values[%d] = %s;" % (i, c) for i, c in enumerate(expr))
+
+    # Adapt evalcode to with/without cell argument if possible
+    evalcode = "\n".join(evalcode)
+    evalcode_cell = evalcode.replace(
+        "__array_, x", "__array_, x, cell")
+    if mesh_function_members:
+        # TODO: Reuse code in Function::eval(values, x) which looks up cell from x?
+        evalcode = []
+        evalcode.append("      dolfin_error(\"generated code\",")
+        evalcode.append("                   \"calling eval\", ")
+        evalcode.append("                   \"Need cell to evaluate this Expression\");")
+        evalcode = "\n".join(evalcode)
 
     # Connect the code fragments using the expression template code
-    fragments["evalcode"]  = "\n".join(evalcode)
-    fragments["evalcode_cell"]  = fragments["evalcode"].replace(
-        "__array_, x", "__array_, x, cell")
+    fragments["evalcode"]  = evalcode
+    fragments["evalcode_cell"]  = evalcode_cell
     fragments["value_shape"] = "\n".join(value_shape_code)
 
     # Assign classname
@@ -145,7 +187,9 @@ def compile_expression_code(code, classnames=None, module_name=None,
     return expression_classes
 
 
-def compile_expressions(cppargs, generic_function_members=None,
+def compile_expressions(cppargs,
+                        generic_function_members=None,
+                        mesh_function_members=None,
                         mpi_comm=None):
     """
     Compiles a list of either C++ expressions of full subclasses of
@@ -174,16 +218,19 @@ def compile_expressions(cppargs, generic_function_members=None,
     #    %s
     #""" % "\n".join("        " + b for b in _builtins)
     # FIXME: Hook up this to a more general debug mechanism
-    assert(isinstance(cppargs, list))
+    assert isinstance(cppargs, list)
 
     generic_function_members_list = (generic_function_members or
                                     [[] for i in range(len(cppargs))])
+    mesh_function_members_list = (mesh_function_members or
+                                 [[] for i in range(len(cppargs))])
 
     # Collect code and classnames
     code_snippets = []; classnames = []; all_members = []; additional_declarations = [];
 
-    for cpparg, generic_function_members in zip(cppargs, generic_function_members_list):
-        assert(isinstance(cpparg, string_types + (tuple, list)))
+    for cpparg, generic_function_members, mesh_function_members in \
+            zip(cppargs, generic_function_members_list, mesh_function_members_list):
+        assert isinstance(cpparg, string_types + (tuple, list))
         # If the cpparg includes the word 'class' and 'Expression',
         # assume it is a c++ code snippet
 
@@ -198,11 +245,16 @@ def compile_expressions(cppargs, generic_function_members=None,
             # FIXME: Check for passed dimension?
         else:
             classname, code, members = \
-                       expression_to_dolfin_expression(cpparg, generic_function_members)
+                expression_to_dolfin_expression(cpparg,
+                    generic_function_members, mesh_function_members)
 
             additional_declarations.extend(
                 "%%rename(%s) dolfin::%s::shared_%s;" % (name, classname, name)
-                for  name in generic_function_members)
+                for name, shape in generic_function_members)
+
+            additional_declarations.extend(
+                "%%rename(%s) dolfin::%s::shared_%s;" % (name, classname, name)
+                for name, typename in mesh_function_members)
 
         all_members.append(members)
         code_snippets.append(code)
