@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 "This module provides functionality for compilation of strings as dolfin SubDomains."
 
-# Copyright (C) 2008-2008 Martin Sandve Alnes
+# Copyright (C) 2008-2016 Martin Sandve Alnæs
 #
 # This file is part of DOLFIN.
 #
@@ -16,14 +17,10 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-#
-# First added:  2008-07-01
-# Last changed: 2011-04-18
 
+from six import string_types
 import re
-import os
 import hashlib
-import instant
 
 # Import local compile_extension_module
 from dolfin.compilemodules.compilemodule import (compile_extension_module,
@@ -72,15 +69,15 @@ def expression_to_subdomain(cpparg, classname):
     """
 
     # Assure we have a simple string expression
-    assert isinstance(cpparg, str)
+    assert isinstance(cpparg, string_types)
 
     # Extract code fragments from the expr and defaults
     fragments, members = expression_to_code_fragments(\
         [cpparg], ["x", "on_boundary", "DOLFIN_EPS"])
-    
+
     # Generate code for inside()
     insidecode = "  return %s;" % cpparg
-    
+
     # Generate code for map()
     #mapcode = "..."
 
@@ -135,24 +132,22 @@ def CompiledSubDomain(cppcode, mpi_comm=None, **kwargs):
 
     """
 
-    if not isinstance(cppcode, str):
+    if not isinstance(cppcode, string_types):
         raise TypeError("expected a 'str'")
-    
-    if isinstance(cppcode, str) and "class" in cppcode and \
+
+    if isinstance(cppcode, string_types) and "class" in cppcode and \
            "SubDomain" in cppcode:
-        members = []
-        classname = re.findall(r"class[ ]+([\w]+).*", code)[0]
-        code = cppcode
+        classname = re.findall(r"class[ ]+([\w]+).*", cppcode)[0]
+        code, members = cppcode, []
 
     else:
-        
         classname = "CompiledSubDomain" + hashlib.sha1(cppcode.encode(\
             "utf-8")).hexdigest()
         code, members = expression_to_subdomain(cppcode, classname)
-    
+
     SubDomainClass = compile_subdomain_code(code, classname, mpi_comm=mpi_comm)
-    
-    # Check passed default arguments 
+
+    # Check passed default arguments
     not_allowed = [n for n in dir(cpp.SubDomain) if n[0] !="_"]
     not_allowed += ["cppcode"]
 
@@ -162,14 +157,18 @@ def CompiledSubDomain(cppcode, mpi_comm=None, **kwargs):
             if member not in kwargs:
                 missing.append(member)
         missing = ", ".join("'%s'" % miss for miss in missing)
-        raise RuntimeError("expected a default value to all member "\
-                           "variables in the SubDomain. Missing: %s." % missing)
+        cpp.dolfin_error("subdomains.py",
+                         "compile C++ subdomain",
+                         "Expected a default value to all member "
+                         "variables in the SubDomain. Missing: %s." % missing)
 
     for name in list(kwargs.keys()):
         if name in not_allowed:
-            raise RuntimeError("Parameter name: '%s' is not allowed. It is "\
-                               "part of the interface of SubDomain" % name)
-            
+            cpp.dolfin_error("subdomains.py",
+                             "compile C++ subdomain",
+                             "Parameter name: '%s' is not allowed. It is "
+                             "part of the interface of SubDomain" % name)
+
         if not (all(isinstance(value, (int, float)) \
                     for value in list(kwargs.values()))):
             raise TypeError("expected default arguments for member variables "\
@@ -179,7 +178,7 @@ def CompiledSubDomain(cppcode, mpi_comm=None, **kwargs):
     SubDomainClass.cppcode = cppcode
 
     # Instantiate CompiledSubDomain
-    subdomain = SubDomainClass()    
+    subdomain = SubDomainClass()
 
     # Set default variables
     for member, value in list(kwargs.items()):
