@@ -22,3 +22,9 @@
 import dolfin.cpp as cpp
 
 #--- Deprecation of ... etc. ---
+
+def has_pybind11():
+    return False
+
+from dolfin import Expression
+UserExpression = Expression
