@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2009-2011 Anders Logg
 #
 # This file is part of DOLFIN.
@@ -20,16 +21,19 @@
 # First added:  2008-10-22
 # Last changed: 2012-08-18
 
-__all__ = ["AutoSubDomain", "DirichletBC", "homogenize"]
-
 import types
 
+from six import string_types
+
 import dolfin.cpp as cpp
 from dolfin.functions.constant import Constant
 from dolfin.compilemodules.subdomains import CompiledSubDomain
 import ufl
 from dolfin.fem.projection import project
 
+__all__ = ["AutoSubDomain", "DirichletBC"]
+
+
 class AutoSubDomain(cpp.SubDomain):
     "Wrapper class for creating a SubDomain from an inside() function."
 
@@ -40,12 +44,12 @@ class AutoSubDomain(cpp.SubDomain):
         if not isinstance(inside_function, types.FunctionType):
             cpp.dolfin_error("bcs.py",
                              "auto-create subdomain",
-                             "Expecting a function (not %s)" % \
-                                 str(type(inside_function)))
+                             "Expecting a function (not %s)" %
+                             str(type(inside_function)))
         self.inside_function = inside_function
 
         # Check the number of arguments
-        if not inside_function.__code__.co_argcount in (1, 2):
+        if inside_function.__code__.co_argcount not in (1, 2):
             cpp.dolfin_error("bcs.py",
                              "auto-create subdomain",
                              "Expecting a function of the form inside(x) or inside(x, on_boundary)")
@@ -61,6 +65,7 @@ class AutoSubDomain(cpp.SubDomain):
         else:
             return self.inside_function(x, on_boundary)
 
+
 class DirichletBC(cpp.DirichletBC):
 
     # Reuse doc-string from cpp.DirichletBC
@@ -74,19 +79,22 @@ class DirichletBC(cpp.DirichletBC):
             if not isinstance(args[0], cpp.DirichletBC):
                 cpp.dolfin_error("bcs.py",
                                  "create DirichletBC",
-                                 "Expecting a DirichleBC as only argument"\
+                                 "Expecting a DirichleBC as only argument"
                                  " for copy constructor")
 
             # Initialize base class
             cpp.DirichletBC.__init__(self, args[0])
+            self.domain_args = args[0].domain_args
             return
 
+        mpi_comm = args[0].mesh().mpi_comm()
+
         # Special case for value specified as float, tuple or similar
         if len(args) >= 2 and not isinstance(args[1], cpp.GenericFunction):
             if isinstance(args[1], ufl.classes.Expr):
                 expr = project(args[1], args[0])
             else:
-                expr = Constant(args[1]) # let Constant handle all problems
+                expr = Constant(args[1])  # let Constant handle all problems
             args = args[:1] + (expr,) + args[2:]
 
         # Special case for sub domain specified as a function
@@ -95,8 +103,8 @@ class DirichletBC(cpp.DirichletBC):
             args = args[:2] + (sub_domain,) + args[3:]
 
         # Special case for sub domain specified as a string
-        if len(args) >= 3 and isinstance(args[2], str):
-            sub_domain = CompiledSubDomain(args[2])
+        if len(args) >= 3 and isinstance(args[2], string_types):
+            sub_domain = CompiledSubDomain(args[2], mpi_comm=mpi_comm)
             args = args[:2] + (sub_domain,) + args[3:]
 
         # Store Expression to avoid scoping issue with SWIG directors
@@ -119,7 +127,7 @@ class DirichletBC(cpp.DirichletBC):
 
         # Check for other keyword arguments
         for key in kwargs:
-            if not key in allowed_kwargs:
+            if key not in allowed_kwargs:
                 cpp.dolfin_error("bcs.py",
                                  "create boundary condition",
                                  "Unknown keyword argument \"%s\"" % key)
@@ -129,57 +137,3 @@ class DirichletBC(cpp.DirichletBC):
 
     # Set doc string
     __init__.__doc__ = cpp.DirichletBC.__init__.__doc__
-
-
-def homogenize(bc):
-    """
-    Return a homogeneous version of the given boundary condition.
-
-    *Arguments*
-        bc
-            a :py:class:`DirichletBC <dolfin.fem.bcs.DirichletBC>` instance,
-            or a list/tuple of
-            :py:class:`DirichletBC <dolfin.fem.bcs.DirichletBC>` instances.
-
-    Other types of boundary conditions are ignored.
-
-    If the given boundary condition is a list of boundary conditions,
-    then a list of homogeneous boundary conditions is returned.
-
-    """
-
-    # Handle case when boundary condition is a list
-    if isinstance(bc, (list, tuple)):
-        bcs = bc
-        return [homogenize(bc) for bc in bcs]
-
-    # Only consider Dirichlet boundary conditions
-    if not isinstance(bc, cpp.DirichletBC):
-        cpp.dolfin_error("bcs.py",
-                         "homogenize boundary condition",
-                         "Can only homogenize DirichletBCs")
-
-    # Create zero function
-    V = bc.function_space()
-    if V.element().value_rank() == 0:
-        zero = Constant(0)
-    elif V.element().value_rank() == 1:
-        zero = Constant([0]*V.element().value_dimension(0))
-    else:
-        cpp.dolfin_error("bcs.py",
-                         "homogenize boundary condition",
-                         "Unhandled value rank %d for homogenization of boundary conditions" % \
-                             V.element().value_rank())
-
-    # Create homogeneous boundary condition
-    if len(bc.domain_args) == 1:
-        new_bc = cpp.DirichletBC(V, zero, bc.domain_args[0])
-    elif len(bc.domain_args) == 2:
-        new_bc = cpp.DirichletBC(V, zero, bc.domain_args[0], bc.domain_args[1])
-    else:
-        cpp.dolfin_error("bcs.py",
-                         "homogenize boundary condition",
-                         "Unknown type of boundary specification")
-    new_bc.domain_args = bc.domain_args
-
-    return new_bc
