@@ -73,10 +73,13 @@ class Form(cpp.Form):
         if len(sd) != 1:
             cpp.dolfin_error("form.py",
                              "creating dolfin.Form",
-                             "Expecting subdomain data for a single domain only.")
+                             "Expecting subdomain data for exactly one domain"
+                             "(got {}).".format(len(sd)))
         self.subdomains, = list(sd.values())  # Assuming single domain
         domain, = list(sd.keys())  # Assuming single domain
         mesh = domain.ufl_cargo()
+        if isinstance(mesh, cpp.MultiMesh):
+            mesh = mesh.part(0)
 
         # Having a mesh in the form is now a strict requirement
         if mesh is None:
