@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2010-2012 Marie E. Rognes
 #
 # This file is part of DOLFIN.
@@ -17,12 +18,13 @@
 #
 # Modified by Johan Hake 2011
 # Modified by Anders Logg 2011
+# Modified by Jan Blechta 2015
 
 import ufl
-import dolfin.cpp as cpp
+import ufl.algorithms.elementtransformations
 
-from dolfin import FunctionSpace, MixedFunctionSpace, VectorFunctionSpace
-from dolfin import Function, TrialFunction, Argument
+import dolfin.cpp as cpp
+from dolfin import FunctionSpace, Function, Argument, MultiMeshFunction
 
 
 __all__ = ["derivative", "adjoint", "increase_order", "tear"]
@@ -48,6 +50,10 @@ def adjoint(form, reordered_arguments=None):
     # (NB: Order does not matter anymore here because number is absolute)
     v_1 = Argument(arguments[1].function_space(), arguments[0].number(), arguments[0].part())
     v_0 = Argument(arguments[0].function_space(), arguments[1].number(), arguments[1].part())
+    # Also copy the extended part of the argument for MultiMesh-functionality
+    if hasattr(arguments[0], '_V_multi'):
+        v_1._V_multi = arguments[1]._V_multi
+        v_0._V_multi = arguments[0]._V_multi
 
     # Call ufl.adjoint with swapped arguments as new arguments
     return ufl.adjoint(form, reordered_arguments=(v_1, v_0))
@@ -64,55 +70,42 @@ def derivative(form, u, du=None, coefficient_derivatives=None):
         if any(arg.part() is not None for arg in form_arguments):
             cpp.dolfin_error("formmanipulation.py",
                              "compute derivative of form",
-                             "Cannot automatically create third argument using parts, please supply one")
+                             "Cannot automatically create new Argument using "
+                             "parts, please supply one")
         part = None
 
-        if isinstance(u, Function):
+        if isinstance(u, (Function, MultiMeshFunction)):
             V = u.function_space()
             du = Argument(V, number, part)
         elif isinstance(u, (list,tuple)) and all(isinstance(w, Function) for w in u):
-            V = MixedFunctionSpace([w.function_space() for w in u])
-            du = ufl.split(Argument(V, number, part))
+            cpp.dolfin_error("formmanipulation.py",
+                             "take derivative of form w.r.t. a tuple of Coefficients",
+                             "Take derivative w.r.t. a single Coefficient on "\
+                             "a mixed space instead.")
         else:
             cpp.dolfin_error("formmanipulation.py",
-                             "compute derivative of form",
-                             "Cannot automatically create third argument, please supply one")
+                             "compute derivative of form w.r.t. '%s'" % u,
+                             "Supply Function as a Coefficient")
+
     return ufl.derivative(form, u, du, coefficient_derivatives)
 
 derivative.__doc__ = ufl.derivative.__doc__
+derivative.__doc__ += """
+
+    A tuple of Coefficients in place of a single Coefficient is not
+    supported in DOLFIN. Supply rather a Function on a mixed space in
+    place of a Coefficient.
+    """
 
 def increase_order(V):
     """
     For a given function space, return the same space, but with a
     higher polynomial degree
     """
-
-    n = V.num_sub_spaces()
-    if n > 0:
-        spaces = []
-        for i in range(n):
-            V_i = V.sub(i)
-            element = V_i.ufl_element()
-            # Handle VectorFunctionSpaces specially
-            if isinstance(element, ufl.VectorElement):
-                if element.family() == "Real":
-                    spaces += [V_i]
-                else:
-                    spaces += [VectorFunctionSpace(V_i.mesh(),
-                                                   element.family(),
-                                                   element.degree() + 1,
-                                                   dim=element.num_sub_elements())]
-            # Handle all else as MixedFunctionSpaces
-            else:
-                spaces += [increase_order(V_i)]
-
-        return MixedFunctionSpace(spaces)
-
-    if V.ufl_element().family() == "Real":
-        return FunctionSpace(V.mesh(), "Real", 0)
-
-    return FunctionSpace(V.mesh(), V.ufl_element().family(),
-                         V.ufl_element().degree() + 1)
+    mesh = V.mesh()
+    element = ufl.algorithms.elementtransformations.increase_order(V.ufl_element())
+    constrained_domain = V.dofmap().constrained_domain
+    return FunctionSpace(mesh, element, constrained_domain=constrained_domain)
 
 def change_regularity(V, family):
     """
@@ -120,24 +113,14 @@ def change_regularity(V, family):
     the finite elements specified by 'family'. Possible families
     are the families supported by the form compiler
     """
+    mesh = V.mesh()
+    element = ufl.algorithms.elementtransformations.change_regularity(V.ufl_element(), family)
+    constrained_domain = V.dofmap().constrained_domain
+    return FunctionSpace(mesh, element, constrained_domain=constrained_domain)
 
-    n = V.num_sub_spaces()
-    if n > 0:
-        return MixedFunctionSpace([change_regularity(V.sub(i), family)
-                                   for i in range(n)])
-
-    element = V.ufl_element()
-    shape = element.value_shape()
-    if not shape:
-        return FunctionSpace(V.mesh(), family, element.degree())
-
-    return MixedFunctionSpace([FunctionSpace(V.mesh(), family, element.degree())
-                               for i in range(shape[0])])
-
-def tear(V_h):
+def tear(V):
     """
     For a given function space, return the corresponding discontinuous
     space
     """
-    W = change_regularity(V_h, "DG")
-    return W
+    return change_regularity(V, "DG")
