@@ -4,7 +4,7 @@
 :py:class:`Expression <dolfin.functions.expression.Expression>` onto a
 finite element space."""
 
-# Copyright (C) 2009-2011 Anders Logg
+# Copyright (C) 2009-2017 Anders Logg
 #
 # This file is part of DOLFIN.
 #
@@ -21,29 +21,35 @@ finite element space."""
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
+# Modified by Jørgen S. Dokken, 2017
 # First added:  2009-08-29
-# Last changed: 2011-11-15
+# Last changed: 2017-03-17
 
 # Local imports
 import dolfin.cpp as cpp
 
-from dolfin.functions.functionspace import FunctionSpace
+from dolfin.functions.functionspace import FunctionSpace, MultiMeshFunctionSpace
 from dolfin.functions.function import Function
+from dolfin.functions.multimeshfunction import MultiMeshFunction
 
 __all__ = ["interpolate"]
 
 
 def interpolate(v, V):
     """Return interpolation of a given function into a given finite
-    element space.
+    element space. Also supports MultiMesh.
 
     *Arguments*
         v
             a :py:class:`Function <dolfin.functions.function.Function>` or
             an :py:class:`Expression <dolfin.functions.expression.Expression>`
+            or an :py:class`MultiMeshFunction <dolfin.functions.multimeshfunction.MultiMeshFunction>`
         V
             a :py:class:`FunctionSpace (standard, mixed, etc.)
-            <dolfin.functions.functionspace.FunctionSpace>`
+            <dolfin.functions.functionspace.FunctionSpace>` or
+            a :py:class:`MultiMeshFunctionSpace
+            <dolfin.cpp.function.MultiMeshFunctionSpace>`.
+
 
     *Example of usage*
 
@@ -56,10 +62,14 @@ def interpolate(v, V):
     """
 
     # Check arguments
-    if not isinstance(V, FunctionSpace):
+    if not isinstance(V, (FunctionSpace, cpp.MultiMeshFunctionSpace)):
         cpp.dolfin_error("interpolation.py",
                          "compute interpolation",
                          "Illegal function space for interpolation, not a FunctionSpace (%s)" % str(v))
+    if isinstance(V, cpp.MultiMeshFunctionSpace):
+        Pv = MultiMeshFunction(V)
+        Pv.interpolate(v)
+        return Pv
 
     # Compute interpolation
     Pv = Function(V)
