@@ -4,7 +4,7 @@
 <dolfin.functions.function.Function>`, including the standard
 :math:`L^2`-norm and other norms."""
 
-# Copyright (C) 2008-2014 Anders Logg
+# Copyright (C) 2008-2017 Anders Logg
 #
 # This file is part of DOLFIN.
 #
@@ -20,20 +20,25 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
+#
+# Modified by: Jørgen S. Dokken, 2017
+# First added: 2007
+# Last changed: 2017-03-17
 
 from six import string_types
 import ufl
-from ufl import inner, grad, div, curl, dx, FiniteElement, VectorElement, Coefficient
+from ufl import inner, grad, div, curl, dx, FiniteElement, VectorElement, Coefficient, TensorElement
 from math import sqrt
 
 import dolfin.cpp as cpp
 
 from dolfin.cpp import GenericVector, GenericFunction, Function, Mesh, error, Vector
-from dolfin.fem.assembling import assemble
+from dolfin.fem.assembling import assemble, assemble_multimesh
 from dolfin.fem.interpolation import interpolate
 from dolfin.functions.function import Function
+from dolfin.functions.multimeshfunction import MultiMeshFunction
 from dolfin.functions.functionspace import FunctionSpace, \
-    VectorFunctionSpace, TensorFunctionSpace
+    VectorFunctionSpace, TensorFunctionSpace, MultiMeshFunctionSpace
 
 __all__ = ["norm", "errornorm"]
 
@@ -45,11 +50,13 @@ def norm(v, norm_type="L2", mesh=None):
     *Arguments*
         v
             a :py:class:`Vector <dolfin.cpp.Vector>` or
-            a :py:class:`Function <dolfin.functions.function.Function>`.
+            a :py:class:`Function <dolfin.functions.function.Function>` or
+            a :py:class:`MultiMeshFunction <dolfin.functions.multimeshfunction.MultiMeshFunction>`
         norm_type
             see below for alternatives.
         mesh
-            optional :py:class:`Mesh <dolfin.cpp.Mesh>` on
+            optional :py:class:`Mesh <dolfin.cpp.Mesh>` or
+            a :py:class:`MultiMesh <dolfin.cpp.MultiMesh>` on
             which to compute the norm.
 
     If the norm type is not specified, the standard :math:`L^2` -norm
@@ -94,48 +101,53 @@ def norm(v, norm_type="L2", mesh=None):
 
     """
 
-    if not isinstance(v, (GenericVector, GenericFunction)):
+    if not isinstance(v, (GenericVector, GenericFunction, MultiMeshFunction)):
         cpp.dolfin_error("norms.py",
                          "compute norm",
-                         "expected a GenericVector or GenericFunction")
-
+                         "expected a GenericVector ,GenericFunction or a MultiMeshFunction")
     # Check arguments
     if not isinstance(norm_type, string_types):
         cpp.dolfin_error("norms.py",
                          "compute norm",
                          "Norm type must be a string, not " +
                          str(type(norm_type)))
-    if mesh is not None and not isinstance(mesh, cpp.Mesh):
+    if mesh is not None and not isinstance(mesh, (cpp.Mesh, cpp.MultiMesh)):
         cpp.dolfin_error("norms.py",
                          "compute norm",
-                         "Expecting a Mesh, not " + str(type(mesh)))
+                         "Expecting a Mesh or a MultiMesh, not " + str(type(mesh)))
 
     # Get mesh from function
     if isinstance(v, Function) and mesh is None:
         mesh = v.function_space().mesh()
+    elif isinstance(v, MultiMeshFunction) and mesh is None:
+        mesh = v.function_space().multimesh()
 
     # Define integration measure and domain
-    dx = ufl.dx(mesh)
-
+    multimesh = False
+    if isinstance(v, MultiMeshFunction):
+        multimesh = True
+        dc = ufl.dx(mesh) + ufl.dC(mesh)
+    else:
+        dc = ufl.dx(mesh)
     # Select norm type
     if isinstance(v, GenericVector):
         return v.norm(norm_type.lower())
-
-    elif (isinstance(v, Coefficient) and isinstance(v, GenericFunction)):
+    elif (isinstance(v, Coefficient) and isinstance(v, (GenericFunction,
+                                                        MultiMeshFunction))):
         if norm_type.lower() == "l2":
-            M = v**2*dx
+            M = v**2*dc
         elif norm_type.lower() == "h1":
-            M = (v**2 + grad(v)**2)*dx
+            M = (v**2 + grad(v)**2)*dc
         elif norm_type.lower() == "h10":
-            M = grad(v)**2*dx
+            M = grad(v)**2*dc
         elif norm_type.lower() == "hdiv":
-            M = (v**2 + div(v)**2)*dx
+            M = (v**2 + div(v)**2)*dc
         elif norm_type.lower() == "hdiv0":
-            M = div(v)**2*dx
+            M = div(v)**2*dc
         elif norm_type.lower() == "hcurl":
-            M = (v**2 + curl(v)**2)*dx
+            M = (v**2 + curl(v)**2)*dc
         elif norm_type.lower() == "hcurl0":
-            M = curl(v)**2*dx
+            M = curl(v)**2*dc
         else:
             cpp.dolfin_error("norms.py",
                              "compute norm",
@@ -144,10 +156,13 @@ def norm(v, norm_type="L2", mesh=None):
     else:
         cpp.dolfin_error("norms.py",
                          "compute norm",
-                         "Unknown object type. Must be a vector or a function")
+                         "Unknown object type. Must be a vector, function or a multimeshfunction")
 
     # Assemble value
-    r = assemble(M, form_compiler_parameters={"representation": "quadrature"})
+    if multimesh:
+        r = assemble_multimesh(M, form_compiler_parameters={"representation": "quadrature"})
+    else:
+        r = assemble(M)
 
     # Check value
     if r < 0.0:
@@ -211,20 +226,26 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
     """
 
     # Check argument
-    if not isinstance(u, cpp.GenericFunction):
+    if not isinstance(u, (cpp.GenericFunction, cpp.MultiMeshFunction)):
         cpp.dolfin_error("norms.py",
                          "compute error norm",
-                         "Expecting a Function or Expression for u")
-    if not isinstance(uh, cpp.Function):
+                         "Expecting a Function, MultiMeshFunction or Expression for u")
+    if not (isinstance(uh, cpp.Function) or (uh, cpp.MultiMeshFunction)):
         cpp.dolfin_error("norms.py",
                          "compute error norm",
                          "Expecting a Function for uh")
+    # Test if errornorm is for multimesh
+    multimesh=False
 
     # Get mesh
     if isinstance(u, Function) and mesh is None:
         mesh = u.function_space().mesh()
     if isinstance(uh, Function) and mesh is None:
         mesh = uh.function_space().mesh()
+    if isinstance(uh, MultiMeshFunction) and mesh is None:
+        mesh = uh.function_space().multimesh()
+        multimesh=True
+
     if mesh is None:
         cpp.dolfin_error("norms.py",
                          "compute error norm",
@@ -253,21 +274,37 @@ def errornorm(u, uh, norm_type="l2", degree_rise=3, mesh=None):
         cpp.warning("Degree of exact solution may be inadequate for accurate result in errornorm.")
 
     # Create function space
-    if rank == 0:
-        V = FunctionSpace(mesh, "Discontinuous Lagrange", degree)
-    elif rank == 1:
-        V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
-                                dim=shape[0])
-    elif rank > 1:
-        V = TensorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
-                                shape=shape)
+    if multimesh:
+        if rank == 0:
+            element = FiniteElement("Discontinuous Lagrange", mesh.ufl_cell(),
+                                    degree)
+        elif rank == 1:
+            element = VectorElement("Discontinuous Lagrange", mesh.ufl_cell(),
+                                    degree)
+        elif rank == 2:
+            element = TensorElement("Discontinuous Lagrange", mesh.ufl_cell(),
+                                    degree)
+        V = MultiMeshFunctionSpace(mesh, element)
+    else:
+        if rank == 0:
+            V = FunctionSpace(mesh, "Discontinuous Lagrange", degree)
+        elif rank == 1:
+            V = VectorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
+                                    dim=shape[0])
+        elif rank > 1:
+            V = TensorFunctionSpace(mesh, "Discontinuous Lagrange", degree,
+                                    shape=shape)
 
     # Interpolate functions into finite element space
     pi_u = interpolate(u, V)
     pi_uh = interpolate(uh, V)
 
     # Compute the difference
-    e = Function(V)
+    if multimesh:
+        e = MultiMeshFunction(V)
+    else:
+        e = Function(V)
+
     e.assign(pi_u)
     e.vector().axpy(-1.0, pi_uh.vector())
 
