@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """This module provides a small Python layer on top of the C++
 VariationalProblem/Solver classes as well as the solve function."""
 
@@ -20,15 +21,6 @@ VariationalProblem/Solver classes as well as the solve function."""
 #
 # Modified by Marie E. Rognes, 2011.
 # Modified by Johan Hake, 2011.
-#
-# First added:  2011-06-22
-# Last changed: 2011-11-10
-
-__all__ = ["LinearVariationalProblem",
-           "LinearVariationalSolver",
-           "NonlinearVariationalProblem",
-           "NonlinearVariationalSolver",
-           "solve"]
 
 # Import SWIG-generated extension module (DOLFIN C++)
 import dolfin.cpp as cpp
@@ -39,8 +31,18 @@ import ufl
 # Local imports
 from dolfin.fem.form import Form
 from dolfin.fem.formmanipulations import derivative
+import six
+
+__all__ = ["LinearVariationalProblem",
+           "LinearVariationalSolver",
+           "LocalSolver",
+           "NonlinearVariationalProblem",
+           "NonlinearVariationalSolver",
+           "solve"]
+
+# Problem classes need special handling since they involve JIT
+# compilation
 
-# Problem classes need special handling since they involve JIT compilation
 
 class LinearVariationalProblem(cpp.LinearVariationalProblem):
 
@@ -82,6 +84,37 @@ class LinearVariationalProblem(cpp.LinearVariationalProblem):
         # Initialize C++ base class
         cpp.LinearVariationalProblem.__init__(self, a, L, u, bcs)
 
+
+class LocalSolver(cpp.LocalSolver):
+
+    # Reuse C++ doc-string
+    __doc__ = cpp.LocalSolver.__doc__
+
+    def __init__(self, a, L=None, solver_type=cpp.LocalSolver.SolverType_LU):
+        """Create a local (cell-wise) solver for a linear variational problem
+        a(u, v) = L(v).
+
+        """
+
+        # Store input UFL forms and solution Function
+        self.a_ufl = a
+        self.L_ufl = L
+
+        # Wrap as DOLFIN forms
+        a = Form(a)
+        if L is None:
+            # Initialize C++ base class
+            cpp.LocalSolver.__init__(self, a, solver_type)
+        else:
+            if L.empty():
+                L = cpp.Form(1, 0)
+            else:
+                L = Form(L)
+
+            # Initialize C++ base class
+            cpp.LocalSolver.__init__(self, a, L, solver_type)
+
+
 class NonlinearVariationalProblem(cpp.NonlinearVariationalProblem):
 
     # Reuse C++ doc-string
@@ -118,18 +151,16 @@ class NonlinearVariationalProblem(cpp.NonlinearVariationalProblem):
             J = Form(J, form_compiler_parameters=form_compiler_parameters)
 
         # Initialize C++ base class
-        if J is None:
-            cpp.NonlinearVariationalProblem.__init__(self, F, u, bcs)
-        else:
-            cpp.NonlinearVariationalProblem.__init__(self, F, u, bcs, J)
+        cpp.NonlinearVariationalProblem.__init__(self, F, u, bcs, J)
+
 
 # Solver classes are imported directly
 from dolfin.cpp import LinearVariationalSolver, NonlinearVariationalSolver
 from dolfin.fem.adaptivesolving import AdaptiveLinearVariationalSolver
 from dolfin.fem.adaptivesolving import AdaptiveNonlinearVariationalSolver
 
-# Solve function handles both linear systems and variational problems
 
+# Solve function handles both linear systems and variational problems
 def solve(*args, **kwargs):
     """Solve linear system Ax = b or variational problem a == L or F == 0.
 
@@ -159,26 +190,26 @@ def solve(*args, **kwargs):
 
     .. code-block:: python
 
-        list_lu_methods()
+        list_lu_solver_methods()
 
     To list all available Krylov methods, run the following command:
 
     .. code-block:: python
 
-        list_krylov_methods()
+        list_krylov_solver_methods()
 
     To list all available preconditioners, run the following command:
 
     .. code-block:: python
 
-        list_preconditioners()
+        list_krylov_solver_preconditioners()
 
     To list all available solver methods, including LU methods, Krylov
     methods and, possibly, other methods, run the following command:
 
     .. code-block:: python
 
-        list_solver_methods()
+        list_linear_solver_methods()
 
     *2. Solving linear variational problems*
 
@@ -202,7 +233,7 @@ def solve(*args, **kwargs):
 
     .. code-block:: python
 
-        info(LinearVariationalSolver.default_parameters(), 1)
+        info(LinearVariationalSolver.default_parameters(), True)
 
     *3. Solving nonlinear variational problems*
 
@@ -228,9 +259,9 @@ def solve(*args, **kwargs):
 
     For available choices for the 'solver_parameters' kwarg, look at:
 
-        .. code-block:: python
+    .. code-block:: python
 
-        info(NonlinearVariationalSolver.default_parameters(), 1)
+        info(NonlinearVariationalSolver.default_parameters(), True)
 
     *4. Solving linear/nonlinear variational problems adaptively
 
@@ -263,7 +294,8 @@ def solve(*args, **kwargs):
     if "tol" in kwargs:
         _solve_varproblem_adaptive(*args, **kwargs)
 
-    # Call variational problem solver if we get an equation (but not a tolerance)
+    # Call variational problem solver if we get an equation (but not a
+    # tolerance)
     elif isinstance(args[0], ufl.classes.Equation):
         _solve_varproblem(*args, **kwargs)
 
@@ -272,11 +304,12 @@ def solve(*args, **kwargs):
         if kwargs:
             cpp.dolfin_error("solving.py",
                              "solve linear algebra problem",
-                             "Not expecting keyword arguments when solving "\
+                             "Not expecting keyword arguments when solving "
                              "linear algebra problem")
 
         return cpp.la_solve(*args)
 
+
 def _solve_varproblem(*args, **kwargs):
     "Solve variational problem a == L or F == 0"
 
@@ -289,7 +322,7 @@ def _solve_varproblem(*args, **kwargs):
 
         # Create problem
         problem = LinearVariationalProblem(eq.lhs, eq.rhs, u, bcs,
-                    form_compiler_parameters=form_compiler_parameters)
+                                           form_compiler_parameters=form_compiler_parameters)
 
         # Create solver and call solve
         solver = LinearVariationalSolver(problem)
@@ -308,19 +341,20 @@ def _solve_varproblem(*args, **kwargs):
 
         # Create problem
         problem = NonlinearVariationalProblem(eq.lhs, u, bcs, J,
-                    form_compiler_parameters=form_compiler_parameters)
+                                              form_compiler_parameters=form_compiler_parameters)
 
         # Create solver and call solve
         solver = NonlinearVariationalSolver(problem)
         solver.parameters.update(solver_parameters)
         solver.solve()
 
+
 def _solve_varproblem_adaptive(*args, **kwargs):
     "Solve variational problem a == L or F == 0 adaptively"
 
     # Extract arguments
-    eq, u, bcs, J, tol, M, form_compiler_parameters, solver_parameters \
-        = _extract_args(*args, **kwargs)
+    eq, u, bcs, J, tol, M, form_compiler_parameters, \
+        solver_parameters = _extract_args(*args, **kwargs)
 
     # Check that we received the goal functional
     if M is None:
@@ -333,7 +367,7 @@ def _solve_varproblem_adaptive(*args, **kwargs):
 
         # Create problem
         problem = LinearVariationalProblem(eq.lhs, eq.rhs, u, bcs,
-                    form_compiler_parameters=form_compiler_parameters)
+                                           form_compiler_parameters=form_compiler_parameters)
 
         # Create solver and call solve
         solver = AdaptiveLinearVariationalSolver(problem, M)
@@ -352,37 +386,38 @@ def _solve_varproblem_adaptive(*args, **kwargs):
 
         # Create problem
         problem = NonlinearVariationalProblem(eq.lhs, u, bcs, J,
-                    form_compiler_parameters=form_compiler_parameters)
+                                              form_compiler_parameters=form_compiler_parameters)
 
         # Create solver and call solve
         solver = AdaptiveNonlinearVariationalSolver(problem, M)
         solver.parameters.update(solver_parameters)
         solver.solve(tol)
 
+
 def _extract_args(*args, **kwargs):
     "Common extraction of arguments for _solve_varproblem[_adaptive]"
 
     # Check for use of valid kwargs
     valid_kwargs = ["bcs", "J", "tol", "M",
                     "form_compiler_parameters", "solver_parameters"]
-    for kwarg in kwargs.iterkeys():
-        if not kwarg in valid_kwargs:
+    for kwarg in six.iterkeys(kwargs):
+        if kwarg not in valid_kwargs:
             cpp.dolfin_error("solving.py",
                              "solve variational problem",
-                             "Illegal keyword argument \"%s\"; valid keywords are %s" % \
-                                 (kwarg,
-                                  ", ".join("\"%s\"" % kwarg for kwarg in valid_kwargs)))
+                             "Illegal keyword argument \"%s\"; valid keywords are %s" %
+                             (kwarg,
+                              ", ".join("\"%s\"" % kwarg for kwarg in valid_kwargs)))
 
     # Extract equation
     if not len(args) >= 2:
         cpp.dolfin_error("solving.py",
                          "solve variational problem",
-                         "Missing arguments, expecting solve(lhs == rhs, "\
+                         "Missing arguments, expecting solve(lhs == rhs, "
                          "u, bcs=bcs), where bcs is optional")
     if len(args) > 3:
         cpp.dolfin_error("solving.py",
                          "solve variational problem",
-                         "Too many arguments, expecting solve(lhs == rhs, "\
+                         "Too many arguments, expecting solve(lhs == rhs, "
                          "u, bcs=bcs), where bcs is optional")
 
     # Extract equation
@@ -426,6 +461,7 @@ def _extract_args(*args, **kwargs):
 
     return eq, u, bcs, J, tol, M, form_compiler_parameters, solver_parameters
 
+
 def _extract_eq(eq):
     "Extract and check argument eq"
     if not isinstance(eq, ufl.classes.Equation):
@@ -434,6 +470,7 @@ def _extract_eq(eq):
                          "Expecting first argument to be an Equation")
     return eq
 
+
 def _extract_u(u):
     "Extract and check argument u"
     if not isinstance(u, cpp.Function):
@@ -442,6 +479,7 @@ def _extract_u(u):
                          "Expecting second argument to be a Function")
     return u
 
+
 def _extract_bcs(bcs):
     "Extract and check argument bcs"
     if bcs is None:
