@@ -1,12 +1,14 @@
 # -*- coding: utf-8 -*-
 """The function module of dolfin"""
 
+from dolfin.functions import multimeshfunction
 from dolfin.functions import functionspace
 from dolfin.functions import function
 from dolfin.functions import constant
 from dolfin.functions import expression
 from dolfin.functions import specialfunctions
 
+from .multimeshfunction import *
 from .functionspace import *
 from .function import *
 from .constant import *
@@ -18,4 +20,5 @@ from .specialfunctions  import *
 # NOTE: purpose.
 
 __all__ = functionspace.__all__ + function.__all__ + constant.__all__ + \
-          expression.__all__ + specialfunctions.__all__
+          expression.__all__ + specialfunctions.__all__ + \
+          multimeshfunction.__all__
