@@ -1,6 +1,7 @@
+# -*- coding: utf-8 -*-
 """Create a constant-valued function with given value."""
 
-# Copyright (C) 2008-2009 Anders Logg
+# Copyright (C) 2008-2015 Anders Logg
 #
 # This file is part of DOLFIN.
 #
@@ -18,9 +19,7 @@
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Johan Hake, 2008.
-#
-# First added:  2008-12-02
-# Last changed: 2010-08-16
+# Modified by Martin Sandve Alnæs, 2008.
 
 from __future__ import print_function
 
@@ -65,13 +64,11 @@ class Constant(ufl.Coefficient, cpp.Constant):
 
         """
 
-        if cell is None:
-            domain = None
-        else:
-            #deprecation_warning("The cell argument is no longer necessary.")
-            domain = ufl.as_domain(cell)
-            if not isinstance(domain, ufl.Domain):
-                raise TypeError("Expected an ufl.Domain as the second argument")
+        # TODO: Either take mesh instead of cell, or drop cell and let
+        # grad(c) be undefined.
+        if cell is not None:
+            cell = ufl.as_cell(cell)
+        ufl_domain = None
 
         array = numpy.array(value)
         rank = len(array.shape)
@@ -79,28 +76,29 @@ class Constant(ufl.Coefficient, cpp.Constant):
 
         # Create UFL element and initialize constant
         if rank == 0:
-            self._ufl_element = ufl.FiniteElement("Real",
-                                                  domain, 0)
+            ufl_element = ufl.FiniteElement("Real", cell, 0)
             cpp.Constant.__init__(self, floats[0])
         elif rank == 1:
-            self._ufl_element = ufl.VectorElement("Real",
-                                                  domain, 0, len(floats))
+            ufl_element = ufl.VectorElement("Real", cell, 0, dim=len(floats))
             cpp.Constant.__init__(self, floats)
         else:
-            self._ufl_element = ufl.TensorElement("Real",
-                                                  domain, 0, shape=array.shape)
+            ufl_element = ufl.TensorElement("Real", cell, 0, shape=array.shape)
             cpp.Constant.__init__(self, list(array.shape), floats)
 
         # Initialize base classes
-        ufl.Coefficient.__init__(self, self._ufl_element, count=self.id())
+        ufl_function_space = ufl.FunctionSpace(ufl_domain, ufl_element)
+        ufl.Coefficient.__init__(self, ufl_function_space, count=self.id())
 
         # Set name as given or automatic
         name = name or "f_%d" % self.count()
         self.rename(name, "a Constant")
 
+    def cell(self):
+        return self.ufl_element().cell()
+
     def __float__(self):
         # Overriding UFL operator in this particular case.
-        if self.shape():
+        if self.ufl_shape:
             raise TypeError("Cannot convert nonscalar constant to float.")
         return cpp.Constant.__float__(self)
 
