@@ -69,6 +69,9 @@ __all__ = ["Expression"]
 import types
 from six import add_metaclass # Requires newer six version than some buildbots have
 from six import string_types
+from six.moves import xrange as range
+from functools import reduce
+import weakref
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
@@ -81,8 +84,6 @@ from dolfin import warning, error
 
 # Local imports
 from dolfin.compilemodules.expressions import compile_expressions
-from functools import reduce
-from six.moves import xrange as range
 
 
 # Local help class
@@ -101,7 +102,7 @@ class UserDefinedParameters(dict):
         if name not in self:
             raise KeyError("'%s' is not a parameter name" % name)
         setattr(self._expr, name, value)
-        dict.__setattr__(self, name, value)
+        dict.__setitem__(self, name, value)
 
     def update(self, other):
         __doc__ = dict.update.__doc__
@@ -181,7 +182,8 @@ def create_compiled_expression_class(cpp_base):
         self.rename(name, label)
 
         # Store the rest of the kwargs (parameters)
-        self.user_parameters = UserDefinedParameters(self, **kwargs)
+        weak_self = weakref.proxy(self)  # Avoid circular reference
+        self.user_parameters = UserDefinedParameters(weak_self, **kwargs)
 
     # Create and return the class
     return type("CompiledExpression", (Expression, ufl.Coefficient, cpp_base),
@@ -375,6 +377,19 @@ class ExpressionMetaClass(type):
         return create_python_derived_expression_class(class_name, user_bases, dict_)
 
 
+def _value_shape(value):
+    return tuple(value.value_dimension(i) for i in range(value.value_rank()))
+
+
+def _is_cell_mesh_function(value):
+    name = value.__class__.__name__
+    if name.startswith("CellFunction"):
+        return True
+    if name.startswith("MeshFunction"):
+        return value.dim() == value.mesh().topology().dim()
+    return False
+
+
 # --- The user interface ---
 
 # Places here so it can be reused in Function
@@ -644,19 +659,29 @@ class Expression(object):
         # Check arguments
         _check_cppcode(cppcode)
 
-        # Extract any Constants from the kwargs
-        generic_function_members = [arg_name for arg_name, value in list(kwargs.items()) \
-                                    if isinstance(value, cpp.GenericFunction)]
+        # Extract any GenericFunctions from the kwargs
+        generic_function_members = [
+            (arg_name, _value_shape(value))
+            for arg_name, value in sorted(kwargs.items())
+            if isinstance(value, cpp.GenericFunction)]
+
+        # Extract any MeshFunctions from thw kwargs
+        # (only cell based currently allowed)
+        mesh_function_members = [
+            (arg_name, value.cpp_value_type())
+            for arg_name, value in sorted(kwargs.items())
+            if _is_cell_mesh_function(value)]
 
         # Compile module and get the cpp.Expression class
         cpp_base, members = compile_expressions([cppcode],
                                                 [generic_function_members],
+                                                [mesh_function_members],
                                                 mpi_comm=kwargs.get("mpi_comm"))
         cpp_base, members = cpp_base[0], members[0]
 
         # Check passed default arguments
-        not_allowed = [n for n in dir(cls) if n[0] != "_"]
-        not_allowed += ["cppcode", "user_parameters"]
+        not_allowed = [n for kl in (cls, ufl.Coefficient, cpp.Expression) for n in dir(kl)]
+        not_allowed += ["cppcode", "user_parameters", "_value_shape"]
         valid_kws = ["element", "cell", "domain", "degree", "name", "label", "mpi_comm"]
         _check_kwargs(members, kwargs, not_allowed, valid_kws)
 
@@ -894,16 +919,15 @@ def _check_kwargs(members, kwargs, not_allowed, valid):
             if member not in kwargs:
                 missing.append(member)
         missing = ", ".join("'%s'" % miss for miss in missing)
-        raise RuntimeError("expected a default value to all member "\
+        raise RuntimeError("expected a default value to all member "
                            "variables in the Expression. Missing: %s." % missing)
 
     for name, value in kwargs.items():
         if name in valid:
             continue
         if name in not_allowed:
-            raise RuntimeError("Parameter name: '%s' is not allowed. It is "\
+            raise RuntimeError("Parameter name: '%s' is not allowed. It is "
                                "part of the interface of Expression" % name)
-        if not (isinstance(value, (int, float)) or
-                (isinstance(value, cpp.GenericFunction) and value.value_size() == 1)):
-            raise TypeError("expected default arguments for member variables "\
-                            "to be scalars or a scalar GenericFunctions.")
+        if not (isinstance(value, (int, float, cpp.GenericFunction)) or _is_cell_mesh_function(value)):
+            raise TypeError("expected default arguments for member variables "
+                            "to be scalars, GenericFunctions, or CellFunctions.")
