@@ -24,6 +24,7 @@
 __all__ = ["Function", "TestFunction", "TrialFunction", "Argument",
            "TestFunctions", "TrialFunctions"]
 
+from six import string_types
 import types
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
@@ -257,11 +258,8 @@ class Function(ufl.Coefficient, cpp.Function):
         if isinstance(args[0], Function):
             other = args[0]
             if len(args) == 1:
-                # NOTE: Turn this into error when removing deprecation
-                # warning
-                cpp.deprecation("Function copy constructor", "2016.1",
-                                "Use 'Function.copy(deepcopy=True)' for copying.")
-                self.__init_copy_constructor(other)
+                # Copy constructor used to be here
+                raise RuntimeError("Use 'Function.copy(deepcopy=True)' for copying.")
             elif len(args) == 2:
                 i = args[1]
                 if not isinstance(i, int):
@@ -302,10 +300,6 @@ class Function(ufl.Coefficient, cpp.Function):
         name = kwargs.get("name") or "f_%d" % self.count()
         self.rename(name, "a Function")
 
-    def __init_copy_constructor(self, other):
-        cpp.Function.__init__(self, other)
-        ufl.Coefficient.__init__(self, other.ufl_function_space(), count=self.id())
-
     def __init_from_cpp_function(self, other):
         # Assign all the members (including 'this' pointer to SWIG wraper)
         # NOTE: This in fact performs assignment of C++ context
@@ -548,7 +542,7 @@ class Function(ufl.Coefficient, cpp.Function):
             raise TypeError("expected at least 1 argument")
 
         # Test for ufl restriction
-        if len(args) == 1 and isinstance(args[0], str):
+        if len(args) == 1 and isinstance(args[0], string_types):
             if args[0] in ('+', '-'):
                 return ufl.Coefficient.__call__(self, *args)
 
