@@ -24,7 +24,9 @@
 # Modified by Martin Sandve Alnæs 2013-2014
 
 __all__ = ["MeshCoordinates", "FacetArea", "FacetNormal", "CellSize", "CellVolume",
-           'SpatialCoordinate', 'CellNormal', 'Circumradius', 'MinFacetEdgeLength', 'MaxFacetEdgeLength']
+           'SpatialCoordinate', 'CellNormal', 'CellDiameter', 'Circumradius',
+           'MinCellEdgeLength', 'MaxCellEdgeLength',
+           'MinFacetEdgeLength', 'MaxFacetEdgeLength']
 
 # Import UFL and SWIG-generated extension module (DOLFIN C++)
 import ufl
@@ -116,9 +118,12 @@ def FacetNormal(mesh):
     return ufl.FacetNormal(_mesh2domain(mesh))
 
 # Simple definition of CellSize via UFL
-def CellSize(mesh):
+def CellDiameter(mesh):
     """
-    Return function cell size for given mesh.
+    Return function cell diameter for given mesh.
+
+    Note that diameter of cell :math:`K` is defined as
+    :math:`\sup_{\mathbf{x,y}\in K} |\mathbf{x-y}|`.
 
     *Arguments*
         mesh
@@ -129,10 +134,36 @@ def CellSize(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            h = CellSize(mesh)
+            h = CellDiameter(mesh)
 
     """
+    return ufl.CellDiameter(_mesh2domain(mesh))
 
+# Simple definition of CellSize via UFL
+def CellSize(mesh):
+    """
+    Return function twice the cell circumradius for given mesh.
+
+    This does not generalize to quadrilateral/hexahedral cells
+    and the function name is misleading. The function is hence
+    deprecated. Use ``CellDiameter`` (or ``2*Circumradius`` if
+    you really want that).
+
+    *Arguments*
+        mesh
+            a :py:class:`Mesh <dolfin.cpp.Mesh>`.
+
+    *Example of usage*
+
+        .. code-block:: python
+
+            mesh = UnitSquare(4,4)
+            D = CellSize(mesh)
+
+    """
+    cpp.deprecation("\"CellSize\"",
+                    "2017.2.0",
+                    "Use \"CellDiameter\" or \"2*Circumradius\".")
     return 2.0*ufl.Circumradius(_mesh2domain(mesh))
 
 # Simple definition of CellVolume via UFL
@@ -169,7 +200,7 @@ def SpatialCoordinate(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            vol = SpatialCoordinate(mesh)
+            x = SpatialCoordinate(mesh)
 
     """
 
@@ -189,7 +220,7 @@ def CellNormal(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            vol = CellNormal(mesh)
+            n = CellNormal(mesh)
 
     """
 
@@ -209,12 +240,52 @@ def Circumradius(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            vol = Circumradius(mesh)
+            R = Circumradius(mesh)
 
     """
 
     return ufl.Circumradius(_mesh2domain(mesh))
 
+# Simple definition of MinCellEdgeLength via UFL
+def MinCellEdgeLength(mesh):
+    """
+    Return symbolic minimum cell edge length of a cell for given mesh.
+
+    *Arguments*
+        mesh
+            a :py:class:`Mesh <dolfin.cpp.Mesh>`.
+
+    *Example of usage*
+
+        .. code-block:: python
+
+            mesh = UnitSquare(4,4)
+            mince = MinCellEdgeLength(mesh)
+
+    """
+
+    return ufl.MinCellEdgeLength(_mesh2domain(mesh))
+
+# Simple definition of MaxCellEdgeLength via UFL
+def MaxCellEdgeLength(mesh):
+    """
+    Return symbolic maximum cell edge length of a cell for given mesh.
+
+    *Arguments*
+        mesh
+            a :py:class:`Mesh <dolfin.cpp.Mesh>`.
+
+    *Example of usage*
+
+        .. code-block:: python
+
+            mesh = UnitSquare(4,4)
+            maxce = MaxCellEdgeLength(mesh)
+
+    """
+
+    return ufl.MaxCellEdgeLength(_mesh2domain(mesh))
+
 # Simple definition of MinFacetEdgeLength via UFL
 def MinFacetEdgeLength(mesh):
     """
@@ -229,7 +300,7 @@ def MinFacetEdgeLength(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            vol = MinFacetEdgeLength(mesh)
+            minfe = MinFacetEdgeLength(mesh)
 
     """
 
@@ -249,7 +320,7 @@ def MaxFacetEdgeLength(mesh):
         .. code-block:: python
 
             mesh = UnitSquare(4,4)
-            vol = MaxFacetEdgeLength(mesh)
+            maxfe = MaxFacetEdgeLength(mesh)
 
     """
 
