@@ -34,7 +34,7 @@ if "linux" in sys.platform:
     sys.setdlopenflags(RTLD_NOW | RTLD_GLOBAL)
 
 # Check for runtime dependencies
-runtime_dependencies = ["ufl", "ffc", "ufc", "ply.lex"]
+runtime_dependencies = ["ufl", "ffc", "ply.lex"]
 
 for dep in runtime_dependencies:
     try:
