@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 # Copyright (C) 2009-2011 Anders Logg
 #
 # This file is part of DOLFIN.
@@ -42,10 +43,10 @@ def compute_vertex_map(mesh0, mesh1):
     """
 
     # Check arguments
-    if not isinstance(mesh0, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
-    if not isinstance(mesh1, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
+    if not isinstance(mesh0, cpp.Mesh) or not isinstance(mesh1, cpp.Mesh):
+        cpp.dolfin_error("ale.py",
+                         "compute vertex map",
+                         "Expected 'Mesh' as argument")
 
     # Get parent vertex numbers
     vertices0 = mesh0.data().array("parent_vertex_indices", 0)
@@ -89,10 +90,10 @@ def compute_edge_map(mesh0, mesh1):
     """
 
     # Check arguments
-    if not isinstance(mesh0, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
-    if not isinstance(mesh1, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
+    if not isinstance(mesh0, cpp.Mesh) or not isinstance(mesh1, cpp.Mesh):
+        cpp.dolfin_error("ale.py",
+                         "compute edge map",
+                         "Expected 'Mesh' as argument")
 
     # Get parent vertex numbers
     vertices0 = mesh0.data().array("parent_vertex_indices", 0)
@@ -136,10 +137,10 @@ def init_parent_edge_indices(submesh, mesh):
     "Initialize data 'parent_edge_indices' for submesh."
 
     # Check arguments
-    if not isinstance(submesh, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
-    if not isinstance(mesh, Mesh):
-        raise TypeError("expected 'Mesh' as argument")
+    if not isinstance(submesh, cpp.Mesh) or not isinstance(mesh, cpp.Mesh):
+        cpp.dolfin_error("ale.py",
+                         "initialize parent edge indices",
+                         "Expected 'Mesh' as argument")
 
     # Check if edge map has already been computed
     if not submesh.data().exists("parent_edge_indices", 1):
