@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 "This module provides a simple Python wrapper for the SWIG-generated Mesh class"
 
 # Copyright (C) 2012-2014 Anders Logg
@@ -18,10 +19,13 @@
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Marie E. Rognes (meg@simula.no)
-# Modified by Martin S. Alnaes (martinal@simula.no) 2014
+# Modified by Martin Sandve Alnæs (martinal@simula.no) 2014
 
 __all__ = []
 
+# Import ufl (used by Mesh extensions in dolfin/swig/mesh/post.i)
+import ufl
+
 # Import SWIG-generated extension module (DOLFIN C++)
 import dolfin.cpp as cpp
 
@@ -62,6 +66,10 @@ def __init__(self, *args, **kwargs):
 cpp.Mesh.__cppinit__ = cpp.Mesh.__init__
 cpp.Mesh.__init__ = __init__
 
+def mesh2html(mesh, display_width=800.0):
+    return cpp.X3DOM.html(mesh)
+
 # Attach ipython notebook svg rendering function for 1D and 2D meshes
 from dolfin.mesh.svgtools import mesh2svg
 cpp.Mesh._repr_svg_ = mesh2svg
+cpp.Mesh._repr_html_ = mesh2html
