@@ -92,7 +92,7 @@ def mesh2svg(mesh, display_width=800.0):
     # Render svg code
     radius = strokewidth
 
-    svg_line = '<line x1="%%s" y1="%%s" x2="%%s" y2="%%s" style="fill:none; stroke:black; stroke-width:"%s" />' % (strokewidth,)
+    svg_line = '<line x1="%%s" y1="%%s" x2="%%s" y2="%%s" style="fill:none; stroke:black; stroke-width:%s" />' % (strokewidth,)
     svg_lines = "\n".join(svg_line % (line[0] + line[1]) for line in lines)
 
     svg_vertex = '<circle cx="%%d" cy="%%d" r="%d" stroke="black" stroke-width="0" fill="red" />' % (radius,)
