@@ -1,7 +1,8 @@
+# -*- coding: utf-8 -*-
 """This module defines different MultiStageScheme classes which can be
-passed to a RKSolver"""
+passed to a RKSolver or PointIntegralSolver"""
 
-# Copyright (C) 2013 Johan Hake
+# Copyright (C) 2013-2015 Johan Hake
 #
 # This file is part of DOLFIN.
 #
@@ -19,9 +20,7 @@ passed to a RKSolver"""
 # along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 #
 # Modified by Patrick Farrell, 2013
-#
-# First added:  2013-02-22
-# Last changed: 2013-06-18
+# Modified by Martin Sandve Alnæs, 2015
 
 import numpy as np
 import functools
@@ -35,25 +34,30 @@ import ufl
 # Import classes from dolfin python layer
 from dolfin.functions.constant import Constant
 from dolfin.functions.expression import Expression
-from dolfin.functions.function import Function
+from dolfin.functions.function import Function, TestFunction
 from dolfin.fem.formmanipulations import derivative, adjoint
+from dolfin.multistage.factorize import extract_tested_expressions
 from ufl import action as ufl_action
 from dolfin.fem.form import Form
 import ufl.algorithms
+from ufl.algorithms import expand_derivatives
 
 # FIXME: Add support for algebraic parts (at least for implicit)
 # FIXME: Add support for implicit/explicit split ala IMEX schemes
 
+
 def safe_adjoint(x):
-    return adjoint(x, reordered_arguments=ufl.algorithms.extract_arguments(x))
+    return adjoint(x, reordered_arguments=x.arguments())
+
 
 def safe_action(x, y):
-    x = ufl.algorithms.expand_derivatives(x)
+    x = expand_derivatives(x)
     if x.integrals() == ():
         return x # form is empty, return anyway
     else:
         return ufl_action(x, y)
 
+
 def _check_abc(a, b, c):
     if not (isinstance(a, np.ndarray) and (len(a) == 1 or \
             (len(a.shape)==2 and a.shape[0] == a.shape[1]))):
@@ -93,7 +97,7 @@ def _check_abc(a, b, c):
             elif sigma != a[i,i]:
                 raise ValueError("Expected only singly diagonally implicit "
                                  "schemes. (Same value on the diagonal of 'a'.)")
-            
+
     # Check if tableau is fully implicit
     for i in range(size):
         for j in range(i):
@@ -102,6 +106,7 @@ def _check_abc(a, b, c):
 
     return a
 
+
 def _check_form(rhs_form):
     if not isinstance(rhs_form, ufl.Form):
         raise TypeError("Expected a ufl.Form as the 5th argument.")
@@ -109,27 +114,31 @@ def _check_form(rhs_form):
     # Check if form contains a cell or point integral
     if rhs_form.integrals_by_type("cell"):
         DX = ufl.dx
-    elif rhs_form.integrals_by_type("point"):
+    elif rhs_form.integrals_by_type("vertex"):
         DX = ufl.dP
     else:
-        raise ValueError("Expected either a cell or point integral in the form.")
+        raise ValueError("Expected either a cell or vertex integral in the form.")
 
-    arguments = ufl.algorithms.extract_arguments(rhs_form)
+    if len(rhs_form.integrals()) != 1:
+        raise ValueError("Expected only one integral in form.")
+
+    arguments = rhs_form.arguments()
     if len(arguments) != 1:
         raise ValueError("Expected the form to have rank 1")
 
     return DX
 
+
 def _time_dependent_expressions(rhs_form, time):
     """
     Return a list of expressions which uses the present time as a parameter
     """
     # FIXME: Add extraction of time dependant expressions from bcs too
     time_dependent_expressions = dict()
-  
-    for coefficient in ufl.algorithms.extract_coefficients(rhs_form):
+
+    for coefficient in rhs_form.coefficients():
         if hasattr(coefficient, "user_parameters"):
-            for c_name, c in coefficient.user_parameters.items():
+            for c_name, c in list(coefficient.user_parameters.items()):
                 if isinstance(c, cpp.GenericFunction) and time.id() == c.id():
                     if coefficient not in time_dependent_expressions:
                         time_dependent_expressions[coefficient] = [c_name]
@@ -137,26 +146,28 @@ def _time_dependent_expressions(rhs_form, time):
                         time_dependent_expressions[coefficient].append(c_name)
 
     return time_dependent_expressions
-          
+
+
 def _replace_dict_time_dependent_expression(time_dep_expressions, time, dt, c):
     assert(isinstance(c, float))
     replace_dict = {}
     if c == 0.0 or not time_dep_expressions:
         return replace_dict
-    new_time = Expression("time+c*dt", time=time, c=c, dt=dt)
-    for expr, c_names in time_dep_expressions.items():
+    new_time = Expression("time + c*dt", time=time, c=c, dt=dt, degree=0)
+    for expr, c_names in list(time_dep_expressions.items()):
         assert(isinstance(expr, Expression))
-        kwargs = dict(name=expr.name(), label=expr.label(), \
+        kwargs = dict(name=expr.name(), label=expr.label(),
                       element=expr.ufl_element(), **expr.user_parameters)
         for c_name in c_names:
             kwargs[c_name] = new_time
         replace_dict[expr] = Expression(expr.cppcode, **kwargs)
-        
+
     return replace_dict
-    
+
+
 def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
     """
-    Generates a list of forms and solutions for a given Butcher tableau 
+    Generates a list of forms and solutions for a given Butcher tableau
 
     *Arguments*
         a (2 dimensional numpy array)
@@ -172,7 +183,7 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
         solution (_Function_)
             The prognostic variable
         rhs_form (ufl.Form)
-            A UFL form representing the rhs for a time differentiated equation 
+            A UFL form representing the rhs for a time differentiated equation
     """
 
     a = _check_abc(a, b, c)
@@ -181,8 +192,8 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
     DX = _check_form(rhs_form)
 
     # Get test function
-    arguments, coefficients = ufl.algorithms.\
-                              extract_arguments_and_coefficients(rhs_form)
+    arguments = rhs_form.arguments()
+    coefficients = rhs_form.coefficients()
     v = arguments[0]
 
     # Create time step
@@ -201,7 +212,7 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
     y_ = solution
     time_ = time
     time_dep_expressions = _time_dependent_expressions(rhs_form, time)
-    zero_ = ufl.zero(*y_.shape())
+    zero_ = ufl.zero(*y_.ufl_shape)
     for i, ki in enumerate(k):
 
         # Check whether the stage is explicit
@@ -212,7 +223,7 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
                                   for j in range(i+1)], zero_)
         time = time_ + dt*c[i]
 
-        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions, \
+        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions,
                                                                time_, dt, c[i])
 
         replace_dict[y_] = evalargs
@@ -236,13 +247,13 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
         last_stage = Form(ufl.inner(y_+sum([dt*float(bi)*ki for bi, ki in \
                                             zip(b, k)], zero_), v)*DX)
     else:
-        # FIXME: Add support for addaptivity in RKSolver and MultiStageScheme
-        
+        # FIXME: Add support for adaptivity in RKSolver and MultiStageScheme
+
         last_stage = [Form(ufl.inner(y_+sum([dt*float(bi)*ki for bi, ki in \
                                              zip(b[0,:], k)], zero_), v)*DX),
                       Form(ufl.inner(y_+sum([dt*float(bi)*ki for bi, ki in \
                                              zip(b[1,:], k)], zero_), v)*DX)]
-    
+
     # Create the Function holding the solution at end of time step
     #k.append(solution.copy())
 
@@ -273,9 +284,10 @@ def _butcher_scheme_generator(a, b, c, time, solution, rhs_form):
     return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
            k, dt, human_form, None
 
+
 def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbation):
     """
-    Generates a list of forms and solutions for a given Butcher tableau 
+    Generates a list of forms and solutions for a given Butcher tableau
 
     *Arguments*
         a (2 dimensional numpy array)
@@ -291,7 +303,7 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
         solution (_Function_)
             The prognostic variable
         rhs_form (ufl.Form)
-            A UFL form representing the rhs for a time differentiated equation 
+            A UFL form representing the rhs for a time differentiated equation
         perturbation (_Function_)
             The perturbation in the initial condition of the solution
     """
@@ -302,8 +314,8 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
     DX = _check_form(rhs_form)
 
     # Get test function
-    arguments, coefficients = ufl.algorithms.\
-                              extract_arguments_and_coefficients(rhs_form)
+    arguments = rhs_form.arguments()
+    coefficients = rhs_form.coefficients()
     v = arguments[0]
 
     # Create time step
@@ -312,7 +324,7 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
     # rhs forms
     dolfin_stage_forms = []
     ufl_stage_forms = []
-    
+
     # Stage solutions
     k = [Function(solution.function_space(), name="k_%d"%i) for i in range(size)]
     kdot = [Function(solution.function_space(), name="kdot_%d"%i) \
@@ -320,10 +332,9 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
 
     # Create the stage forms
     y_ = solution
-    ydot = solution.copy()
     time_ = time
     time_dep_expressions = _time_dependent_expressions(rhs_form, time)
-    zero_ = ufl.zero(*y_.shape())
+    zero_ = ufl.zero(*y_.ufl_shape)
     forward_forms = []
     stage_solutions = []
     jacobian_indices = []
@@ -337,8 +348,8 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
         evalargs = y_ + dt * sum([float(a[i,j]) * k[j] \
                                   for j in range(i+1)], zero_)
         time = time_ + dt*c[i]
-        
-        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions, \
+
+        replace_dict = _replace_dict_time_dependent_expression(time_dep_expressions,
                                                                time_, dt, c[i])
 
         replace_dict[y_] = evalargs
@@ -378,14 +389,14 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
         ufl_stage_forms.append(stage_forms_tlm)
         dolfin_stage_forms.append([Form(form) for form in stage_forms_tlm])
         stage_solutions.append(kdot[i])
-        
+
     # Only one last stage
     if len(b.shape) == 1:
         last_stage = Form(ufl.inner(perturbation + sum(\
             [dt*float(bi)*kdoti for bi, kdoti in zip(b, kdot)], zero_), v)*DX)
     else:
         raise Exception("Not sure what to do here")
-    
+
     human_form = []
     for i in range(size):
         kterm = " + ".join("%sh*k_%s" % ("" if a[i,j] == 1.0 else \
@@ -403,7 +414,7 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
                                "cih": cih,
                                "kterm": kterm} \
                               for j in range(size) if a[i,j] != 0)
-            
+
         if len(kterm) == 0:
             human_form.append("k_%(i)s = f(t_n%(cih)s, y_n)" % {"i": i, "cih": cih})
             human_form.append("kdot_%(i)s = action(derivative("\
@@ -424,11 +435,12 @@ def _butcher_scheme_generator_tlm(a, b, c, time, solution, rhs_form, perturbatio
     human_form = "\n".join(human_form)
 
     return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
-           stage_solutions, dt, human_form, perturbation
+        stage_solutions, dt, human_form, perturbation
+
 
 def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
     """
-    Generates a list of forms and solutions for a given Butcher tableau 
+    Generates a list of forms and solutions for a given Butcher tableau
 
     *Arguments*
         a (2 dimensional numpy array)
@@ -444,7 +456,7 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
         solution (_Function_)
             The prognostic variable
         rhs_form (ufl.Form)
-            A UFL form representing the rhs for a time differentiated equation 
+            A UFL form representing the rhs for a time differentiated equation
         adj (_Function_)
             The derivative of the functional with respect to y_n+1
     """
@@ -455,8 +467,8 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
     DX = _check_form(rhs_form)
 
     # Get test function
-    arguments, coefficients = ufl.algorithms.\
-                              extract_arguments_and_coefficients(rhs_form)
+    arguments = rhs_form.arguments()
+    coefficients = rhs_form.coefficients()
     v = arguments[0]
 
     # Create time step
@@ -465,7 +477,7 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
     # rhs forms
     dolfin_stage_forms = []
     ufl_stage_forms = []
-    
+
     # Stage solutions
     k = [Function(solution.function_space(), name="k_%d"%i) for i in range(size)]
     kbar = [Function(solution.function_space(), name="kbar_%d"%i) \
@@ -473,10 +485,9 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
 
     # Create the stage forms
     y_ = solution
-    ydot = solution.copy()
     time_ = time
     time_dep_expressions = _time_dependent_expressions(rhs_form, time)
-    zero_ = ufl.zero(*y_.shape())
+    zero_ = ufl.zero(*y_.ufl_shape)
     forward_forms = []
     stage_solutions = []
     jacobian_indices = []
@@ -491,7 +502,7 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
         evalargs = y_ + dt * sum([float(a[i,j]) * k[j] \
                                   for j in range(i+1)], zero_)
         time = time_ + dt*c[i]
-        
+
         replace_dict = _replace_dict_time_dependent_expression(\
             time_dep_expressions, time_, dt, c[i])
 
@@ -536,7 +547,7 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
         ufl_stage_forms.append(stage_forms_adm)
         dolfin_stage_forms.append([Form(form) for form in stage_forms_adm])
         stage_solutions.append(kbari)
-        
+
     # Only one last stage
     if len(b.shape) == 1:
         last_stage = Form(ufl.inner(adj, v)*DX + sum(\
@@ -544,43 +555,39 @@ def _butcher_scheme_generator_adm(a, b, c, time, solution, rhs_form, adj):
              for i in range(size)]))
     else:
         raise Exception("Not sure what to do here")
-    
+
     human_form = "unimplemented"
 
     return ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage,\
            stage_solutions, dt, human_form, adj
-    
+
 class MultiStageScheme(cpp.MultiStageScheme):
     """
     Base class for all MultiStageSchemes
     """
-    def __init__(self, rhs_form, solution, t, bcs, a, b, c, order, \
-                 generator=_butcher_scheme_generator):
-        bcs = bcs or []
-        t = t or Constant(0.0)
-        ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
-                         k, dt, human_form, contraction = generator(\
-            a, b, c, t, solution, rhs_form)
+    def __init__(self, rhs_form, ufl_stage_forms,
+                 dolfin_stage_forms, last_stage, stage_solutions,
+                 solution, time, dt, dt_stage_offsets, jacobian_indices, order,
+                 name, human_form, bcs, contraction=None):
 
-        # Store data
+        # Store Python data
         self._rhs_form = rhs_form
         self._ufl_stage_forms = ufl_stage_forms
         self._dolfin_stage_forms = dolfin_stage_forms
-        self._t = t
+        self._t = time
         self._bcs = bcs
         self._dt = dt
         self._last_stage = last_stage
         self._solution = solution
-        self._k = k
-        self.a = a
-        self.b = b
-        self.c = c
+        self._stage_solutions = stage_solutions
         self._order = order
-        self.contraction = contraction
         self.jacobian_indices = jacobian_indices
+        self.contraction = contraction
 
-        cpp.MultiStageScheme.__init__(self, dolfin_stage_forms, last_stage, k, \
-                                      solution, t, dt, c, jacobian_indices, order,\
+        # Pass args to C++ constructor
+        cpp.MultiStageScheme.__init__(self, dolfin_stage_forms, last_stage,
+                                      stage_solutions, solution, time, dt,
+                                      dt_stage_offsets, jacobian_indices, order,
                                       self.__class__.__name__,
                                       human_form, bcs)
 
@@ -614,7 +621,37 @@ class MultiStageScheme(cpp.MultiStageScheme):
 
     def stage_solutions(self):
         "Return the stage solutions"
-        return self._k
+        return self._stage_solutions
+    def to_tlm(self, perturbation):
+        raise NotImplementedError("'to_tlm:' implement in derived classes")
+
+    def to_adm(self, perturbation):
+        raise NotImplementedError("'to_adm:' implement in derived classes")
+
+
+class ButcherMultiStageScheme(MultiStageScheme):
+    """
+    Base class for all MultiStageSchemes
+    """
+    def __init__(self, rhs_form, solution, time, bcs, a, b, c, order,
+                 generator=_butcher_scheme_generator):
+        bcs = bcs or []
+        time = time or Constant(0.0)
+        ufl_stage_forms, dolfin_stage_forms, jacobian_indices, last_stage, \
+                         stage_solutions, dt, human_form, contraction = \
+                         generator(a, b, c, time, solution, rhs_form)
+
+        # Store data
+        self.a = a
+        self.b = b
+        self.c = c
+
+        MultiStageScheme.__init__(self, rhs_form, ufl_stage_forms,
+                                  dolfin_stage_forms, last_stage,
+                                  stage_solutions, solution, time, dt,
+                                  c, jacobian_indices, order,\
+                                  self.__class__.__name__, human_form,
+                                  bcs, contraction)
 
     def to_tlm(self, perturbation):
         r"""
@@ -626,14 +663,14 @@ class MultiStageScheme(cpp.MultiStageScheme):
         with respect to that parameter).
         """
 
-        generator = functools.partial(_butcher_scheme_generator_tlm, \
+        generator = functools.partial(_butcher_scheme_generator_tlm,
                                       perturbation=perturbation)
         new_solution = self._solution.copy()
         new_form = ufl.replace(self._rhs_form, {self._solution: new_solution})
-        return MultiStageScheme(new_form, new_solution, self._t, self._bcs, 
-                                self.a, self.b, self.c, self._order,
-                                generator=generator)
-        
+        return ButcherMultiStageScheme(new_form, new_solution, self._t, self._bcs,
+                                       self.a, self.b, self.c, self._order,
+                                       generator=generator)
+
     def to_adm(self, adj):
         r"""
         Return another MultiStageScheme that implements the adjoint
@@ -647,11 +684,12 @@ class MultiStageScheme(cpp.MultiStageScheme):
         generator = functools.partial(_butcher_scheme_generator_adm, adj=adj)
         new_solution = self._solution.copy()
         new_form = ufl.replace(self._rhs_form, {self._solution: new_solution})
-        return MultiStageScheme(new_form, new_solution, self._t, self._bcs, 
-                                self.a, self.b, self.c, self._order,
-                                generator=generator)
-        
-class ERK1(MultiStageScheme):
+        return ButcherMultiStageScheme(new_form, new_solution, self._t, self._bcs,
+                                       self.a, self.b, self.c, self._order,
+                                       generator=generator)
+
+
+class ERK1(ButcherMultiStageScheme):
     """
     Explicit first order Scheme
     """
@@ -659,9 +697,10 @@ class ERK1(MultiStageScheme):
         a = np.array([0.])
         b = np.array([1.])
         c = np.array([0.])
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 1)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 1)
 
-class BDF1(MultiStageScheme):
+
+class BDF1(ButcherMultiStageScheme):
     """
     Implicit first order scheme
     """
@@ -669,9 +708,10 @@ class BDF1(MultiStageScheme):
         a = np.array([1.])
         b = np.array([1.])
         c = np.array([1.])
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 1)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 1)
 
-class ExplicitMidPoint(MultiStageScheme):
+
+class ExplicitMidPoint(ButcherMultiStageScheme):
     """
     Explicit 2nd order scheme
     """
@@ -680,9 +720,10 @@ class ExplicitMidPoint(MultiStageScheme):
         a = np.array([[0, 0],[0.5, 0.0]])
         b = np.array([0., 1])
         c = np.array([0, 0.5])
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 2)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 2)
 
-class CN2(MultiStageScheme):
+
+class CN2(ButcherMultiStageScheme):
     """
     Semi-implicit 2nd order scheme
     """
@@ -691,9 +732,10 @@ class CN2(MultiStageScheme):
         b = np.array([0.5, 0.5])
         c = np.array([0, 1.0])
 
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 2)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 2)
 
-class ERK4(MultiStageScheme):
+
+class ERK4(ButcherMultiStageScheme):
     """
     Explicit 4th order scheme
     """
@@ -704,12 +746,16 @@ class ERK4(MultiStageScheme):
                       [0, 0, 1, 0]])
         b = np.array([1./6, 1./3, 1./3, 1./6])
         c = np.array([0, 0.5, 0.5, 1])
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 4)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 4)
 
 
-class ESDIRK3(MultiStageScheme):
-    """
-    Explicit implicit 3rd order scheme
+class ESDIRK3(ButcherMultiStageScheme):
+    """Explicit implicit 3rd order scheme
+
+    See also "Singly diagonally implicit Runge–Kutta methods with an
+    explicit first stage" by A Kværnø - BIT Numerical Mathematics,
+    2004 (p.497)
+
     """
     def __init__(self, rhs_form, solution, t=None, bcs=None):
         a = np.array([[0,                   0,                   0,                   0 ],
@@ -718,11 +764,17 @@ class ESDIRK3(MultiStageScheme):
                       [0.308809969973036,   1.490563388254108,  -1.235239879727145,   0.435866521500000 ]])
         b = a[-1,:].copy()
         c = a.sum(1)
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 3)
-                     
-class ESDIRK4(MultiStageScheme):
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 3)
+
+
+class ESDIRK4(ButcherMultiStageScheme):
     """
     Explicit implicit 4rd order scheme
+
+    See also "Singly diagonally implicit Runge–Kutta methods with an
+    explicit first stage" by A Kværnø - BIT Numerical Mathematics,
+    2004 (p.498)
+
     """
     def __init__(self, rhs_form, solution, t=None, bcs=None):
         a = np.array([[0,                  0,                 0,                  0,                   0],
@@ -733,7 +785,8 @@ class ESDIRK4(MultiStageScheme):
 
         b = a[-1,:].copy()
         c = a.sum(1)
-        MultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 4)
+        ButcherMultiStageScheme.__init__(self, rhs_form, solution, t, bcs, a, b, c, 4)
+
 
 # Aliases
 CrankNicolson = CN2
@@ -744,7 +797,7 @@ BackwardEuler = BDF1
 ERK = ERK1
 RK4 = ERK4
 
-__all__ = [name for name, attr in globals().items() \
+__all__ = [name for name, attr in list(globals().items()) \
            if isinstance(attr, type) and issubclass(attr, MultiStageScheme)]
 
 __all__.append("MultiStageScheme")
