@@ -26,7 +26,7 @@ source tree."""
 # First added:  2010-08-26
 # Last changed: 2011-07-10
 
-import os
+import os, io
 
 def extract_documentation(dolfin_dir, header, module):
     "Extract documentation for given header in given module"
@@ -49,7 +49,7 @@ def extract_documentation(dolfin_dir, header, module):
     indent = 0
 
     # Iterate over each line
-    f = open(os.path.join(dolfin_dir, "dolfin", module, header))
+    f = io.open(os.path.join(dolfin_dir, "dolfin", module, header), encoding="utf-8")
     for line in f:
 
         # Check for comment
