@@ -342,7 +342,12 @@ def gmsh2xml(ifilename, handler):
             _error("DOLFIN must be installed to handle Gmsh boundary regions")
         mesh = Mesh()
         mesh_editor = MeshEditor ()
-        mesh_editor.open( mesh, highest_dim, highest_dim )
+        if (highest_dim == 2):
+            mesh_editor.open(mesh, "triangle", highest_dim, highest_dim)
+        elif (highest_dim == 3):
+            mesh_editor.open(mesh, "tetrahedron", highest_dim, highest_dim)
+        else:
+            raise RuntimeError
         process_facets = True
     else:
         # TODO: Output a warning or an error here
@@ -1074,7 +1079,7 @@ class XmlHandler(DataHandler):
     def __init__(self, ofilename):
         DataHandler.__init__(self)
         self._ofilename = ofilename
-        self.__ofile = file(ofilename, "wb")
+        self.__ofile = open(ofilename, "w")
         self.__ofile_meshfunc = None
 
     def ofile(self):
@@ -1118,7 +1123,7 @@ class XmlHandler(DataHandler):
     def start_meshfunction(self, name, dim, size):
         DataHandler.start_meshfunction(self, name, dim, size)
         fname = os.path.splitext(self.__ofile.name)[0]
-        self.__ofile_meshfunc = file("%s_%s.xml" % (fname, name), "wb")
+        self.__ofile_meshfunc = open("%s_%s.xml" % (fname, name), "w")
         xml_writer.write_header_meshfunction(self.__ofile_meshfunc, dim, size)
 
     def add_entity_meshfunction(self, index, value):
@@ -1192,9 +1197,9 @@ def netcdf2xml(ifilename,ofilename):
 
     # Set cell type
     if dim == 2:
-        cell_type ="triangle"
+        cell_type = "triangle"
     if dim == 3:
-        cell_type ="tetrahedron"
+        cell_type = "tetrahedron"
 
     # Check that we got the cell type
     if cell_type == None:
