@@ -25,12 +25,53 @@ import shutil
 import tempfile
 import gc
 import platform
+import decorator
+
 from instant import get_status_output
 from dolfin import MPI, mpi_comm_world, parameters
-from dolfin import  *
+from dolfin import *
 
 # --- Test fixtures (use as is or as examples): ---
 
+def fixture(func):
+    """Decorator for creating module scope fixture.
+    Also take care of garbage collection of temporaries.
+
+    NOTE: Probably does not work with yield fixtures or
+    maybe just ignores post-yield code
+
+    This is the preferred decorator for writing fixtures
+    involving objects which might have collective destructors.
+    If in a need for using ``pytest.fixture`` directly, do::
+
+        yield result_of_fixture
+
+        gc.collect()
+        MPI.barrier(mpi_comm_world())
+    """
+
+    def wrapper(func, *args, **kwargs):
+
+        # Run function
+        rv = func(*args, **kwargs)
+
+        # Collect garbage of temporaries of the function
+        # and return collectivelly
+        gc.collect()
+        MPI.barrier(mpi_comm_world())
+
+        return rv
+
+        # FIXME: Need also to find a way how to clean-up fixture
+        #        return value; yield does not work here
+
+    # Decorate function with the wrapper
+    wrapped = decorator.decorator(wrapper, func)
+
+    # Mark as fixture and return
+    return pytest.fixture(scope='module')(wrapped)
+
+
 def gc_barrier():
     """Internal utility to easily switch on and off calls to
     gc.collect() and MPI.barrier(world) in all fixtures here.
@@ -40,11 +81,25 @@ def gc_barrier():
     if MPI.size(mpi_comm_world()) > 1:
         MPI.barrier(mpi_comm_world())
 
+
+@pytest.fixture()
+def worker_id(request):
+    """Returns thread id when running with pytest-xdist in parallel."""
+    if hasattr(request.config, 'slaveinput'):
+        return request.config.slaveinput['slaveid']
+    else:
+        return 'master'
+
+
 @pytest.yield_fixture(scope="function")
 def gc_barrier_fixture():
-    """Function decorator to call gc.collect() and MPI.barrier(world) before and after a test.
+    """Function decorator to call gc.collect() and
+    MPI.barrier(world) before and after a test.  Helps
+    make the tests deterministic when debugging.
 
-    Helps make the tests deterministic when debugging.
+    NOTE: This decorator is not needed now for writing tests,
+    as there is ``gc.collect()`` call in ``conftest.py`` on
+    teardown of every test.
     """
     gc_barrier()
     yield
@@ -67,6 +122,7 @@ def filedir(request):
     d = os.path.dirname(os.path.abspath(request.module.__file__))
     return d
 
+
 @pytest.fixture(scope="module")
 def rootdir(request):
     "Return the root directory of the repository. Assumes run from within repository filetree."
@@ -77,15 +133,20 @@ def rootdir(request):
         d, t = os.path.split(d)
     return d
 
+
 @pytest.fixture(scope="module")
 def datadir(request):
-    "Return the directory of the shared test data. Assumes run from within repository filetree."
+    """Return the directory of the shared test data. Assumes run from
+    within repository filetree.
+
+    """
     d = os.path.dirname(os.path.abspath(request.module.__file__))
     t = ''
     while t != "test":
         d, t = os.path.split(d)
     return os.path.join(d, "test", "data")
 
+
 def _create_tempdir(request):
     # Get directory name of test_foo.py file
     testfile = request.module.__file__
@@ -93,7 +154,7 @@ def _create_tempdir(request):
 
     # Construct name test_foo_tempdir from name test_foo.py
     testfilename = os.path.basename(testfile)
-    outputname = testfilename.replace(".py", "_tempdir")
+    outputname = testfilename.replace(".py", "_tempdir_{}".format(worker_id(request)))
 
     # Get function name test_something from test_foo.py
     function = request.function.__name__
@@ -116,25 +177,27 @@ def _create_tempdir(request):
         # First time visiting this basepath, delete the old and create a new
         if basepath not in _create_tempdir._basepaths:
             _create_tempdir._basepaths.add(basepath)
-            #if os.path.exists(basepath):
-            #    shutil.rmtree(basepath)
+            if os.path.exists(basepath):
+                shutil.rmtree(basepath)
             # Make sure we have the base path test_foo_tempdir for this test_foo.py file
             if not os.path.exists(basepath):
                 os.mkdir(basepath)
 
         # Delete path from old test run
-        #if os.path.exists(path):
-        #    shutil.rmtree(path)
+        if os.path.exists(path):
+            shutil.rmtree(path)
         # Make sure we have the path for this test execution: e.g. test_foo_tempdir/test_something__3
         if not os.path.exists(path):
             os.mkdir(path)
     MPI.barrier(mpi_comm_world())
 
     return path
+
 from collections import defaultdict
 _create_tempdir._sequencenumber = defaultdict(int)
 _create_tempdir._basepaths = set()
 
+
 @pytest.fixture(scope="function")
 def tempdir(request):
     """Return a unique directory name for this test function instance.
@@ -148,10 +211,13 @@ def tempdir(request):
     Does NOT change the current directory.
 
     MPI safe (assuming mpi_comm_world() context).
+
     """
+
     gc_barrier()
     return _create_tempdir(request)
 
+
 @pytest.yield_fixture(scope="function")
 def cd_tempdir(request):
     """Return a unique directory name for this test function instance.
@@ -173,6 +239,7 @@ def cd_tempdir(request):
     yield path
     os.chdir(cwd)
 
+
 @pytest.yield_fixture
 def pushpop_parameters():
     global parameters
