@@ -21,8 +21,6 @@
 import pytest
 from dolfin import *
 
-# Standard fixture
-fixture = pytest.fixture(scope='module')
 
 # Skips with dependencies
 skip_if_not_PETsc_or_not_slepc = \
@@ -32,14 +30,20 @@ skip_if_not_HDF5 = pytest.mark.skipif(not has_hdf5(),
                                     reason="Skipping unit test(s) depending on HDF5.")
 skip_if_not_PETSc = pytest.mark.skipif(not has_linear_algebra_backend("PETSc"),
                                        reason="Skipping unit test(s) depending on PETSc.")
+skip_if_not_SLEPc = pytest.mark.skipif(not has_slepc(),
+                                       reason="Skipping unit test(s) depending on SLEPc.")
+skip_if_not_MPI = pytest.mark.skipif(not has_mpi(),
+                                     reason="Skipping unit test(s) depending on MPI.")
 skip_if_not_petsc4py = pytest.mark.skipif(not has_petsc4py(),
                                           reason="Skipping unit test(s) depending on petsc4py.")
 
 # Skips with respect to parallel or serial
 xfail_in_parallel = pytest.mark.xfail(MPI.size(mpi_comm_world()) > 1,
                                       reason="This test does not yet work in parallel.")
+xfail_with_serial_hdf5_in_parallel = pytest.mark.xfail(MPI.size(mpi_comm_world()) > 1 and not has_hdf5_parallel(), reason="Serial HDF5 library cannot work in parallel.")
 skip_in_parallel = pytest.mark.skipif(MPI.size(mpi_comm_world()) > 1,
                                       reason="This test should only be run in serial.")
+skip_with_serial_hdf5_in_parallel = pytest.mark.skipif(MPI.size(mpi_comm_world()) > 1 and not has_hdf5_parallel(), reason="Serial HDF5 library cannot work in parallel.")
 skip_in_serial = pytest.mark.skipif(MPI.size(mpi_comm_world()) <= 1,
                                     reason="This test should only be run in parallel.")
 
@@ -47,19 +51,15 @@ skip_in_serial = pytest.mark.skipif(MPI.size(mpi_comm_world()) <= 1,
 skip_64bit_int = pytest.mark.skipif(cpp.common.sizeof_la_index() == 8,
                                     reason="This test does not work with 64-bit linear algebra indices.")
 
-# This should be moved to dolfin.cpp.common (Jan Blechta to fix)
-has_debug = compile_extension_module(
-"""bool has_debug() {
-#ifdef DEBUG
-return true;
-#else
-return false;
-#endif
-}
-""").has_debug
-
 # Skips with respect to build type
 skip_in_debug = pytest.mark.skipif(has_debug(),
                                    reason="This test does not work in debug mode.")
 skip_in_release = pytest.mark.skipif(not has_debug(),
                                      reason="This test does not work in release mode.")
+
+# Skips for pybind11
+import dolfin
+skip_if_pybind11 = pytest.mark.skipif(has_pybind11(),
+                                      reason="Skipping this test with pybind11.")
+skip_if_not_pybind11 = pytest.mark.skipif(not has_pybind11(),
+                                          reason="Skipping this test without pybind11.")
