@@ -117,6 +117,8 @@ def main():
     not_implemented = \
       [os.path.join(demodir, 'undocumented', 'projection-interpolation',    'cpp'),
        os.path.join(demodir, 'undocumented', 'interpolation',               'cpp'),
+       os.path.join(demodir, 'undocumented', 'gmg-poisson',                 'python'),
+       os.path.join(demodir, 'undocumented', 'gmg-stokes',                  'python'),
        os.path.join(demodir, 'undocumented', 'adaptive-poisson',            'cpp'),
        os.path.join(demodir, 'undocumented', 'multistage-solver',           'cpp'),
        os.path.join(demodir, 'undocumented', 'smoothing',                   'cpp'),
@@ -125,7 +127,10 @@ def main():
        os.path.join(demodir, 'undocumented', 'compiled-extension-module',   'cpp'),
        os.path.join(demodir, 'undocumented', 'timing',                      'cpp'),
        os.path.join(demodir, 'undocumented', 'mplot',                       'cpp'),
+       os.path.join(demodir, 'undocumented', 'plot',                        'cpp'),
        os.path.join(demodir, 'undocumented', 'coordinates',                 'cpp'),
+       os.path.join(demodir, 'undocumented', 'multimesh-quadrature',        'cpp'),
+       os.path.join(demodir, 'undocumented', 'multimesh-3d',                'cpp'),       
        os.path.join(demodir, 'documented',   'stokes-mini',                 'cpp'),
        os.path.join(demodir, 'documented',   'tensor-weighted-poisson',     'cpp'),
        os.path.join(demodir, 'documented',   'subdomains-poisson',          'cpp'),
@@ -133,11 +138,25 @@ def main():
        os.path.join(demodir, 'documented',   'maxwell-eigenvalues',        'cpp'),
        ]
 
+    # Check if we should run only Python tests, use for quick testing
+    if (len(sys.argv) == 2 and sys.argv[1] == "--only-python") or \
+       "DISABLE_CPP_TESTING" in os.environ:
+        only_python = True
+    else:
+        only_python = False
+
+    # Check if we should run only C++ tests
+    if (len(sys.argv) == 2 and sys.argv[1] == "--only-cpp") or \
+       "DISABLE_PYTHON_TESTING" in os.environ:
+        only_cpp = True
+    else:
+        only_cpp = False
+
     # Demos to run
     cppdemos = []
     pydemos = []
     for dpath, dnames, fnames in os.walk(demodir):
-        if os.path.basename(dpath) == 'cpp':
+        if os.path.basename(dpath) == 'cpp' and not only_python:
             if os.path.isfile(os.path.join(dpath, 'Makefile')):
                 cppdemos.append(dpath)
                 assert not dpath in not_implemented, \
@@ -145,7 +164,7 @@ def main():
             else:
                 assert dpath in not_implemented, \
                     "Non-existing demo '%s' not marked as not_implemented" % dpath
-        elif os.path.basename(dpath) == 'python':
+        elif os.path.basename(dpath) == 'python' and not only_cpp:
             tmp = dpath.split(os.path.sep)[-2]
             if os.path.isfile(os.path.join(dpath, 'demo_' + tmp + '.py')):
                 pydemos.append(dpath)
@@ -156,7 +175,7 @@ def main():
                     "Non-existing demo '%s' not marked as not_implemented" % dpath
 
     # Set non-interactive
-    os.putenv('DOLFIN_NOPLOT', '1')
+    os.putenv('MPLBACKEND', 'agg')
 
     print("Running all demos (non-interactively)")
     print("")
@@ -213,20 +232,6 @@ def main():
     failed = []
     timing = []
 
-    # Check if we should run only Python tests, use for quick testing
-    if (len(sys.argv) == 2 and sys.argv[1] == "--only-python") or \
-       "DISABLE_CPP_TESTING" in os.environ:
-        only_python = True
-    else:
-        only_python = False
-
-    # Check if we should run only C++ tests
-    if (len(sys.argv) == 2 and sys.argv[1] == "--only-cpp") or \
-       "DISABLE_PYTHON_TESTING" in os.environ:
-        only_cpp = True
-    else:
-        only_cpp = False
-
     # Check if we should skip C++ demos
     if only_python:
         print("Skipping C++ demos")
