@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for assembly"""
 
 # Copyright (C) 2011-2014 Garth N. Wells
@@ -33,7 +31,8 @@ from dolfin_utils.test import skip_in_parallel, filedir, pushpop_parameters
 
 def test_cell_size_assembly_1D():
     mesh = UnitIntervalMesh(10)
-    assert round(assemble(CellSize(mesh)*dx) - 0.1, 12) == 0
+    assert round(assemble(2*Circumradius(mesh)*dx) - 0.1, 12) == 0
+    assert round(assemble(CellDiameter(mesh)*dx) - 0.1, 12) == 0
     assert round(assemble(CellVolume(mesh)*dx) - 0.1, 12) == 0
 
 
@@ -56,28 +55,6 @@ def test_cell_assembly_1D():
     assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
 
 
-@skip_in_parallel
-def test_cell_assembly_1D_multithreaded(pushpop_parameters):
-
-    mesh = UnitIntervalMesh(48)
-    V = FunctionSpace(mesh, "CG", 1)
-
-    v = TestFunction(V)
-    u = TrialFunction(V)
-    f = Constant(10.0)
-
-    a = inner(grad(v), grad(u))*dx
-    L = inner(v, f)*dx
-
-    A_frobenius_norm = 811.75365721381274397572
-    b_l2_norm = 1.43583841167606474087
-
-    # Assemble A and b
-    parameters["num_threads"] = 4
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-
-
 def test_cell_assembly():
     mesh = UnitCubeMesh(4, 4, 4)
     V = VectorFunctionSpace(mesh, "DG", 1)
@@ -100,139 +77,91 @@ def test_cell_assembly():
     assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
 
 
-@skip_in_parallel
-def test_cell_assembly_multithreaded():
-    mesh = UnitCubeMesh(4, 4, 4)
-    V = VectorFunctionSpace(mesh, "DG", 1)
-
-    v = TestFunction(V)
-    u = TrialFunction(V)
-    f = Constant((10, 20, 30))
-
-    def epsilon(v):
-        return 0.5*(grad(v) + grad(v).T)
-
-    a = inner(epsilon(v), epsilon(u))*dx
-    L = inner(v, f)*dx
-
-    A_frobenius_norm = 4.3969686527582512
-    b_l2_norm = 0.95470326978246278
-
-    # Assemble A and b
-    parameters["num_threads"] = 4
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-    parameters["num_threads"] = 0
-
-
-def test_facet_assembly():
+def test_facet_assembly(pushpop_parameters):
     parameters["ghost_mode"] = "shared_facet"
     mesh = UnitSquareMesh(24, 24)
+    V = FunctionSpace(mesh, "DG", 1)
+
+    # Define test and trial functions
+    v = TestFunction(V)
+    u = TrialFunction(V)
+
+    # Define normal component, mesh size and right-hand side
+    n = FacetNormal(mesh)
+    h = 2*Circumradius(mesh)
+    h_avg = (h('+') + h('-'))/2
+    f = Expression("500.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=1)
+
+    # Define bilinear form
+    a = dot(grad(v), grad(u))*dx \
+        - dot(avg(grad(v)), jump(u, n))*dS \
+        - dot(jump(v, n), avg(grad(u)))*dS \
+        + 4.0/h_avg*dot(jump(v, n), jump(u, n))*dS \
+        - dot(grad(v), u*n)*ds \
+        - dot(v*n, grad(u))*ds \
+        + 8.0/h*v*u*ds
+
+    # Define linear form
+    L = v*f*dx
+
+    # Reference values
+    A_frobenius_norm = 157.867392938645
+    b_l2_norm = 1.48087142738768
+
+    # Assemble A and b
+    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
+    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
+
+
+def test_ghost_mode_handling(pushpop_parameters):
+    def _form():
+        # Return form with trivial interior facet integral
+        mesh = UnitSquareMesh(10, 10)
+        ff = MeshFunction('size_t', mesh, mesh.topology().dim()-1, 0)
+        AutoSubDomain(lambda x: near(x[0], 0.5)).mark(ff, 1)
+        return Constant(1.0)*dS(domain=mesh, subdomain_data=ff, subdomain_id=1)
+
+    # Not-ghosted mesh won't work in parallel and assembler should raise
     parameters["ghost_mode"] = "none"
-    V = FunctionSpace(mesh, "DG", 1)
+    if MPI.size(mpi_comm_world()) == 1:
+        assert numpy.isclose(assemble(_form()), 1.0)
+    else:
+        form = _form()
+        with pytest.raises(RuntimeError) as excinfo:
+            assemble(form)
+        assert "Incorrect mesh ghost mode" in repr(excinfo.value)
 
-    # Define test and trial functions
-    v = TestFunction(V)
-    u = TrialFunction(V)
+    # Ghosted meshes work everytime
+    parameters["ghost_mode"] = "shared_vertex"
+    assert numpy.isclose(assemble(_form()), 1.0)
+    parameters["ghost_mode"] = "shared_facet"
+    assert numpy.isclose(assemble(_form()), 1.0)
 
-    # Define normal component, mesh size and right-hand side
-    n = FacetNormal(mesh)
-    h = CellSize(mesh)
-    h_avg = (h('+') + h('-'))/2
-    f = Expression("500.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=1)
-
-    # Define bilinear form
-    a = dot(grad(v), grad(u))*dx \
-        - dot(avg(grad(v)), jump(u, n))*dS \
-        - dot(jump(v, n), avg(grad(u)))*dS \
-        + 4.0/h_avg*dot(jump(v, n), jump(u, n))*dS \
-        - dot(grad(v), u*n)*ds \
-        - dot(v*n, grad(u))*ds \
-        + 8.0/h*v*u*ds
-
-    # Define linear form
-    L = v*f*dx
-
-    # Reference values
-    A_frobenius_norm = 157.867392938645
-    b_l2_norm = 1.48087142738768
-
-    # Assemble A and b
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-
-
-@skip_in_parallel
-def test_facet_assembly_multithreaded(pushpop_parameters):
-    mesh = UnitSquareMesh(24, 24)
-    V = FunctionSpace(mesh, "DG", 1)
-
-    # Define test and trial functions
-    v = TestFunction(V)
-    u = TrialFunction(V)
-
-    # Define normal component, mesh size and right-hand side
-    n = FacetNormal(mesh)
-    h = CellSize(mesh)
-    h_avg = (h('+') + h('-'))/2
-    f = Expression("500.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=1)
-
-    # Define bilinear form
-    a = dot(grad(v), grad(u))*dx \
-        - dot(avg(grad(v)), jump(u, n))*dS \
-        - dot(jump(v, n), avg(grad(u)))*dS \
-        + 4.0/h_avg*dot(jump(v, n), jump(u, n))*dS \
-        - dot(grad(v), u*n)*ds \
-        - dot(v*n, grad(u))*ds \
-        + 8.0/h*v*u*ds
-
-    # Define linear form
-    L = v*f*dx
-
-    # Reference values
-    A_frobenius_norm = 157.867392938645
-    b_l2_norm = 1.48087142738768
-
-    # Assemble A and b
-    parameters["num_threads"] = 4
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-
-
-def test_functional_assembly():
-    mesh = UnitSquareMesh(24, 24)
+@pytest.mark.parametrize('mesh_factory, facet_area', [((UnitSquareMesh, (4, 4)), 4.0),
+                                                      ((UnitCubeMesh, (2, 2, 2)), 6.0),
+                                                      ((UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral)), 4.0),
+                                                      ((UnitCubeMesh.create, (2, 2, 2, CellType.Type_hexahedron)), 6.0)])
+def test_functional_assembly(mesh_factory, facet_area):
+    func, args = mesh_factory
+    mesh = func(*args)
 
     f = Constant(1.0)
     M0 = f*dx(mesh)
     assert round(assemble(M0) - 1.0, 7) == 0
 
     M1 = f*ds(mesh)
-    assert round(assemble(M1) - 4.0, 7) == 0
+    assert round(assemble(M1) - facet_area, 7) == 0
 
 
-@skip_in_parallel
-def test_functional_assembly_multithreaded(pushpop_parameters):
-
-    mesh = UnitSquareMesh(24, 24)
-
-    f = Constant(1.0)
-    M0 = f*dx(mesh)
-    assert round(assemble(M0) - 1.0, 7) == 0
-
-    M1 = f*ds(mesh)
-    assert round(assemble(M1) - 4.0, 7) == 0
-
-    parameters["num_threads"] = 4
-    assert round(assemble(M0) - 1.0, 7) == 0
-
-
-def test_subdomain_and_fulldomain_assembly_meshdomains():
+@pytest.mark.parametrize('mesh_factory', [(UnitCubeMesh, (4, 4, 4)), (UnitCubeMesh.create, (4, 4, 4, CellType.Type_hexahedron))])
+def test_subdomain_and_fulldomain_assembly_meshdomains(mesh_factory):
     """Test assembly over subdomains AND the full domain with markers
     stored as part of the mesh.
     """
 
     # Create a mesh of the unit cube
-    mesh = UnitCubeMesh(4, 4, 4)
+    func, args = mesh_factory
+    mesh = func(*args)
 
     # Define subdomains for 3 faces of the unit cube
     class F0(SubDomain):
@@ -298,7 +227,7 @@ def test_subdomain_assembly_form_1():
     class Left(SubDomain):
         def inside(self, x, on_boundary):
             return x[0] < 0.49
-    subdomains = CellFunction("size_t", mesh)
+    subdomains = MeshFunction("size_t", mesh, mesh.topology().dim())
     subdomains.set_all(0)
     left = Left()
     left.mark(subdomains, 1)
@@ -306,7 +235,7 @@ def test_subdomain_assembly_form_1():
     class RightBoundary(SubDomain):
         def inside(self, x, on_boundary):
             return x[0] > 0.95
-    boundaries = FacetFunction("size_t", mesh)
+    boundaries = MeshFunction("size_t", mesh, mesh.topology().dim()-1)
     boundaries.set_all(0)
     right = RightBoundary()
     right.mark(boundaries, 1)
@@ -355,83 +284,6 @@ def test_subdomain_assembly_form_1():
     assert round(assemble(b).norm("l2") - reference, 8) == 0
 
 
-@skip_in_parallel
-def test_subdomain_assembly_form_1_multithreaded(pushpop_parameters):
-    "Test assembly over subdomains with markers stored as part of form"
-
-    mesh = UnitSquareMesh(4, 4)
-
-    # Define cell/facet function
-    class Left(SubDomain):
-        def inside(self, x, on_boundary):
-            return x[0] < 0.49
-    subdomains = CellFunction("size_t", mesh)
-    subdomains.set_all(0)
-    left = Left()
-    left.mark(subdomains, 1)
-
-    class RightBoundary(SubDomain):
-        def inside(self, x, on_boundary):
-            return x[0] > 0.95
-    boundaries = FacetFunction("size_t", mesh)
-    boundaries.set_all(0)
-    right = RightBoundary()
-    right.mark(boundaries, 1)
-
-    V = FunctionSpace(mesh, "CG", 2)
-    f = Expression("x[0] + 2", degree=1)
-    g = Expression("x[1] + 1", degree=1)
-
-    f = interpolate(f, V)
-    g = interpolate(g, V)
-
-    mesh1 = subdomains.mesh()
-    mesh2 = boundaries.mesh()
-    assert mesh1.id() == mesh2.id()
-    assert mesh1.ufl_domain().ufl_id() == mesh2.ufl_domain().ufl_id()
-
-    dxs = dx(subdomain_data=subdomains)
-    dss = ds(subdomain_data=boundaries)
-    assert dxs.ufl_domain() == None
-    assert dss.ufl_domain() == None
-    assert dxs.subdomain_data() == subdomains
-    assert dss.subdomain_data() == boundaries
-
-    M = f*f*dxs(0) + g*f*dxs(1) + f*f*dss(1)
-    assert M.ufl_domains() == (mesh.ufl_domain(),)
-    sd = M.subdomain_data()[mesh.ufl_domain()]
-    assert sd["cell"] == subdomains
-    assert sd["exterior_facet"] == boundaries
-
-    # Check that subdomains are respected
-    reference = 15.0
-    assert round(assemble(M) - reference, 10) == 0
-
-    # Assemble form (multi-threaded)
-    parameters["num_threads"] = 4
-    assert round(assemble(M) - reference, 10) == 0
-
-    # Check that the form itself assembles as before
-    assert round(assemble(M) - reference, 10) == 0
-
-    # Assemble form  (multi-threaded)
-    assert round(assemble(M) - reference, 10) == 0
-
-    # Take action of derivative of M on f
-    df = TestFunction(V)
-    L = derivative(M, f, df)
-    dg = TrialFunction(V)
-    F = derivative(L, g, dg)
-    b = action(F, f)
-
-    # Check that domain data carries across transformations:
-    reference = 0.136477465659
-    assert round(assemble(b).norm("l2") - reference, 8) == 0
-
-    # Assemble form  (multi-threaded)
-    assert round(assemble(b).norm("l2") - reference, 8) == 0
-
-
 def test_subdomain_assembly_form_2():
     "Test assembly over subdomains with markers stored as part of form"
 
@@ -470,40 +322,6 @@ def test_subdomain_assembly_form_2():
     assert round(assemble(a1) - 1.0, 7) == 0
 
 
-@skip_in_parallel
-def test_colored_cell_assembly():
-
-    old_mesh = UnitCubeMesh(4, 4, 4)
-
-    # Create mesh, then color and renumber
-    old_mesh.color("vertex")
-    mesh = old_mesh.renumber_by_color()
-
-    V = VectorFunctionSpace(mesh, "DG", 1)
-    v = TestFunction(V)
-    u = TrialFunction(V)
-    f = Constant((10, 20, 30))
-
-    def epsilon(v):
-        return 0.5*(grad(v) + grad(v).T)
-
-    a = inner(epsilon(v), epsilon(u))*dx
-    L = inner(v, f)*dx
-
-    A_frobenius_norm = 4.3969686527582512
-    b_l2_norm = 0.95470326978246278
-
-    # Assemble A and b
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-
-    # Assemble A and b multi-threaded
-    parameters["num_threads"] = 4
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    assert round(assemble(L).norm("l2") - b_l2_norm, 10) == 0
-    parameters["num_threads"] = 0
-
-
 def test_nonsquare_assembly():
     """Test assembly of a rectangular matrix"""
 
@@ -529,34 +347,12 @@ def test_nonsquare_assembly():
     assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
 
 
-@skip_in_parallel
-def test_nonsquare_assembly_multithreaded():
-    """Test assembly of a rectangular matrix"""
-
-    mesh = UnitSquareMesh(16, 16)
-
-    V = VectorElement("Lagrange", mesh.ufl_cell(), 2)
-    Q = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
-    W = FunctionSpace(mesh, V*Q)
-
-    (v, q) = TestFunctions(W)
-    (u, p) = TrialFunctions(W)
-
-    a = div(v)*p*dx
-    A_frobenius_norm = 9.6420303878382718e-01
-
-    parameters["num_threads"] = 4
-    assert round(assemble(a).norm("frobenius") - A_frobenius_norm, 10) == 0
-    parameters["num_threads"] = 0
-
-
 @skip_in_parallel
 def test_reference_assembly(filedir, pushpop_parameters):
     "Test assembly against a reference solution"
 
     # NOTE: This test is not robust as it relies on specific
     #       DOF order, which cannot be guaranteed
-    reorder_dofs = parameters["reorder_dofs_serial"]
     parameters["reorder_dofs_serial"] = False
 
     # Load reference mesh (just a simple tetrahedron)
