@@ -1,8 +1,6 @@
-#!/usr/bin/env py.test
-
 """Unit tests for Dirichlet boundary conditions"""
 
-# Copyright (C) 2011-2012 Garth N. Wells
+# Copyright (C) 2011-2017 Garth N. Wells
 #
 # This file is part of DOLFIN.
 #
@@ -24,10 +22,13 @@
 # Modified by Martin Alnaes 2012
 
 import os
+
 import pytest
 import numpy
+import six
+
 from dolfin import *
-from dolfin_utils.test import skip_in_parallel, datadir
+from dolfin_utils.test import skip_in_parallel, datadir, skip_if_pybind11
 
 
 def test_instantiation():
@@ -48,7 +49,7 @@ def test_director_lifetime():
     class Boundary(SubDomain):
         def inside(self, x, on_boundary): return on_boundary
 
-    class BoundaryFunction(Expression):
+    class BoundaryFunction(UserExpression):
         def eval(self, values, x): values[0] = 1.0
 
     mesh = UnitSquareMesh(8, 8)
@@ -77,11 +78,12 @@ def test_get_values():
     bc = DirichletBC(V, 0.0, upper)
     bc_values = bc.get_boundary_values()
 
-
+@skip_if_pybind11
 def test_meshdomain_bcs(datadir):
-    """Test application of Dirichlet boundary conditions stored as
-    part of the mesh. This test is also a compatibility test for
-    VMTK."""
+    """Test application of Dirichlet boundary conditions stored as part of
+    the mesh. This test is also a compatibility test for VMTK.
+
+    """
 
     mesh = Mesh(os.path.join(datadir, "aneurysm.xml"))
     V = FunctionSpace(mesh, "CG", 1)
@@ -108,49 +110,40 @@ def test_user_meshfunction_domains():
     mesh1 = UnitSquareMesh(12, 12)
     V = FunctionSpace(mesh0, "CG", 1)
 
-    DirichletBC(V, Constant(0.0), EdgeFunction("size_t", mesh0), 0)
-    DirichletBC(V, Constant(0.0), FacetFunction("size_t", mesh0), 0)
+    DirichletBC(V, Constant(0.0), MeshFunction("size_t", mesh0, 1), 0)
+    DirichletBC(V, Constant(0.0), MeshFunction("size_t", mesh0, mesh0.topology().dim()-1), 0)
     with pytest.raises(RuntimeError):
-        DirichletBC(V, 0.0, CellFunction("size_t", mesh0), 0)
-        DirichletBC(V, 0.0, VertexFunction("size_t", mesh0), 0)
-        DirichletBC(V, 0.0, FacetFunction("size_t", mesh1), 0)
+        DirichletBC(V, 0.0, MeshFunction("size_t", mesh0, mesh0.topology().dim()), 0)
+        DirichletBC(V, 0.0, MeshFunction("size_t", mesh0, 0), 0)
+        DirichletBC(V, 0.0, MeshFunction("size_t", mesh1, mesh1.topology().dim()-1), 0)
 
 
 @skip_in_parallel
-def test_bc_for_piola_on_manifolds():
-    "Testing DirichletBC for piolas over standard domains vs manifolds."
+@pytest.mark.parametrize("degree", [1, 2])
+@pytest.mark.parametrize("element_type",
+                         ["RT", "DRT", "BDM", "N1curl", "N2curl"])
+def test_bc_for_piola_on_manifolds(element_type, degree):
+    """Testing DirichletBC for piolas over standard domains vs manifolds.
+
+    """
     n = 4
     side = CompiledSubDomain("near(x[2], 0.0)")
     mesh = SubMesh(BoundaryMesh(UnitCubeMesh(n, n, n), "exterior"), side)
-    square = UnitSquareMesh(n, n)
     mesh.init_cell_orientations(Expression(("0.0", "0.0", "1.0"), degree=0))
+    square = UnitSquareMesh(n, n)
 
-    RT1 = lambda mesh: FunctionSpace(mesh, "RT", 1)
-    RT2 = lambda mesh: FunctionSpace(mesh, "RT", 2)
-    DRT1 = lambda mesh: FunctionSpace(mesh, "DRT", 1)
-    DRT2 = lambda mesh: FunctionSpace(mesh, "DRT", 2)
-    BDM1 = lambda mesh: FunctionSpace(mesh, "BDM", 1)
-    BDM2 = lambda mesh: FunctionSpace(mesh, "BDM", 2)
-    N1curl1 = lambda mesh: FunctionSpace(mesh, "N1curl", 1)
-    N2curl1 = lambda mesh: FunctionSpace(mesh, "N2curl", 1)
-    N1curl2 = lambda mesh: FunctionSpace(mesh, "N1curl", 2)
-    N2curl2 = lambda mesh: FunctionSpace(mesh, "N2curl", 2)
-    elements = [N1curl1, N2curl1,  N1curl2, N2curl2, RT1, RT2, BDM1,
-                BDM2, DRT1, DRT2]
+    V = FunctionSpace(mesh, element_type, degree)
+    bc = DirichletBC(V, (1.0, 0.0, 0.0), "on_boundary")
+    u = Function(V)
+    bc.apply(u.vector())
+    b0 = assemble(inner(u, u)*dx)
 
-    for element in elements:
-        V = element(mesh)
-        bc = DirichletBC(V, (1.0, 0.0, 0.0), "on_boundary")
-        u = Function(V)
-        bc.apply(u.vector())
-        b0 = assemble(inner(u, u)*dx)
-
-        V = element(square)
-        bc = DirichletBC(V, (1.0, 0.0), "on_boundary")
-        u = Function(V)
-        bc.apply(u.vector())
-        b1 = assemble(inner(u, u)*dx)
-        assert round(b0 - b1, 7) == 0
+    V = FunctionSpace(square, element_type, degree)
+    bc = DirichletBC(V, (1.0, 0.0), "on_boundary")
+    u = Function(V)
+    bc.apply(u.vector())
+    b1 = assemble(inner(u, u)*dx)
+    assert round(b0 - b1, 7) == 0
 
 
 def test_zero():
@@ -178,6 +171,80 @@ def test_zero():
     assert near(boundaryIntegral, 0.0)
 
 
+@skip_in_parallel
+def test_zero_columns_offdiag():
+    """Test zero_columns applied to offdiagonal block"""
+    mesh = UnitSquareMesh(20, 20)
+    V = VectorFunctionSpace(mesh, "P", 2)
+    Q = FunctionSpace(mesh, "P", 1)
+    u = TrialFunction(V)
+    q = TestFunction(Q)
+    a = inner(div(u), q)*dx
+    L = inner(Constant(0), q)*dx
+    A = assemble(a)
+    b = assemble(L)
+
+    bc = DirichletBC(V, Constant((-32.23333, 43243.1)), 'on_boundary')
+
+    # Compute residual with x satisfying bc before zero_columns
+    u = Function(V)
+    x = u.vector()
+    bc.apply(x)
+    r0 = A*x - b
+
+    bc.zero_columns(A, b)
+
+    # Test that A gets zero columns
+    bc_dict = bc.get_boundary_values()
+    for i in six.moves.xrange(*A.local_range(0)):
+        cols, vals = A.getrow(i)
+        for j, v in six.moves.zip(cols, vals):
+            if j in bc_dict:
+                assert v == 0.0
+
+    # Compute residual with x satisfying bc after zero_columns
+    # and check that it is preserved
+    r1 = A*x - b
+    assert numpy.isclose((r1-r0).norm('linf'), 0.0)
+
+
+@skip_in_parallel
+def test_zero_columns_square():
+    """Test zero_columns applied to square matrix"""
+    mesh = UnitSquareMesh(20, 20)
+    V = FunctionSpace(mesh, "P", 1)
+    u, v = TrialFunction(V), TestFunction(V)
+    a = inner(grad(u), grad(v))*dx
+    L = Constant(0)*v*dx
+    A = assemble(a)
+    b = assemble(L)
+    u = Function(V)
+    x = u.vector()
+
+    bc = DirichletBC(V, 666.0, 'on_boundary')
+    bc.zero_columns(A, b, 42.0)
+
+    # Check that A gets zeros in bc rows and bc columns and 42 on
+    # diagonal
+    bc_dict = bc.get_boundary_values()
+    for i in six.moves.xrange(*A.local_range(0)):
+        cols, vals = A.getrow(i)
+        for j, v in six.moves.zip(cols, vals):
+            if i in bc_dict or j in bc_dict:
+                if i == j:
+                    assert numpy.isclose(v, 42.0)
+                else:
+                    assert v == 0.0
+
+    # Check that solution of linear system works
+    solve(A, x, b)
+    assert numpy.isclose((b-A*x).norm('linf'), 0.0)
+    x1 = x.copy()
+    bc.apply(x1)
+    x1 -= x
+    assert numpy.isclose(x1.norm('linf'), 0.0)
+
+
 def test_homogenize_consistency():
     mesh = UnitIntervalMesh(10)
     V = FunctionSpace(mesh, "CG", 1)
@@ -187,3 +254,25 @@ def test_homogenize_consistency():
         bc_new = DirichletBC(bc)
         bc_new.homogenize()
         assert bc_new.method() == bc.method()
+
+
+def test_nocaching_values():
+    """There might be caching of dof indices in DirichletBC.
+    But caching of values is _not_ allowed."""
+    mesh = UnitSquareMesh(4, 4)
+    V = FunctionSpace(mesh, "P", 1)
+    u = Function(V)
+    x = u.vector()
+
+    for method in ["topological", "geometric", "pointwise"]:
+        bc = DirichletBC(V, 0.0, lambda x, b: True, method=method)
+
+        x.zero()
+        bc.set_value(Constant(1.0))
+        bc.apply(x)
+        assert numpy.allclose(x.get_local(), 1.0)
+
+        x.zero()
+        bc.set_value(Constant(2.0))
+        bc.apply(x)
+        assert numpy.allclose(x.get_local(), 2.0)
