@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the fem interface"""
 
 # Copyright (C) 2009-2014 Garth N. Wells
@@ -22,37 +20,38 @@
 from __future__ import print_function
 import pytest
 import numpy as np
-from dolfin import *
 
+import sys
+
+from dolfin import *
 from dolfin_utils.test import *
 
 
+xfail = pytest.mark.xfail(strict=True)
+
 @fixture
 def mesh():
     return UnitSquareMesh(4, 4)
 
 
-@fixture
-def V(mesh):
-    return FunctionSpace(mesh, "Lagrange", 1)
-
-
-@fixture
-def Q(mesh):
-    return VectorFunctionSpace(mesh, "Lagrange", 1)
-
-
-@fixture
-def W(mesh):
-    V = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
-    Q = VectorElement("Lagrange", mesh.ufl_cell(), 1)
-    return FunctionSpace(mesh, V*Q)
-
-
 reorder_dofs = set_parameters_fixture("reorder_dofs_serial", [True, False])
 
 
-def test_tabulate_all_coordinates(mesh, V, W):
+@pytest.mark.parametrize('mesh_factory', [(UnitIntervalMesh, (8,)),
+                                          (UnitSquareMesh, (4, 4)),
+                                          (UnitCubeMesh, (2, 2, 2)),
+                                          # cell.contains(Point) does not work correctly
+                                          # for quad/hex cells once it is fixed, this test will pass
+                                          xfail((UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral))),
+                                          xfail((UnitCubeMesh.create, (2, 2, 2, CellType.Type_hexahedron)))])
+def test_tabulate_all_coordinates(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
+    V = FunctionSpace(mesh, "Lagrange", 1)
+    W0 = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
+    W1 = VectorElement("Lagrange", mesh.ufl_cell(), 1)
+    W = FunctionSpace(mesh, W0*W1)
+
     D = mesh.geometry().dim()
     V_dofmap = V.dofmap()
     W_dofmap = W.dofmap()
@@ -62,9 +61,6 @@ def test_tabulate_all_coordinates(mesh, V, W):
     local_size_V = V_dofmap.ownership_range()[1]-V_dofmap.ownership_range()[0]
     local_size_W = W_dofmap.ownership_range()[1]-W_dofmap.ownership_range()[0]
 
-    assert all_coords_V.shape == (D*local_size_V,)
-    assert all_coords_W.shape == (D*local_size_W,)
-
     all_coords_V = all_coords_V.reshape(local_size_V, D)
     all_coords_W = all_coords_W.reshape(local_size_W, D)
 
@@ -92,7 +88,13 @@ def test_tabulate_all_coordinates(mesh, V, W):
     assert all(checked_W)
 
 
-def test_tabulate_dofs(mesh, W):
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (4, 4)), (UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral))])
+def test_tabulate_dofs(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
+    W0 = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
+    W1 = VectorElement("Lagrange", mesh.ufl_cell(), 1)
+    W = FunctionSpace(mesh, W0*W1)
 
     L0 = W.sub(0)
     L1 = W.sub(1)
@@ -116,7 +118,8 @@ def test_tabulate_dofs(mesh, W):
         assert np.array_equal(np.append(dofs1, dofs2), dofs3)
 
 
-def test_tabulate_coord_periodic():
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (4, 4)), (UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral))])
+def test_tabulate_coord_periodic(mesh_factory):
 
     class PeriodicBoundary2(SubDomain):
         def inside(self, x, on_boundary):
@@ -129,7 +132,8 @@ def test_tabulate_coord_periodic():
     # Create periodic boundary condition
     periodic_boundary = PeriodicBoundary2()
 
-    mesh = UnitSquareMesh(4, 4)
+    func, args = mesh_factory
+    mesh = func(*args)
 
     V = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
     Q = VectorElement("Lagrange", mesh.ufl_cell(), 1)
@@ -143,26 +147,28 @@ def test_tabulate_coord_periodic():
     L01 = L1.sub(0)
     L11 = L1.sub(1)
 
-    coord0 = np.zeros((3, 2), dtype="d")
-    coord1 = np.zeros((3, 2), dtype="d")
-    coord2 = np.zeros((3, 2), dtype="d")
-    coord3 = np.zeros((3, 2), dtype="d")
+    sdim = V.element().space_dimension()
+    coord0 = np.zeros((sdim, 2), dtype="d")
+    coord1 = np.zeros((sdim, 2), dtype="d")
+    coord2 = np.zeros((sdim, 2), dtype="d")
+    coord3 = np.zeros((sdim, 2), dtype="d")
 
     for cell in cells(mesh):
-        V.element().tabulate_dof_coordinates(cell, coord0)
-        L0.element().tabulate_dof_coordinates(cell, coord1)
-        L01.element().tabulate_dof_coordinates(cell, coord2)
-        L11.element().tabulate_dof_coordinates(cell, coord3)
+        coord0 = V.element().tabulate_dof_coordinates(cell)
+        coord1 = L0.element().tabulate_dof_coordinates(cell)
+        coord2 = L01.element().tabulate_dof_coordinates(cell)
+        coord3 = L11.element().tabulate_dof_coordinates(cell)
         coord4 = L1.element().tabulate_dof_coordinates(cell)
 
         assert (coord0 == coord1).all()
         assert (coord0 == coord2).all()
         assert (coord0 == coord3).all()
-        assert (coord4[:3] == coord0).all()
-        assert (coord4[3:] == coord0).all()
+        assert (coord4[:sdim] == coord0).all()
+        assert (coord4[sdim:] == coord0).all()
 
 
-def test_tabulate_dofs_periodic():
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (5, 5)), (UnitSquareMesh.create, (5, 5, CellType.Type_quadrilateral))])
+def test_tabulate_dofs_periodic(mesh_factory):
 
     class PeriodicBoundary2(SubDomain):
         def inside(self, x, on_boundary):
@@ -172,7 +178,8 @@ def test_tabulate_dofs_periodic():
             y[0] = x[0] - 1.0
             y[1] = x[1]
 
-    mesh = UnitSquareMesh(5, 5)
+    func, args = mesh_factory
+    mesh = func(*args)
 
     # Create periodic boundary
     periodic_boundary = PeriodicBoundary2()
@@ -215,8 +222,10 @@ def test_tabulate_dofs_periodic():
         assert np.array_equal(np.append(dofs1, dofs2), dofs3)
 
 
-def test_global_dof_builder():
-    mesh = UnitSquareMesh(3, 3)
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (3, 3)), (UnitSquareMesh.create, (3, 3, CellType.Type_quadrilateral))])
+def test_global_dof_builder(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
 
     V = VectorElement("CG", mesh.ufl_cell(), 1)
     Q = FiniteElement("CG", mesh.ufl_cell(), 1)
@@ -228,7 +237,10 @@ def test_global_dof_builder():
     W = FunctionSpace(mesh, R*V)
 
 
-def test_dof_to_vertex_map(mesh, reorder_dofs):
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (3, 3)), (UnitSquareMesh.create, (3, 3, CellType.Type_quadrilateral))])
+def test_dof_to_vertex_map(mesh_factory, reorder_dofs):
+    func, args = mesh_factory
+    mesh = func(*args)
 
     def _test_maps_consistency(space):
         v2d = vertex_to_dof_map(space)
@@ -255,8 +267,11 @@ def test_dof_to_vertex_map(mesh, reorder_dofs):
     u.interpolate(e)
 
     vert_values = mesh.coordinates().sum(1)
-    func_values = np.empty(len(vert_values))
-    u.vector().get_local(func_values, vertex_to_dof_map(V))
+    if has_pybind11():
+        func_values = u.vector().get_local(vertex_to_dof_map(V))
+    else:
+        func_values = np.empty(len(vert_values))
+        u.vector().get_local(func_values, vertex_to_dof_map(V))
     assert round(max(abs(func_values - vert_values)), 7) == 0
 
     c0 = Constant((1, 2))
@@ -333,6 +348,37 @@ def test_entity_dofs(mesh):
         assert all(d == cd for d, cd in zip(dofs, cdofs))
 
 
+@skip_in_parallel
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (2, 2)), (UnitSquareMesh.create, (2, 2, CellType.Type_quadrilateral))])
+def test_entity_closure_dofs(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
+    tdim = mesh.topology().dim()
+
+    for degree in (1, 2, 3):
+        V = FunctionSpace(mesh, "CG", degree)
+        for d in range(tdim + 1):
+            covered = set()
+            covered2 = set()
+            all_entities = np.array([entity for entity in range(mesh.num_entities(d))], dtype=np.uintp)
+            for entity in all_entities:
+                entities = np.array([entity], dtype=np.uintp)
+                dofs_on_this_entity = V.dofmap().entity_dofs(mesh, d, entities)
+                closure_dofs = V.dofmap().entity_closure_dofs(mesh, d, entities)
+                assert len(dofs_on_this_entity) == V.dofmap().num_entity_dofs(d)
+                assert len(dofs_on_this_entity) <= len(closure_dofs)
+                covered.update(dofs_on_this_entity)
+                covered2.update(closure_dofs)
+            dofs_on_all_entities = V.dofmap().entity_dofs(mesh, d, all_entities)
+            closure_dofs_on_all_entities = V.dofmap().entity_closure_dofs(mesh, d, all_entities)
+            assert len(dofs_on_all_entities) == V.dofmap().num_entity_dofs(d) * mesh.num_entities(d)
+            assert covered == set(dofs_on_all_entities)
+            assert covered2 == set(closure_dofs_on_all_entities)
+        d = tdim
+        all_cells = np.array([entity for entity in range(mesh.num_entities(d))], dtype=np.uintp)
+        assert set(V.dofmap().entity_closure_dofs(mesh, d, all_cells)) == set(range(V.dim()))
+
+
 def test_clear_sub_map_data_scalar(mesh):
     V = FunctionSpace(mesh, "CG", 2)
     with pytest.raises(ValueError):
@@ -368,7 +414,9 @@ def test_clear_sub_map_data_vector(mesh):
 
 
 def test_block_size(mesh):
-    meshes = [UnitSquareMesh(8, 8), UnitCubeMesh(4, 4, 4)]
+    meshes = [UnitSquareMesh(8, 8), UnitCubeMesh(4, 4, 4),
+              UnitSquareMesh.create(8, 8, CellType.Type_quadrilateral),
+              UnitCubeMesh.create(4, 4, 4, CellType.Type_hexahedron)]
     for mesh in meshes:
         P2 = FiniteElement("Lagrange", mesh.ufl_cell(), 2)
 
@@ -395,7 +443,14 @@ def test_block_size_real(mesh):
 
 
 @skip_in_serial
-def test_mpi_dofmap_stats(mesh):
+@pytest.mark.parametrize('mesh_factory', [(UnitIntervalMesh, (8,)),
+                                          (UnitSquareMesh, (4, 4)),
+                                          (UnitCubeMesh, (2, 2, 2)),
+                                          (UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral)),
+                                          (UnitCubeMesh.create, (2, 2, 2, CellType.Type_hexahedron))])
+def test_mpi_dofmap_stats(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
 
     V = FunctionSpace(mesh, "CG", 1)
     assert len(V.dofmap().shared_nodes()) > 0
@@ -407,8 +462,19 @@ def test_mpi_dofmap_stats(mesh):
     for owner in V.dofmap().off_process_owner():
         assert owner in neighbours
 
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (4, 4)), (UnitSquareMesh.create, (4, 4, CellType.Type_quadrilateral))])
+def test_local_dimension(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
+
+    v = FiniteElement("Lagrange", mesh.ufl_cell(), 1)
+    q = VectorElement("Lagrange", mesh.ufl_cell(), 1)
+    w = v*q
+
+    V = FunctionSpace(mesh, v)
+    Q = FunctionSpace(mesh, q)
+    W = FunctionSpace(mesh, w)
 
-def test_local_dimension(V, Q, W):
     for space in [V, Q, W]:
         dofmap = space.dofmap()
         local_to_global_map = dofmap.tabulate_local_to_global_dofs()
@@ -423,32 +489,82 @@ def test_local_dimension(V, Q, W):
         #    dofmap.index_map().size('foo')
 
 
+# Failures in FFC on quads/hexes
+xfail_ffc = pytest.mark.xfail(raises=Exception, strict=True)
+
 @skip_in_parallel
-def test_dofs_dim(mesh, V, Q, W):
+@pytest.mark.parametrize('space', [
+    "FunctionSpace(UnitIntervalMesh.create(10),                                        'P', 1)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'P', 1)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'P', 1)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral),           'Q', 1)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),             'Q', 1)",
+    "FunctionSpace(UnitIntervalMesh.create(10),                                        'P', 2)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'P', 2)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'P', 2)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral),           'Q', 2)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),             'Q', 2)",
+    "FunctionSpace(UnitIntervalMesh.create(10),                                        'P', 3)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'P', 3)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'P', 3)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral),           'Q', 3)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),             'Q', 3)",
+    "FunctionSpace(UnitIntervalMesh.create(10),                                        'DP', 1)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'DP', 1)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'DP', 1)",
+    xfail_ffc("FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral), 'DQ', 1)"),
+    xfail_ffc("FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),   'DQ', 1)"),
+    "FunctionSpace(UnitIntervalMesh.create(10),                                        'DP', 2)",
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'DP', 2)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'DP', 2)",
+    xfail_ffc("FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral), 'DQ', 2)"),
+    xfail_ffc("FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),   'DQ', 2)"),
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'N1curl', 1)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'N1curl', 1)",
+    xfail_ffc("FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral), 'N1curl', 1)"),
+    xfail_ffc("FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),   'N1curl', 1)"),
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'N1curl', 2)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'N1curl', 2)",
+    xfail_ffc("FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral), 'N1curl', 2)"),
+    xfail_ffc("FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),   'N1curl', 2)"),
+    "FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_triangle),                'RT', 1)",
+    "FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_tetrahedron),            'RT', 1)",
+    xfail_ffc("FunctionSpace(UnitSquareMesh.create(6, 6, CellType.Type_quadrilateral), 'RT', 1)"),
+    xfail_ffc("FunctionSpace(UnitCubeMesh.create(2, 2, 2, CellType.Type_hexahedron),   'RT', 1)"),
+])
+def test_dofs_dim(space):
     """Test function GenericDofMap::dofs(mesh, dim)"""
-    meshes = [UnitIntervalMesh(10),
-              UnitSquareMesh(6, 6),
-              UnitCubeMesh(2, 2, 2)]
+    V = eval(space)
+    dofmap = V.dofmap()
+    mesh = V.mesh()
+    for dim in range(0, mesh.topology().dim()):
+        edofs = dofmap.dofs(mesh, dim)
+        num_mesh_entities = mesh.num_entities(dim)
+        dofs_per_entity = dofmap.num_entity_dofs(dim)
+        assert len(edofs) == dofs_per_entity*num_mesh_entities
 
-    for mesh in meshes:
-        tdim = mesh.topology().dim()
-        spaces = [FunctionSpace(mesh, "Discontinuous Lagrange", 1),
-                  FunctionSpace(mesh, "Discontinuous Lagrange", 2),
-                  FunctionSpace(mesh, "Lagrange", 1),
-                  FunctionSpace(mesh, "Lagrange", 2),
-                  FunctionSpace(mesh, "Lagrange", 3)]
 
-        if tdim > 1:
-            N1 = "Nedelec 1st kind H(curl)"
-            vspaces = [VectorFunctionSpace(mesh, N1, 1),
-                       VectorFunctionSpace(mesh, N1, 2),
-                       VectorFunctionSpace(mesh, "RT", 1)]
-            spaces = spaces + vspaces
+@skip_if_not_pybind11
+def test_readonly_view_local_to_global_unwoned(mesh):
+    """Test that local_to_global_unwoned() returns readonly
+    view into the data; in particular test lifetime of data
+    owner"""
+    V = FunctionSpace(mesh, "P", 1)
+    dofmap = V.dofmap()
+    index_map = dofmap.index_map()
 
-        for V in spaces:
-            dofmap = V.dofmap()
-            for dim in range(0, mesh.topology().dim()):
-                edofs = dofmap.dofs(mesh, dim)
-                num_mesh_entities = mesh.num_entities(dim)
-                dofs_per_entity = dofmap.num_entity_dofs(dim)
-                assert len(edofs) == dofs_per_entity*num_mesh_entities
+    rc = sys.getrefcount(dofmap)
+    l2gu = dofmap.local_to_global_unowned()
+    assert sys.getrefcount(dofmap) == rc + 1 if l2gu.size else rc
+    assert not l2gu.flags.writeable
+    assert all(l2gu < V.dofmap().global_dimension())
+    del l2gu
+    assert sys.getrefcount(dofmap) == rc
+
+    rc = sys.getrefcount(index_map)
+    l2gu = index_map.local_to_global_unowned()
+    assert sys.getrefcount(index_map) == rc + 1 if l2gu.size else rc
+    assert not l2gu.flags.writeable
+    assert all(l2gu < V.dofmap().global_dimension())
+    del l2gu
+    assert sys.getrefcount(index_map) == rc
