@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for dP assembly"""
 
 # Copyright (C) 2014 Johan Hake
@@ -65,7 +63,7 @@ def _create_dp_problem(dim):
     # Subdomains
     subdomain = AutoSubDomain(lambda x, on_boundary: x[0] <= 0.5)
     disjoint_subdomain = AutoSubDomain(lambda x, on_boundary: x[0] > 0.5)
-    vertex_domain = VertexFunction("size_t", mesh, 0)
+    vertex_domain = MeshFunction("size_t", mesh, 0, 0)
     subdomain.mark(vertex_domain, 1)
     bc = DirichletBC(VV, Constant((0, 0)), disjoint_subdomain)
     dPP = dP(subdomain_data=vertex_domain)
@@ -73,7 +71,6 @@ def _create_dp_problem(dim):
     return (u, uu), (v, vv), (U, UU), dPP, bc
 
 
-@use_gc_barrier
 def test_scalar_assemble(dim):
     eps = 1000*DOLFIN_EPS
 
@@ -90,25 +87,23 @@ def test_scalar_assemble(dim):
     assert abs(scalar_value-uu.vector().sum()) < eps
 
 
-@use_gc_barrier
 def test_vector_assemble(dim):
     eps = 1000*DOLFIN_EPS
 
     (u, uu), (v, vv), (U, UU), dPP, bc = _create_dp_problem(dim)
 
-    # In parallel vec.array() will return only local to process values
+    # In parallel vec.get_local() will return only local to process values
     vec = assemble(u*v*dPP)
-    assert sum(np.absolute(vec.array() - u.vector().array())) < eps
+    assert sum(np.absolute(vec.get_local() - u.vector().get_local())) < eps
 
     vec = assemble(inner(uu, vv)*dP)
-    assert sum(np.absolute(vec.array() - uu.vector().array())) < eps
+    assert sum(np.absolute(vec.get_local() - uu.vector().get_local())) < eps
 
     vec = assemble(inner(uu, vv)*dPP(1))
     bc.apply(uu.vector())
-    assert sum(np.absolute(vec.array() - uu.vector().array())) < eps
+    assert sum(np.absolute(vec.get_local() - uu.vector().get_local())) < eps
 
 
-@use_gc_barrier
 def test_matrix_assemble(dim):
     eps = 1000*DOLFIN_EPS
 
@@ -122,7 +117,7 @@ def test_matrix_assemble(dim):
     loc_range = u.vector().local_range()
     vec_mat = np.zeros_like(mat.array())
     vec_mat[range(loc_range[1] - loc_range[0]),
-            range(loc_range[0], loc_range[1])] = u.vector().array()
+            range(loc_range[0], loc_range[1])] = u.vector().get_local()
 
     assert np.sum(np.absolute(mat.array() - vec_mat)) < eps
 
@@ -134,6 +129,6 @@ def test_matrix_assemble(dim):
     loc_range = uu.vector().local_range()
     vec_mat = np.zeros_like(mat.array())
     vec_mat[range(loc_range[1] - loc_range[0]),
-            range(loc_range[0], loc_range[1])] = uu.vector().array()
+            range(loc_range[0], loc_range[1])] = uu.vector().get_local()
 
     assert np.sum(np.absolute(mat.array() - vec_mat)) < eps
