@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the fem interface"""
 
 # Copyright (C) 2011-2014 Johan Hake
@@ -112,7 +110,7 @@ def test_assemble_linear(V1, Q1, square_boundary, V2, Q2, cube_boundary):
     w = TestFunction(Q1)
     u.vector()[:] = 0.5
     facetareas = MPI.sum(square_boundary.mpi_comm(),
-                         assemble(u*w*dx).array().sum())
+                         assemble(u*w*dx).get_local().sum())
     assert round(facetareas - 2.0, 7) == 0
 
     u = Function(V2)
@@ -121,7 +119,7 @@ def test_assemble_linear(V1, Q1, square_boundary, V2, Q2, cube_boundary):
     a = u*w*dx
     b = assemble(a)
     facetareas = MPI.sum(cube_boundary.mpi_comm(),
-                         assemble(u*w*dx).array().sum())
+                         assemble(u*w*dx).get_local().sum())
     assert round(facetareas - 3.0, 7) == 0
 
     mesh = UnitSquareMesh(8, 8)
@@ -159,7 +157,7 @@ def test_assemble_bilinear_1D_2D(square, V1, square_boundary):
 
     # Assemble over subset of mesh facets
     subdomain = CompiledSubDomain("near(x[1], 0.0)")
-    bottom = FacetFunctionSizet(square)
+    bottom = MeshFunction("size_t", square, square.topology().dim()-1)
     bottom.set_all(0)
     subdomain.mark(bottom, 1)
     dss = ds(subdomain_data=bottom)
@@ -169,7 +167,7 @@ def test_assemble_bilinear_1D_2D(square, V1, square_boundary):
 
     # Assemble over all cells of submesh created from subset of
     # boundary mesh
-    bottom2 = CellFunctionSizet(square_boundary)
+    bottom2 = MeshFunction("size_t", square_boundary, square_boundary.topology().dim())
     bottom2.set_all(0)
     subdomain.mark(bottom2, 1)
     BV = FunctionSpace(SubMesh(square_boundary, bottom2, 1), "CG", 1)
@@ -201,7 +199,7 @@ def test_assemble_bilinear_2D_3D(cube, V2, cube_boundary):
 
     # Assemble over subset of mesh facets
     subdomain = CompiledSubDomain("near(x[1], 0.0)")
-    bottom = FacetFunctionSizet(cube)
+    bottom = MeshFunction("size_t", cube, cube.topology().dim()-1)
     bottom.set_all(0)
     subdomain.mark(bottom, 1)
     dss = ds(subdomain_data=bottom)
@@ -211,7 +209,7 @@ def test_assemble_bilinear_2D_3D(cube, V2, cube_boundary):
 
     # Assemble over all cells of submesh created from subset of
     # boundary mesh
-    bottom2 = CellFunctionSizet(cube_boundary)
+    bottom2 = MeshFunction("size_t", cube_boundary, cube_boundary.topology().dim())
     bottom2.set_all(0)
     subdomain.mark(bottom2, 1)
     BV = FunctionSpace(SubMesh(cube_boundary, bottom2, 1), "CG", 1)
