@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the solve function on manifolds
 embedded in higher dimensional spaces."""
 
@@ -235,25 +233,45 @@ def basis_test(family, degree, basemesh, rotmesh, rotation, piola=False):
 
         for i in range(f_base.element().space_dimension()):
             for point in points:
-                f_base.element().evaluate_basis(i, values_base,
-                                                point,
-                                                vertex_coordinates_base,
-                                                cell_base.orientation())
 
-                f_base.element().evaluate_basis_derivatives(i, 1, derivs_base,
-                                                            point,
-                                                            vertex_coordinates_base,
-                                                            cell_base.orientation())
+                if has_pybind11():
+                    values_base = f_base.element().evaluate_basis(i,
+                                          point, vertex_coordinates_base,
+                                          cell_base.orientation())
 
-                f_rot.element().evaluate_basis(i, values_rot,
-                                                rotation.rotate_point(point),
-                                                vertex_coordinates_rot,
-                                                cell_rot.orientation())
+                    derivs_base = f_base.element().evaluate_basis_derivatives(i, 1,
+                                          point, vertex_coordinates_base,
+                                          cell_base.orientation())
 
-                f_base.element().evaluate_basis_derivatives(i, 1, derivs_rot,
-                                                            rotation.rotate_point(point),
-                                                            vertex_coordinates_rot,
-                                                            cell_rot.orientation())
+                    values_rot = f_rot.element().evaluate_basis(i,
+                                          rotation.rotate_point(point),
+                                          vertex_coordinates_rot,
+                                          cell_rot.orientation())
+
+                    derivs_rot = f_base.element().evaluate_basis_derivatives(i, 1,
+                                          rotation.rotate_point(point),
+                                          vertex_coordinates_rot,
+                                          cell_rot.orientation())
+                else:
+                    f_base.element().evaluate_basis(i, values_base,
+                                                    point,
+                                                    vertex_coordinates_base,
+                                                    cell_base.orientation())
+
+                    f_base.element().evaluate_basis_derivatives(i, 1, derivs_base,
+                                                                point,
+                                                                vertex_coordinates_base,
+                                                                cell_base.orientation())
+
+                    f_rot.element().evaluate_basis(i, values_rot,
+                                                   rotation.rotate_point(point),
+                                                   vertex_coordinates_rot,
+                                                   cell_rot.orientation())
+
+                    f_base.element().evaluate_basis_derivatives(i, 1, derivs_rot,
+                                                                rotation.rotate_point(point),
+                                                                vertex_coordinates_rot,
+                                                                cell_rot.orientation())
 
                 if piola:
                     values_cmp = rotation.rotate_point(values_base)
@@ -276,7 +294,6 @@ def basis_test(family, degree, basemesh, rotmesh, rotation, piola=False):
     parameters["form_compiler"]["no-evaluate_basis_derivatives"] = basis_derivatives
 
 
-@use_gc_barrier
 def test_elliptic_eqn_on_intersecting_surface(datadir):
     """Solves -grad^2 u + u = f on domain of two intersecting square
      surfaces embedded in 3D with natural bcs. Test passes if at end
@@ -296,7 +313,7 @@ def test_elliptic_eqn_on_intersecting_surface(datadir):
     u = TrialFunction(V)
     v = TestFunction(V)
 
-    class Source(Expression):
+    class Source(UserExpression):
         def eval(self, value, x):
             # r0 should be less than 0.5 * sqrt(2) in order for source to be
             # exactly zero on vertical part of domain
