@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the solve function"""
 
 # Copyright (C) 2011 Anders Logg
@@ -24,7 +22,6 @@ from dolfin import *
 from dolfin_utils.test import *
 
 
-@use_gc_barrier
 def test_bcs():
     "Check that the bcs argument is picked up"
 
@@ -62,7 +59,6 @@ def test_bcs():
     assert round(u4.vector().norm("l2") - 14.9362601686, 10) == 0
 
 
-@use_gc_barrier
 def test_bcs_space():
     "Check that the bc space is checked to be a subspace of trial space"
     mesh = UnitSquareMesh(4, 4)
@@ -86,7 +82,6 @@ def test_bcs_space():
         solve(action(a, u) - L == 0, u, bc)
 
 
-@use_gc_barrier
 def test_calling():
     "Test that unappropriate arguments are not allowed"
     mesh = UnitSquareMesh(4, 4)
@@ -111,3 +106,20 @@ def test_calling():
         solve(A, x, b, **kwargs)
 
     # FIXME: Include more tests for this versatile function
+
+
+def test_nonlinear_variational_solver_custom_comm():
+    "Check that nonlinear variational solver works on subset of comm_world"
+    if MPI.rank(mpi_comm_world()) == 0:
+        mesh = UnitIntervalMesh(mpi_comm_self(), 2)
+        V = FunctionSpace(mesh, "CG", 1)
+        f = Constant(1)
+        u = Function(V)
+        v = TestFunction(V)
+        F = inner(u, v)*dx - inner(f, v)*dx
+
+        # Check that following does not deadlock
+        solve(F == 0, u)
+        solve(F == 0, u, solver_parameters={"nonlinear_solver": "newton"})
+        if has_petsc():
+            solve(F == 0, u, solver_parameters={"nonlinear_solver": "snes"})
