@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for class SystemAssembler"""
 
 # Copyright (C) 2011-2013 Garth N. Wells, 2013 Jan Blechta
@@ -131,7 +129,7 @@ def test_facet_assembly():
 
         # Define normal component, mesh size and right-hand side
         n = FacetNormal(mesh)
-        h = CellSize(mesh)
+        h = 2*Circumradius(mesh)
         h_avg = (h('+') + h('-'))/2
         f = Expression("500.0*exp(-(pow(x[0] - 0.5, 2) + pow(x[1] - 0.5, 2)) / 0.02)", degree=1)
 
@@ -197,7 +195,7 @@ def test_vertex_assembly():
         return 0.25 <= x[0] and x[0] <= 0.75 and near(x[1], 0.5)
 
     # Define domain for point integral
-    center_domain = VertexFunction("size_t", mesh, 0)
+    center_domain = MeshFunction("size_t", mesh, 0, 0)
     center = AutoSubDomain(center_func)
     center.mark(center_domain, 1)
     dPP = dP(subdomain_data=center_domain)
@@ -213,12 +211,15 @@ def test_vertex_assembly():
         A, b = assemble_system(a, L)
 
 
-def test_incremental_assembly():
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (20, 20)),
+                                          (UnitSquareMesh.create, (20, 20, CellType.Type_quadrilateral))])
+def test_incremental_assembly(mesh_factory):
 
     for f in [Constant(0.0), Constant(1e4)]:
 
         # Laplace/Poisson problem
-        mesh = UnitSquareMesh(20, 20)
+        func, args = mesh_factory
+        mesh = func(*args)
         V = FunctionSpace(mesh, 'CG', 1)
         u, v = TrialFunction(V), TestFunction(V)
         a, L = inner(grad(u), grad(v))*dx, f*v*dx
@@ -252,13 +253,19 @@ def test_incremental_assembly():
 
 
 @skip_in_parallel
-def test_domains():
+@pytest.mark.parametrize('mesh_factory', [
+    (UnitSquareMesh.create, (24, 24, CellType.Type_triangle)),
+    # FFC PR #91 disables (broken) DQ elements
+    pytest.mark.xfail((UnitSquareMesh.create, (24, 24, CellType.Type_quadrilateral)), strict=True, raises=Exception),
+])
+def test_domains(mesh_factory):
 
     class RightSubDomain(SubDomain):
         def inside(self, x, on_boundary):
             return x[0] > 0.5
 
-    mesh = UnitSquareMesh(24, 24)
+    func, args = mesh_factory
+    mesh = func(*args)
 
     sub_domains = MeshFunction("size_t", mesh, mesh.topology().dim())
     sub_domains.set_all(1)
@@ -367,8 +374,11 @@ def test_facet_assembly_cellwise_insertion(filedir):
     run_test(Mesh(os.path.join(filedir, "gmsh_unit_interval.xml")))
 
 
-def test_non_square_assembly():
-    mesh = UnitSquareMesh(14, 14)
+@pytest.mark.parametrize('mesh_factory', [(UnitSquareMesh, (24, 24)),
+                                          (UnitSquareMesh.create, (24, 24, CellType.Type_quadrilateral))])
+def test_non_square_assembly(mesh_factory):
+    func, args = mesh_factory
+    mesh = func(*args)
 
     def bound(x):
         return (x[0] == 0)
@@ -434,3 +444,39 @@ def test_non_square_assembly():
     Anorm2 = A.norm("frobenius")**2
     assert round(1.0 - bnorm1/bnorm2, 10) == 0
     assert round(1.0 - Anorm1/Anorm2, 10) == 0
+
+
+def test_ghost_mode_handling(pushpop_parameters):
+    def _forms():
+        # Return forms with interior facet integral
+        mesh = UnitSquareMesh(10, 10)
+        V = FunctionSpace(mesh, "P", 1)
+        u, v = TrialFunction(V), TestFunction(V)
+        a, L = u('+')*v('+')*dS, v('+')*dS
+        return a, L
+
+    def _check_value(forms):
+        A, b = assemble_system(*forms)
+        # Test by vector of ones; gives length of interior facets
+        x = Vector()
+        A.init_vector(x, 1)
+        x[:] = 1.0
+        assert numpy.isclose(b.inner(x), 18.0+10*2.0**0.5)
+        assert numpy.isclose((A*x).inner(x), 18.0+10*2.0**0.5)
+
+    # Not-ghosted mesh won't work in parallel and assembler should raise
+    parameters["ghost_mode"] = "none"
+    if MPI.size(mpi_comm_world()) == 1:
+        _check_value(_forms())
+    else:
+        assembler = SystemAssembler(*_forms())
+        A, b = Matrix(), Vector()
+        with pytest.raises(RuntimeError) as excinfo:
+            assembler.assemble(A, b)
+        assert "Incorrect mesh ghost mode" in repr(excinfo.value)
+
+    # Ghosted meshes work everytime
+    parameters["ghost_mode"] = "shared_vertex"
+    _check_value(_forms())
+    parameters["ghost_mode"] = "shared_facet"
+    _check_value(_forms())
