@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the function library"""
 
 # Copyright (C) 2007 Anders Logg
@@ -25,7 +23,7 @@
 import pytest
 from numpy import array
 from dolfin import *
-
+from ufl import UFLException
 
 def test_name_argument():
     u = Constant(1.0)
@@ -40,17 +38,23 @@ def testConstantInit():
     c1 = Constant([2, 3], interval)
     c2 = Constant([[2, 3], [3, 4]], triangle)
     c3 = Constant(array([2, 3]), tetrahedron)
+    c4 = Constant([[2, 3], [3, 4]], quadrilateral)
+    c5 = Constant(array([2, 3]), hexahedron)
 
     # FIXME:
     assert c0.cell() is None
     assert c1.cell() == interval
     assert c2.cell() == triangle
     assert c3.cell() == tetrahedron
+    assert c4.cell() == quadrilateral
+    assert c5.cell() == hexahedron
 
     assert c0.ufl_shape == ()
     assert c1.ufl_shape == (2,)
     assert c2.ufl_shape == (2, 2)
     assert c3.ufl_shape == (2,)
+    assert c4.ufl_shape == (2, 2)
+    assert c5.ufl_shape == (2,)
 
 
 def testGrad():
@@ -66,10 +70,12 @@ def testGrad():
     assert zero == gradient(c3)
 
 
-def test_compute_vertex_values():
+@pytest.mark.parametrize('mesh_factory', [(UnitCubeMesh, (8, 8, 8)), (UnitCubeMesh.create, (8, 8, 8, CellType.Type_hexahedron))])
+def test_compute_vertex_values(mesh_factory):
     from numpy import zeros, all, array
 
-    mesh = UnitCubeMesh(8, 8, 8)
+    func, args = mesh_factory
+    mesh = func(*args)
 
     e0 = Constant(1)
     e1 = Constant((1, 2, 3))
@@ -110,3 +116,15 @@ def test_str():
     c1 = Constant((1., 2., 3.))
     c1.str(False)
     c1.str(True)
+
+
+def test_assign():
+    c0 = Constant(1.)
+    assert c0.values() == (1,)
+    c0.assign(Constant(3))
+    assert c0.values() == (3,)
+
+    c1 = Constant([1, 2])
+    assert (c1.values() == (1, 2)).all()
+    c1.assign(Constant([3, 4]))
+    assert (c1.values() == (3, 4)).all()
