@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the function library"""
 
 # Copyright (C) 2007-2014 Anders Logg
@@ -25,9 +23,9 @@ import pytest
 from dolfin import *
 from math import sin, cos, exp, tan
 from numpy import array, zeros, float_
+import numpy as np
 
-from dolfin_utils.test import fixture, skip_in_parallel
-
+from dolfin_utils.test import fixture, skip_in_parallel, skip_if_pybind11, skip_if_not_pybind11
 
 @fixture
 def mesh():
@@ -45,7 +43,7 @@ def W(mesh):
 
 
 def test_arbitrary_eval(mesh):
-    class F0(Expression):
+    class F0(UserExpression):
         def eval(self, values, x):
             values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
@@ -113,15 +111,15 @@ def test_arbitrary_eval(mesh):
     assert round(u3 - u5, 4) == 0
 
     if has_petsc():
-        PETScOptions.set("mat_mumps_icntl_14", 20)
+        PETScOptions.clear("mat_mumps_icntl_14")
 
 
 def test_ufl_eval():
-    class F0(Expression):
+    class F0(UserExpression):
         def eval(self, values, x):
             values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
-    class V0(Expression):
+    class V0(UserExpression):
         def eval(self, values, x):
             values[0] = x[0]**2
             values[1] = x[1]**2
@@ -157,12 +155,14 @@ def test_ufl_eval():
 
 
 def test_overload_and_call_back(V, mesh):
-    class F0(Expression):
+    class F0(UserExpression):
         def eval(self, values, x):
             values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
-    class F1(Expression):
+    class F1(UserExpression):
         def __init__(self, mesh, *arg, **kwargs):
+            if has_pybind11():
+                super().__init__(*arg, **kwargs)
             self.mesh = mesh
 
         def eval_cell(self, values, x, cell):
@@ -173,9 +173,13 @@ def test_overload_and_call_back(V, mesh):
     e1 = F1(mesh, degree=2)
     e2 = Expression("sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])", degree=2)
 
-    s0 = norm(interpolate(e0, V))
-    s1 = norm(interpolate(e1, V))
-    s2 = norm(interpolate(e2, V))
+    u0 = interpolate(e0, V)
+    u1 = interpolate(e1, V)
+    u2 = interpolate(e2, V)
+
+    s0 = norm(u0)
+    s1 = norm(u1)
+    s2 = norm(u2)
 
     ref = 0.36557637568519191
     assert round(s0 - ref, 7) == 0
@@ -185,7 +189,7 @@ def test_overload_and_call_back(V, mesh):
 
 def test_wrong_eval():
     # Test wrong evaluation
-    class F0(Expression):
+    class F0(UserExpression):
         def eval(self, values, x):
             values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
 
@@ -209,8 +213,81 @@ def test_wrong_eval():
             f(zeros(4), values=zeros(3))
 
 
+def test_vector_valued_expression_member_function(mesh):
+    V = FunctionSpace(mesh,'CG',1)
+    W = VectorFunctionSpace(mesh,'CG',1, dim=3)
+    fs = [
+        Expression(("1", "2", "3"), degree=1),
+        Constant((1, 2, 3)),
+        interpolate(Constant((1, 2, 3)), W),
+    ]
+    for f in fs:
+        u = Expression("f[0] + f[1] + f[2]", f=f, degree=1)
+        v = interpolate(u, V)
+        assert np.allclose(v.vector().get_local(), 6.0)
+        for g in fs:
+            u.f = g
+            v = interpolate(u, V)
+            assert np.allclose(v.vector().get_local(), 6.0)
+
+
+# NOTE: Do we want this to work (attaching MeshFunctions to
+# Expressions) with pybind11, or use full JIT?
+@skip_in_parallel
+@skip_if_pybind11
+def test_meshfunction_expression():
+    mesh = UnitSquareMesh(1, 1)
+    V = FunctionSpace(mesh, "DG", 0)
+
+    c = MeshFunction("size_t", mesh, mesh.topology().dim())
+    c[0] = 2
+    c[1] = 3
+    e = Expression("(double)c", c=c, degree=0)
+    e.c = c
+
+    h = interpolate(e, V)
+    v = h.vector()
+    assert v[0] == float(c[0])
+    assert v[1] == float(c[1])
+
+    a = MeshFunctionDouble(mesh, 2)
+    a[0] = 2.0
+    a[1] = 4.0
+    e = Expression("a", a=a, degree=0)
+    e.a = a
+
+    h = interpolate(e, V)
+    v = h.vector()
+    assert v[0] == float(a[0])
+    assert v[1] == float(a[1])
+
+    f = Function(V)
+    f.vector()[:] = 2.0
+    g = Constant(3.0)
+    e = Expression("a*(c == 2 ? f: g)", a=1.0, c=c, f=f, g=g, degree=0)
+    e.a = 5.0
+    e.c = c
+    e.f = f
+    e.g = g
+
+    h = interpolate(e, V)
+    v = h.vector()
+    assert v[0] == 5.0 * 2.0
+    assert v[1] == 5.0 * 3.0
+
+    w = Constant((0.0, 1.0, 2.0, 3.0, 4.0, 5.0))
+    e = Expression("w[c]", w=w, c=c, degree=0)
+    e.w = w
+    e.c = c
+
+    h = interpolate(e, V)
+    v = h.vector()
+    assert v[0] == float(c[0])
+    assert v[1] == float(c[1])
+
+
 def test_no_write_to_const_array():
-    class F1(Expression):
+    class F1(UserExpression):
         def eval(self, values, x):
             x[0] = 1.0
             values[0] = sin(3.0*x[0])*sin(3.0*x[1])*sin(3.0*x[2])
@@ -235,36 +312,36 @@ def test_compute_vertex_values(mesh):
     assert all(e1_values[mesh.num_vertices():mesh.num_vertices()*2] == 2)
     assert all(e1_values[mesh.num_vertices()*2:mesh.num_vertices()*3] == 3)
 
-
+@skip_if_pybind11
 def test_wrong_sub_classing():
 
     def noAttributes():
-        class NoAttributes(Expression):
+        class NoAttributes(UserExpression):
             pass
 
     def wrongEvalAttribute():
-        class WrongEvalAttribute(Expression):
+        class WrongEvalAttribute(UserExpression):
             def eval(values, x):
                 pass
 
     def wrongEvalDataAttribute():
-        class WrongEvalDataAttribute(Expression):
+        class WrongEvalDataAttribute(UserExpression):
             def eval_cell(values, data):
                 pass
 
     def noEvalAttribute():
-        class NoEvalAttribute(Expression):
+        class NoEvalAttribute(UserExpression):
             def evaluate(self, values, data):
                 pass
 
     def wrongArgs():
-        class WrongArgs(Expression):
+        class WrongArgs(UserExpression):
             def eval(self, values, x):
                 pass
         e = WrongArgs(V)
 
     def deprecationWarning():
-        class Deprecated(Expression):
+        class Deprecated(UserExpression):
             def eval(self, values, x):
                 pass
 
@@ -280,7 +357,7 @@ def test_wrong_sub_classing():
         Expression("a", a="1", degree=1)
 
     def wrongParameterNames0():
-        Expression("long", str=1.0, degree=1)
+        Expression("foo", bar=1.0, degree=1)
 
     def wrongParameterNames1():
         Expression("user_parameters", user_parameters=1.0, degree=1)
@@ -306,13 +383,50 @@ def test_wrong_sub_classing():
     with pytest.raises(RuntimeError):
         wrongParameterNames1()
 
+@skip_if_not_pybind11
+def test_runtime_exceptions():
+
+    def noDefaultValues():
+        Expression("a")
+
+    def wrongDefaultType():
+        Expression("a", a="1", degree=1)
+
+    def wrongParameterNames0():
+        Expression("foo", bar=1.0, degree=1)
+
+    def wrongParameterNames1():
+        Expression("user_parameters", user_parameters=1.0, degree=1)
+
+    with pytest.raises(RuntimeError):
+        noDefaultValues()
+    with pytest.raises(RuntimeError):
+        wrongDefaultType()
+    with pytest.raises(RuntimeError):
+        wrongParameterNames0()
+    with pytest.raises(RuntimeError):
+        wrongParameterNames1()
+
+
+def test_fail_expression_compilation():
+    # Compilation failure only happens on one process,
+    # and involves a barrier to let the compilation finish
+    # before the other processes loads from disk.
+    # This tests that a failure can be caught without deadlock.
+
+    def invalidCppExpression():
+        Expression("/", degree=0)
+
+    with pytest.raises(RuntimeError):
+        invalidCppExpression()
+
 
 def test_element_instantiation():
-    class F0(Expression):
+    class F0(UserExpression):
         def eval(self, values, x):
             values[0] = 1.0
 
-    class F1(Expression):
+    class F1(UserExpression):
         def eval(self, values, x):
             values[0] = 1.0
             values[1] = 1.0
@@ -320,7 +434,7 @@ def test_element_instantiation():
         def value_shape(self):
             return (2,)
 
-    class F2(Expression):
+    class F2(UserExpression):
         def eval(self, values, x):
             values[0] = 1.0
             values[1] = 1.0
@@ -389,6 +503,18 @@ def test_name_space_usage(mesh):
     assert round(assemble(e0*dx(mesh)) - assemble(e1*dx(mesh)), 7) == 0
 
 
+@skip_if_pybind11(reason="CNR What is this for?")
+def test_expression_self_assignment(mesh, V):
+    tc = Constant(2.0)
+    te = Expression("value", value=tc, degree=0)
+    e2 = Expression("t", t=te, degree=0)
+
+    # Test self assignment
+    e2.t = e2
+    with pytest.raises(RuntimeError):
+        e2(0, 0)
+
+
 def test_generic_function_attributes(mesh, V):
     tc = Constant(2.0)
     te = Expression("value", value=tc, degree=0)
@@ -432,20 +558,15 @@ def test_generic_function_attributes(mesh, V):
     W = FunctionSpace(mesh, V.ufl_element()*V.ufl_element())
 
     # Test wrong kwargs
-    with pytest.raises(TypeError):
-        Expression("t", t=Constant((1, 0)), degree=0)
-    with pytest.raises(TypeError):
-        Expression("t", t=Function(W), degree=0)
+    with pytest.raises(Exception):
+        Expression("t", t=mesh, degree=0)
+    with pytest.raises(Exception):
+        Expression("t", t=W, degree=0)
 
     # Test non-scalar GenericFunction
     f2 = Function(W)
     e2.t = f2
 
-    with pytest.raises(RuntimeError):
-        e2(0, 0)
-
-    # Test self assignment
-    e2.t = e2
     with pytest.raises(RuntimeError):
         e2(0, 0)
 
@@ -461,7 +582,7 @@ def test_generic_function_attributes(mesh, V):
     assert te(0.0) == 4.0
 
     # Test wrong assignment
-    with pytest.raises(TypeError):
+    with pytest.raises(Exception):
         te.user_parameters.__setitem__("value", 1.0)
     with pytest.raises(KeyError):
         te.user_parameters.__setitem__("values", 1.0)
@@ -499,6 +620,7 @@ def test_doc_string_eval():
 
 
 @skip_in_parallel
+@skip_if_pybind11
 def test_doc_string_complex_compiled_expression(mesh):
     """
     This test tests all features documented in the doc string of
@@ -539,7 +661,7 @@ def test_doc_string_complex_compiled_expression(mesh):
       }
     };'''
 
-    cell_data = CellFunction('size_t', mesh)
+    cell_data = MeshFunction('size_t', mesh, mesh.topology().dim())
     cell_data.set_all(3)
     CompiledSubDomain("x[0] <= 0.25").mark(cell_data, 0)
     CompiledSubDomain("x[0] > 0.25 && x[0] < 0.75").mark(cell_data, 1)
@@ -591,6 +713,7 @@ def test_doc_string_complex_compiled_expression(mesh):
 
 @pytest.mark.slow
 @skip_in_parallel
+@skip_if_pybind11
 def test_doc_string_compiled_expression_with_system_headers():
     """
     This test tests all features documented in the doc string of
@@ -685,7 +808,7 @@ def test_doc_string_python_expressions(mesh):
 
     square = UnitSquareMesh(4, 4)
 
-    class MyExpression0(Expression):
+    class MyExpression0(UserExpression):
         def eval(self, value, x):
             dx = x[0] - 0.5
             dy = x[1] - 0.5
@@ -706,7 +829,7 @@ def test_doc_string_python_expressions(mesh):
     ufc_cell_attrs = ["cell_shape", "index", "topological_dimension",
                       "geometric_dimension", "local_facet", "mesh_identifier"]
 
-    class MyExpression1(Expression):
+    class MyExpression1(UserExpression):
         def eval_cell(self_expr, value, x, ufc_cell):
             # Check attributes in ufc cell
             for attr in ufc_cell_attrs:
@@ -720,15 +843,17 @@ def test_doc_string_python_expressions(mesh):
     f1 = MyExpression1(degree=0)
     assemble(f1*ds(square))
 
-    class MyExpression2(Expression):
+    class MyExpression2(UserExpression):
         def __init__(self, mesh, domain, *arg, **kwargs):
+            if has_pybind11():
+                super().__init__(*arg, **kwargs)
             self._mesh = mesh
             self._domain = domain
 
         def eval(self, values, x):
             pass
 
-    cell_data = CellFunction('size_t', square)
+    cell_data = MeshFunction('size_t', square, square.topology().dim())
 
     P1 = FiniteElement("Lagrange", square.ufl_cell(), 1)
     f3 = MyExpression2(square, cell_data, element=P1)
