@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the Attribute interface of the HDF5 io library"""
 
 # Copyright (C) 2013 Chris Richardson
@@ -21,9 +19,11 @@
 
 import pytest
 import os
+import gc
 from dolfin import *
 import numpy
-from dolfin_utils.test import skip_if_not_HDF5, fixture, tempdir
+from dolfin_utils.test import skip_if_not_HDF5, fixture, tempdir, \
+    xfail_with_serial_hdf5_in_parallel, skip_with_serial_hdf5_in_parallel
 
 
 @pytest.yield_fixture
@@ -35,33 +35,42 @@ def attr(tempdir):
 
     yield attr
 
-    del hdf_file
+    # Destroy everything deterministically
+    hdf_file.close()
+    del hdf_file, x, attr
+    gc.collect()
+    MPI.barrier(mpi_comm_world())
 
 @skip_if_not_HDF5
+@xfail_with_serial_hdf5_in_parallel
 def test_fail_on_accessing_attribute_on_non_existing_dataset(tempdir):
     hdf_file = HDF5File(mpi_comm_world(), os.path.join(tempdir, "hdf_file.h5"), "w")
     with pytest.raises(RuntimeError):
         attr = hdf_file.attributes("/a_vector")
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_read_write_str_attribute(attr):
     attr['name'] = 'Vector'
     assert attr.type_str("name") == "string"
     assert attr['name'] == 'Vector'
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_read_write_float_attribute(attr):
     attr['val'] = -9.2554
     assert attr.type_str("val") == "float"
     assert attr['val'] == -9.2554
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_read_write_int_attribute(attr):
     attr['val'] = 1
     assert attr.type_str("val") == "int"
     assert attr['val'] == 1
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_read_write_vec_float_attribute(attr):
     vec = numpy.array([1,2,3,4.5], dtype='float')
     attr['val'] = vec
@@ -72,6 +81,7 @@ def test_read_write_vec_float_attribute(attr):
         assert val1 == val2
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_read_write_vec_int_attribute(attr):
     vec = numpy.array([1,2,3,4,5], dtype=numpy.uintp)
     attr['val'] = vec
@@ -82,30 +92,40 @@ def test_read_write_vec_int_attribute(attr):
         assert val1 == val2
 
 @skip_if_not_HDF5
+@skip_with_serial_hdf5_in_parallel
 def test_attribute_container_interface(attr):
     names = ["data_0", "data_1", "data_2", "data_3"]
     values = [i for i in range(4)]
-    
+
     for name, value in zip(names, values):
         attr[name] = value
-        
+
     # Added from the vector storage call above
     if "partition" in attr:
         names.append("partition")
         values.append(attr["partition"])
-    
-    assert(attr.list_attributes()==names)
-    assert(attr.to_dict() == dict(zip(names, values)))
 
-    for name, value in attr.items():
-        assert(name in names)
-        assert(value in values)
-        assert(names.index(name)==values.index(value))
+    assert(attr.list_attributes() == names)
+    for (name0, value0), (name1, value1) in zip(attr.to_dict().items(),
+                                                dict(zip(names, values)).items()):
+        assert name0 == name1
+        if isinstance(value0, numpy.ndarray):
+            assert (value0 == value1).all()
+        else:
+            assert value0 == value1
 
-    for name in attr:
-        assert(name in names)
+    # GNW: Should really just use h5py rather than wrap this. It could
+    # be interfaced with pybind11, but not worth it
+    if not has_pybind11():
+        for name, value in attr.items():
+            assert(name in names)
+            assert(value in values)
+            assert(names.index(name)==values.index(value))
 
-    for value in attr.values():
-        assert(value in values)
+        for name in attr:
+            assert(name in names)
 
-    assert(attr.keys()==attr.list_attributes())
+        for value in attr.values():
+            assert(value in values)
+
+        assert(attr.keys()==attr.list_attributes())
