@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the linear algebra interface"""
 
 # Copyright (C) 2008 Johan Hake
@@ -109,7 +107,10 @@ class TestBasicLaOperations:
         for T in [int,int16,int32,int64,uint,uint0,uint16,uint32,uint64,\
                   int0,integer_types[-1]]:
             v[T(lind)] = 2.0
-            assert round(sum(v[T(lind)] - 2.0), 7) == 0
+            if not has_pybind11():
+                assert round(sum(v[T(lind)] - 2.0), 7) == 0
+            else:
+                assert round(v[T(lind)] - 2.0, 7) == 0
 
         A = v.copy()
         B = as_backend_type(v.copy())
@@ -118,48 +119,83 @@ class TestBasicLaOperations:
 
         # Test global index access
         if A.owns_index(gind):
-            assert round(sum(A[lind] - B[lind]), 7) == 0
+            if not has_pybind11():
+                assert round(sum(A[lind] - B[lind]), 7) == 0
+            else:
+                assert round(A[lind] - B[lind], 7) == 0
 
         lind0 = 5
-        round(sum(A[lind0] - B[lind0]), 7) == 0
+        if not has_pybind11():
+            round(sum(A[lind0] - B[lind0]), 7) == 0
+        else:
+            round(A[lind0] - B[lind0], 7) == 0
 
         B *= 0.5
         A *= 2
-        assert round(sum(A[lind0] - 4*B[lind0]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - 4*B[lind0]), 7) == 0
+        else:
+            assert round(A[lind0] - 4*B[lind0], 7) == 0
 
         B /= 2
         A /= 0.5
-        assert round(sum(A[lind0] - 16*B[lind0]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - 16*B[lind0]), 7) == 0
+        else:
+            assert round(A[lind0] - 16*B[lind0], 7) == 0
 
         val1 = A[lind0]
         val2 = B[lind0]
 
         A += B
-        assert round(sum(A[lind0] - val1-val2), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - val1 - val2), 7) == 0
+        else:
+            assert round(A[lind0] - val1 - val2, 7) == 0
 
         A -= B
-        assert round(sum(A[lind0] - val1), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - val1), 7) == 0
+        else:
+            assert round(A[lind0] - val1, 7) == 0
 
         C = 16*B
-        assert round(sum(A[lind0] - C[lind0]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - C[lind0]), 7) == 0
+        else:
+            assert round(A[lind0] - C[lind0], 7) == 0
 
         D = (C + B)*5
-        assert round(sum(D[lind0] - (val1 + val2)*5), 7) == 0
+        if not has_pybind11():
+            assert round(sum(D[lind0] - (val1 + val2)*5), 7) == 0
+        else:
+            assert round(D[lind0] - (val1 + val2)*5, 7) == 0
+
 
         F = (A-B)/4
-        assert round(sum(F[lind0] - (val1 - val2)/4), 7) == 0
+        if not has_pybind11():
+            assert round(sum(F[lind0] - (val1 - val2)/4), 7) == 0
+        else:
+            assert round(F[lind0] - (val1 - val2)/4, 7) == 0
 
         A.axpy(100, B)
-        assert round(sum(A[lind0] - val1 - val2*100), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A[lind0] - val1 - val2*100), 7) == 0
+        else:
+            assert round(A[lind0] - val1 - val2*100, 7) == 0
 
-        A2 = A.array()
+        A2 = A.get_local()
         assert isinstance(A2,ndarray)
         assert A2.shape == (n1 - n0, )
 
-        assert round(sum(A2[lind0] - A[lind0]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(A2[lind0] - A[lind0]), 7) == 0
+        else:
+            assert round(A2[lind0] - A[lind0], 7) == 0
+
         assert round(MPI.sum(A.mpi_comm(), A2.sum()) - A.sum(), 7) == 0
 
-        B2 = B.array()
+        B2 = B.get_local()
 
         inds = [1,3,6,9,15,20,24,28,32,40,50,60,70,100000]
 
@@ -202,16 +238,24 @@ class TestBasicLaOperations:
         assert (A3==A2).all()
 
         A[:] = A2
-        assert (A.array()==A2).all()
+        assert (A.get_local()==A2).all()
 
         H  = A.copy()
-        H._assign(0.0)
+        if not has_pybind11():
+            H._assign(0.0)
         H[linds0] = G
 
         C[:] = 2
-        D._assign(2)
-        assert round(sum(C[0] - 2), 7) == 0
-        assert round(sum(C[len(linds0)-1] - 2), 7) == 0
+        if not has_pybind11():
+            D._assign(2)
+        else:
+            D[:] = 2
+        if not has_pybind11():
+            assert round(sum(C[0] - 2), 7) == 0
+            assert round(sum(C[len(linds0)-1] - 2), 7) == 0
+        else:
+            assert round(C[0] - 2, 7) == 0
+            assert round(C[len(linds0)-1] - 2, 7) == 0
         assert round(C.sum() - D.sum(), 7) == 0
 
         C[linds0] = 3
@@ -243,7 +287,7 @@ class TestBasicLaOperations:
         with pytest.raises(IndexError):
             wrong_dim([0,2], slice(0,4,1))
 
-        A2 = A.array()
+        A2 = A.get_local()
         A *= B
         A2 *= B2
         I = A*B
@@ -255,8 +299,11 @@ class TestBasicLaOperations:
         def wrong_assign(A, ind):
             A[linds0[::2]] = linds0[::2]
 
-        with pytest.raises(TypeError):
-            wrong_assign(A, linds2)
+        # GNW: I don't see what's wrong assigning a list than can be
+        # converted to a NumPy array
+        if not has_pybind11():
+            with pytest.raises(TypeError):
+                wrong_assign(A, linds2)
 
 
     def test_matrix_vector(self, any_backend, use_backend):
@@ -267,7 +314,10 @@ class TestBasicLaOperations:
 
         a, b = get_forms(mesh)
         if use_backend:
-            backend = getattr(cpp, self.backend+self.sub_backend+'Factory').instance()
+            if has_pybind11():
+                backend = getattr(cpp.la, self.backend+self.sub_backend+'Factory').instance()
+            else:
+                backend = getattr(cpp.la, self.backend+self.sub_backend+'Factory').instance()
         else:
             backend = None
 
@@ -299,7 +349,9 @@ class TestBasicLaOperations:
 
         u = A*v
 
-        assert isinstance(u, type(v))
+        if not has_pybind11():
+            assert isinstance(u, type(v))
+
         assert len(u) == len(v)
 
         # Test basic square matrix multiply results
@@ -316,22 +368,28 @@ class TestBasicLaOperations:
 
         # Miscellaneous tests
         u2 = 2*u - A*v
-        assert round(sum(u2[4] - u[4]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(u2[4] - u[4]), 7) == 0
+        else:
+            assert round(u2[4] - u[4], 7) == 0
 
         u3 = 2*u + -1.0*(A*v)
-        assert round(sum(u3[4] - u[4]), 7) == 0
+        if not has_pybind11():
+            assert round(sum(u3[4] - u[4]), 7) == 0
+        else:
+            assert round(u3[4] - u[4], 7) == 0
 
-        # Numpy arrays are not alligned in parallel
+        # Numpy arrays are not aligned in parallel
         if distributed:
             return
 
-        v_numpy = v.array()
+        v_numpy = v.get_local()
         A_numpy = A.array()
 
         u_numpy = dot(A_numpy, v_numpy)
         u_numpy2 = A*v_numpy
 
-        assert absolute(u.array() - u_numpy).sum() < DOLFIN_EPS*len(v)
+        assert absolute(u.get_local() - u_numpy).sum() < DOLFIN_EPS*len(v)
         assert absolute(u_numpy2 - u_numpy).sum() < DOLFIN_EPS*len(v)
 
 
