@@ -1,5 +1,3 @@
-#!/usr/bin/env py.test
-
 """Unit tests for the RKSolver interface"""
 
 # Copyright (C) 2013 Johan Hake
@@ -44,8 +42,6 @@ def convergence_order(errors, base = 2):
 @skip_in_parallel
 def test_butcher_schemes_scalar():
 
-    LEVEL = cpp.get_log_level()
-    cpp.set_log_level(cpp.WARNING)
     mesh = UnitSquareMesh(4, 4)
 
     V = FunctionSpace(mesh, "R", 0)
@@ -66,17 +62,13 @@ def test_butcher_schemes_scalar():
             solver.step_interval(0., tstop, dt)
             u_errors.append(u_true(0.0, 0.0) - u(0.0, 0.0))
 
-        assert scheme.order()-min(convergence_order(u_errors))<0.1
-
-    cpp.set_log_level(LEVEL)
+        assert scheme.order() - min(convergence_order(u_errors)) < 0.1
 
 
 @pytest.mark.slow
 @skip_in_parallel
 def test_butcher_schemes_vector():
 
-    LEVEL = cpp.get_log_level()
-    cpp.set_log_level(cpp.WARNING)
     mesh = UnitSquareMesh(4, 4)
 
     V = VectorFunctionSpace(mesh, "R", 0, dim=2)
@@ -99,7 +91,5 @@ def test_butcher_schemes_vector():
             u_errors_0.append(u_true(0.0, 0.0)[0] - u(0.0, 0.0)[0])
             u_errors_1.append(u_true(0.0, 0.0)[1] - u(0.0, 0.0)[1])
 
-        assert scheme.order()-min(convergence_order(u_errors_0))<0.1
-        assert scheme.order()-min(convergence_order(u_errors_1))<0.1
-
-    cpp.set_log_level(LEVEL)
+        assert scheme.order() - min(convergence_order(u_errors_0)) < 0.1
+        assert scheme.order() - min(convergence_order(u_errors_1)) < 0.1
